@@ -196,7 +196,7 @@ int g_b_set_masters_and_set_leafs(GB_CTREE *node,
     return 1;
 }
 
-static GB_INLINE long g_b_read_number2(unsigned char **s) {
+static GB_INLINE long g_b_read_number2(const unsigned char **s) {
     register unsigned int c0,c1,c2,c3,c4;
     c0 = (*((*s)++));
     if (c0 & 0x80){
@@ -463,52 +463,80 @@ NOT4PERL GB_ERROR GBT_compress_sequence_tree(GBDATA *gb_main, GB_CTREE *tree, co
 
 /* Compress sequences, call only outside a transaction */
 GB_ERROR GBT_compress_sequence_tree2(GBDATA *gb_main, const char *tree_name, const char *ali_name){
-    GB_ERROR error = 0;
-    GB_CTREE *ctree;
-    GB_UNDO_TYPE undo_type =  GB_get_requested_undo_type(gb_main);
-    GB_MAIN_TYPE *Main = GB_MAIN(gb_main);
-    char *to_free = 0;
+    GB_ERROR      error     = 0;
+    GB_CTREE     *ctree;
+    GB_UNDO_TYPE  undo_type = GB_get_requested_undo_type(gb_main);
+    GB_MAIN_TYPE *Main      = GB_MAIN(gb_main);
+    char         *to_free   = 0;
+    
     if (Main->transaction>0){
-        GB_internal_error("Internal Error: Compress Sequences called during a running transacton");
-        return GB_export_error("Internal Error: Compress Sequences called during a running transacton");
+        GB_internal_error("Internal Error: Compress Sequences called during a running transaction");
+        return GB_export_error("Internal Error: Compress Sequences called during a running transaction");
     }
+    
     GB_request_undo_type(gb_main,GB_UNDO_KILL);
     GB_begin_transaction(gb_main);
     GB_push_my_security(gb_main);
 
     if (!tree_name || !strlen(tree_name)) {
-        to_free = (char *)(tree_name = (const char *)GBT_find_largest_tree(gb_main));
+        to_free   = GBT_find_largest_tree(gb_main);
+        tree_name = to_free;
     }
     ctree = (GB_CTREE *)GBT_read_tree(gb_main,(char *)tree_name,-sizeof(GB_CTREE));
-    if (!ctree) return GB_export_error("Tree %s not found in database",tree_name);
-    error = GBT_link_tree((GBT_TREE *)ctree,gb_main,GB_FALSE);
-    if (!error) error = GBT_compress_sequence_tree(gb_main,ctree,ali_name);
-    GBT_delete_tree((GBT_TREE *)ctree);
+    if (!ctree) {
+        error = GB_export_error("Tree %s not found in database",tree_name);
+    }
+    else {
+        error             = GBT_link_tree((GBT_TREE *)ctree, gb_main, GB_FALSE, 0, 0);
+        if (!error) error = GBT_compress_sequence_tree(gb_main,ctree,ali_name);
+        GBT_delete_tree((GBT_TREE *)ctree);
+    }
+    
     GB_pop_my_security(gb_main);
-    if (error){
+    if (error) {
         GB_abort_transaction(gb_main);
-    }else{
+    }
+    else {
         GB_commit_transaction(gb_main);
+        GB_disable_quicksave(gb_main,"Database optimized");
     }
-    GB_disable_quicksave(gb_main,"Database optimized");
     GB_request_undo_type(gb_main,undo_type);
+    
     if (to_free) free(to_free);
     return error;
 }
 
-void GBT_compression_test(void *dummy, GBDATA *gb_main){
-    GB_ERROR error;
-    char *ali_name;
+void GBT_compression_test(void *dummy, GBDATA *gb_main) {
+    GB_ERROR  error     = 0;
+    char     *tree_name = 0;
+    char     *ali_name;
+    GBDATA   *gb_tree_name;
+
     GBUSE(dummy);
+
     GB_begin_transaction(gb_main);
-    ali_name = GBT_get_default_alignment(gb_main);
-    error = GBT_compress_sequence_tree2(gb_main,"tree_test",ali_name);
-    if (error){
-        GB_warning("%s",error);
-        GB_abort_transaction(gb_main);
-    }else{
-        GB_commit_transaction(gb_main);
+
+    ali_name     = GBT_get_default_alignment(gb_main);
+    gb_tree_name = GB_search(gb_main, "/focus/tree_name", GB_FIND);
+    if (!gb_tree_name) {
+        error = "Can't detect current treename";
     }
+    else {
+        tree_name = GB_read_string(gb_tree_name);
+    }
+
+    GB_commit_transaction(gb_main);
+
+    if (!error) {
+        printf("Recompression data in alignment '%s' using tree '%s'\n", ali_name, tree_name);
+        error = GBT_compress_sequence_tree2(gb_main, tree_name, ali_name);
+    }
+    if (error) {
+        GB_warning("%s",error);
+    }
+
+    free(tree_name);
+    free(ali_name);
 }
 
 
@@ -602,47 +630,50 @@ char *g_b_uncompress_single_sequence_by_master(const char *s, const char *master
     return buffer;
 }
 
-char *gb_uncompress_by_sequence(GBDATA *gbd, const char *s,long size, GB_ERROR *error){
-    register char *dest;
-    GB_MAIN_TYPE *Main;
-    GBDATA *gb_main;
-    GBDATA *gb_master;
-    char *master;
-    GBQUARK quark;
-    int index;
-    long master_size;
-    char *to_free;
+char *gb_uncompress_by_sequence(GBDATA *gbd, const char *ss,long size, GB_ERROR *error){
+    char *dest = 0;
 
     *error = 0;
-
     if (!GB_FATHER(gbd)) {
         *error = "Cannot uncompress this sequence: Sequence has no father";
-        return 0;
     }
+    else {
+        GB_MAIN_TYPE *Main    = GB_MAIN(gbd);
+        GBDATA       *gb_main = (GBDATA*)Main->data;
+        char         *to_free = gb_check_out_buffer(ss); /* Remove 'ss' from memory management, otherwise load_single_key_data() may destroy it */
+        int           index;
+        GBQUARK       quark;
 
-    Main = GB_MAIN(gbd);
-    gb_main = (GBDATA *)Main->data;
+        {
+            const unsigned char *s = (const unsigned char *)ss;
 
-    to_free = gb_check_out_buffer(s); /* Get our own buffer, maybe load_single_key_data will destroy it */
+            index = g_b_read_number2(&s);
+            quark = g_b_read_number2(&s);
 
-    index = g_b_read_number2((unsigned char **)&s);
-    quark = g_b_read_number2((unsigned char **)&s);
+            ss = (const char *)s;
+        }
 
-    if (!Main->keys[quark].gb_master_ali){
-        gb_load_single_key_data(gb_main,quark);
+        if (!Main->keys[quark].gb_master_ali){
+            gb_load_single_key_data(gb_main,quark);
+        }
+
+        if (!Main->keys[quark].gb_master_ali){
+            *error = "Cannot uncompress this sequence: Cannot find a master sequence";
+        }
+        else {
+            GBDATA *gb_master = gb_find_by_nr(Main->keys[quark].gb_master_ali,index);
+            if (!gb_master){
+                dest = (char*)GB_get_error();
+            }
+            else {
+                long        master_size = GB_read_string_count(gb_master); // @@@ why unused ?
+                const char *master      = GB_read_char_pntr(gb_master); /* make sure that this is not a buffer !!! */
+
+                dest = g_b_uncompress_single_sequence_by_master(ss, master, size);
+            }
+        }
+        free(to_free);
     }
-    if (!Main->keys[quark].gb_master_ali){
-        *error = "Cannot uncompress this sequence: Cannot find a master sequence";
-        return 0;
-    }
-
-    gb_master = gb_find_by_nr(Main->keys[quark].gb_master_ali,index);
-    if (!gb_master){
-        return (char *)GB_get_error();
-    }
-    master_size = GB_read_string_count(gb_master);
-    master = GB_read_char_pntr(gb_master); /* make sure that this is not a buffer !!! */
-    dest = g_b_uncompress_single_sequence_by_master(s,master,size);
-    if (to_free) free(to_free);
-    return (char *)dest;
+    
+    return dest;
 }
