@@ -151,8 +151,10 @@ long gbcm_read_buffered(int socket,char *ptr, long size)
     if (holding <= 0) {
         holding = read(socket,gb_local->write_buffer,(size_t)gb_local->write_bufsize);
 
-        if (!holding){
-            fprintf(stderr,"Cannot read data: len %li: errno %i\n",holding,errno);
+        if (holding < 0)
+        {
+            fprintf(stderr,"Cannot read data from client: len=%li (%s, errno %i)\n",
+                    holding, strerror(errno), errno);
             return 0;
         }
         gbcm_read_flush(socket);
@@ -225,9 +227,8 @@ gbcm_write_flush(int socket)
     }
     return GBCM_SERVER_OK;
 }
-int
-gbcm_write(int socket,const char *ptr,long size)
-{
+
+int gbcm_write(int socket,const char *ptr,long size) {
 
     while  (size >= gb_local->write_free){
         GB_MEMCPY(gb_local->write_ptr, ptr, (int)gb_local->write_free);
@@ -255,25 +256,24 @@ void *gbcm_sigio()
 
 GB_ERROR gbcm_get_m_id(const char *path, char **m_name, long *id)
 {
-    char           *p;
-    char           *mn;
-    char    *to_free = NULL;
-    long             i;
+    const char *p;
+    char       *mn;
+    long        i;
+
     if (!path) {
         return "OPEN_ARB_DB_CLIENT ERROR: missing hostname:socketid";
     }
-    if (!strcmp(path,":")) {
-        to_free = (char *)(path = GBS_read_arb_tcp("ARB_DB_SERVER"));
+    if (strcmp(path,":") == 0) {
+        path = GBS_read_arb_tcp("ARB_DB_SERVER");
         if (!path) return GB_get_error();
     }
-    p = (char *) strchr(path, ':');
+    p = strchr(path, ':');
     if (path[0] == '*' || path[0] == ':'){  /* UNIX MODE */
         if (!p) {
             return GB_export_error("OPEN_ARB_DB_CLIENT ERROR: missing ':' in %s",path);
         }
         *m_name = GB_STRDUP(p+1);
         *id = -1;
-        if (to_free) free(to_free);
         return 0;
     }
     if (!p) {
@@ -287,7 +287,6 @@ GB_ERROR gbcm_get_m_id(const char *path, char **m_name, long *id)
         return GB_export_error("OPEN_ARB_DB_CLIENT ERROR: socketnumber %li not in [1024..4095]",i);
     }
     *id = i;
-    if (to_free) free(to_free);
     return 0;
 }
 
@@ -414,7 +413,7 @@ struct gbcmc_comm *gbcmc_open(const char *path)
         if (link->unix_name) free(link->unix_name); /* @@@ */
         free((char *)link);
         if(*err){
-            GB_internal_error("ARB_DB_CLIENT_OPEN");
+            GB_internal_error(GBS_global_string("ARB_DB_CLIENT_OPEN\n(Reason: %s)", err));
         }
         return 0;
     }
@@ -435,7 +434,7 @@ long gbcm_write_two(int socket, long a, long c)
     ia[1] = 3;
     ia[2] = c;
     if (!socket) return 1;
-    return  gbcm_write(socket,(char *)(&(ia[0])),sizeof(long)*3);
+    return  gbcm_write(socket, (const char *)ia, sizeof(long)*3);
 }
 
 
@@ -449,19 +448,19 @@ long gbcm_read_two(int socket, long a, long *b, long *c)
     long    size;
     size = gbcm_read(socket,(char *)&(ia[0]),sizeof(long)*3);
     if (size != sizeof(long) * 3) {
-        GB_internal_error("recieve failed: %i bytes expected, %li got, keyword %lX",
+        GB_internal_error("receive failed: %i bytes expected, %li got, keyword %lX",
                           sizeof(long) * 3,size, a );
         return GBCM_SERVER_FAULT;
     }
     if (ia[0] != a) {
-        GB_internal_error("recieved keyword failed %lx != %lx\n",ia[0],a);
+        GB_internal_error("received keyword failed %lx != %lx\n",ia[0],a);
         return GBCM_SERVER_FAULT;
     }
     if (b) {
         *b = ia[1];
     }else{
         if (ia[1]!=3) {
-            GB_internal_error("recieve failed: size not 3\n");
+            GB_internal_error("receive failed: size not 3\n");
             return GBCM_SERVER_FAULT;
         }
     }
@@ -471,16 +470,13 @@ long gbcm_read_two(int socket, long a, long *b, long *c)
 
 long gbcm_write_string(int socket, const char *key)
 {
-    long x[1];
-    if (key){
-        x[0] = strlen(key);
-        gbcm_write(socket,(char *)x,sizeof(long));
-        if (x[0]) {
-            gbcm_write(socket,key,x[0]);
-        }
-    }else{
-        x[0] = -1;
-        gbcm_write(socket,(char *)x,sizeof(long));
+    if (key) {
+        size_t len = strlen(key);
+        gbcm_write_long(socket, len);
+        if (len) gbcm_write(socket,key,len);
+    }
+    else {
+        gbcm_write_long(socket, -1);
     }
     return GBCM_SERVER_OK;
 }
@@ -488,21 +484,34 @@ long gbcm_write_string(int socket, const char *key)
 char *gbcm_read_string(int socket)
 {
     char *key;
-    long strlen[1];
-    gbcm_read(socket,(char *)strlen,sizeof(long));
-    if (strlen[0]>=0) {
-        if (strlen[0]) {
-            key = (char *)GB_calloc(sizeof(char),(size_t)strlen[0]+1);
-            gbcm_read(socket,key,strlen[0]);
-        }else{
-            key = GB_strdup("");
+    long  strlen = gbcm_read_long(socket);
+
+    if (strlen) {
+        if (strlen>0) {
+            key = (char *)GB_calloc(sizeof(char), (size_t)strlen+1);
+            gbcm_read(socket, key, strlen);
+        }
+        else {
+            key = 0;
         }
-    }else{
-        key = 0;
     }
+    else {
+        key = GB_strdup("");
+    }
+
     return key;
 }
 
+long gbcm_write_long(int socket, long data) {
+    gbcm_write(socket, (char*)&data, sizeof(data));
+    return GBCM_SERVER_OK;
+}
+
+long gbcm_read_long(int socket) {
+    long data;
+    gbcm_read(socket, (char*)&data, sizeof(data));
+    return data;
+}
 
 
 struct stat gb_global_stt;
@@ -746,19 +755,6 @@ GB_ULONG GB_last_saved_time(GBDATA *gb_main){
     return GB_MAIN(gb_main)->last_saved_time;
 }
 
-void GB_edit(const char *path){
-    const char *editor  = GB_getenvARB_TEXTEDIT();
-    char       *fpath   = GBS_eval_env(path);
-    char       *command = GBS_global_string_copy("%s %s &", editor, fpath);
-
-    GB_information("Executing '%s'", command);
-    if (system(command) != 0) {
-        GB_warning("Could not start editor command '%s'", command);
-    }
-    free(command);
-    free(fpath);
-}
-
 void GB_textprint(const char *path){
     char buffer[1024];
     char *fpath = GBS_eval_env(path);
@@ -1072,28 +1068,37 @@ GB_CSTR GB_getenv(const char *env){
 
 int GB_host_is_local(const char *hostname){
     /* returns 1 if host is local */
-    if (!strcmp(hostname,"localhost")) return 1;
-    if (!strcmp(hostname,GBC_get_hostname())) return 1;
+    if (strcmp(hostname,"localhost")        == 0) return 1;
+    if (strcmp(hostname,GBC_get_hostname()) == 0) return 1;
+    if (strstr(hostname, "127.0.0.")        == hostname) return 1;
     return 0;
 }
+
+#define MIN(a,b) (((a)<(b))?(a):(b))
+
 /* Returns the physical memory size in k available for one process */
 GB_ULONG GB_get_physical_memory(void){
 #if defined(SUN5) || defined(LINUX)
-    long pagesize     = sysconf(_SC_PAGESIZE);
-    long pages        = sysconf(_SC_PHYS_PAGES);
+    long pagesize = sysconf(_SC_PAGESIZE);
+    long pages    = sysconf(_SC_PHYS_PAGES);
     /*    long test = sysconf(_SC_AVPHYS_PAGES); */
-    long memsize      = (pagesize/1024) * pages;
-    long nettomemsize = memsize - 10000; /* kernel size */
 
-    long maxmemsize4arb = (nettomemsize*95)/100; /* arb uses max. 95 % of memory (was 70% in the past) */
+    long memsize        = (pagesize/1024) * pages;
+    long nettomemsize   = memsize - 10240; /* reduce by 10Mb (for kernel etc.) */
+    long maxmemsize4arb = (nettomemsize*95)/100; /* arb uses max. 95 % of available memory (was 70% in the past) */
+    long addressable    = (long)1<<((long)sizeof(void*)*(long)8-(long)10); /* e.g. 2^32 with 4byte-pointers; -10 because we calculate in kBytes */
+    long usedmemsize    = MIN(maxmemsize4arb, addressable); /* limit to max addressable memory */
 
 #if defined(DEBUG)
-    printf("- memsize(real)   =%li\n", memsize);
-    printf("- memsize(netto)  =%li\n", nettomemsize);
-    printf("- memsize(for ARB)=%li\n", maxmemsize4arb);
+    printf("- memsize(real)        = %20li k\n", memsize);
+    printf("- memsize(netto)       = %20li k\n", nettomemsize);
+    printf("- memsize(95%%)         = %20li k\n", maxmemsize4arb);
+    printf("- memsize(addressable) = %20li k\n", addressable);
+    printf("- memsize(used by ARB) = %20li k\n", usedmemsize);
 #endif /* DEBUG */
-
-    return maxmemsize4arb;
+    
+    arb_assert(usedmemsize != 0);
+    return usedmemsize;
 #else
     return 128*1024;            /* 128 Mb default memory */
 #endif
