@@ -279,16 +279,67 @@ void GB_clear_error() {         /* clears the error buffer */
 
 /* -------------------------------------------------------------------------------- */
 
+#if defined(DEBUG)
+#if defined(DEVEL_RALF)
+/* #define TRACE_BUFFER_USAGE */
+#endif /* DEBUG */
+#endif /* DEVEL_RALF */
+
+
 #define GLOBAL_STRING_BUFFERS 4
 
 static GB_CSTR gbs_vglobal_string(const char *templat, va_list parg, int allow_reuse)
 {
-    static char buffer[GLOBAL_STRING_BUFFERS][GBS_GLOBAL_STRING_SIZE+2]; // two buffers - used alternately
-    static int  idx    = 0;
-    int         my_idx = (idx+1)%GLOBAL_STRING_BUFFERS; // use next buffer
-    int         psize;
+    static char buffer[GLOBAL_STRING_BUFFERS][GBS_GLOBAL_STRING_SIZE+2]; // serveral buffers - used alternately
+    static int  idx                             = 0;
+    static char lifetime[GLOBAL_STRING_BUFFERS] = { };
+    static char nextIdx[GLOBAL_STRING_BUFFERS] = { };
+    
+    int my_idx;
+    int psize;
 
-    ad_assert(my_idx >= 0 && my_idx<GLOBAL_STRING_BUFFERS);
+    if (nextIdx[0] == 0) { // initialize nextIdx
+        for (my_idx = 0; my_idx<GLOBAL_STRING_BUFFERS; my_idx++) {
+            nextIdx[my_idx] = (my_idx+1)%GLOBAL_STRING_BUFFERS;
+        }
+    }
+
+    if (allow_reuse == -1) { /* called from GBS_reuse_buffer */
+        /* buffer to reuse is passed in 'templat' */
+
+        for (my_idx = 0; my_idx<GLOBAL_STRING_BUFFERS; my_idx++) {
+            if (buffer[my_idx] == templat) {
+                lifetime[my_idx] = 0;
+#if defined(TRACE_BUFFER_USAGE)
+                printf("Reusing buffer #%i\n", my_idx);
+#endif /* TRACE_BUFFER_USAGE */
+                if (nextIdx[my_idx] == idx) idx = my_idx;
+                return 0;
+            }
+#if defined(TRACE_BUFFER_USAGE)
+            else {
+                printf("(buffer to reuse is not buffer #%i (%p))\n", my_idx, buffer[my_idx]);
+            }
+#endif /* TRACE_BUFFER_USAGE */
+        }
+        for (my_idx = 0; my_idx<GLOBAL_STRING_BUFFERS; my_idx++) {
+            printf("buffer[%i]=%p\n", my_idx, buffer[my_idx]);
+        }
+        ad_assert(0);       /* GBS_reuse_buffer called with illegal buffer */
+        return 0;
+    }
+
+    if (lifetime[idx] == 0) {
+        my_idx = idx;
+    }
+    else {
+        for (my_idx = nextIdx[idx]; lifetime[my_idx]>0; my_idx = nextIdx[my_idx]) {
+#if defined(TRACE_BUFFER_USAGE)
+            printf("decreasing lifetime[%i] (%i->%i)\n", my_idx, lifetime[my_idx], lifetime[my_idx]-1);
+#endif /* TRACE_BUFFER_USAGE */
+            lifetime[my_idx]--;
+        }
+    }
 
 #ifdef LINUX
     psize = vsnprintf(buffer[my_idx],GBS_GLOBAL_STRING_SIZE,templat,parg);
@@ -296,16 +347,35 @@ static GB_CSTR gbs_vglobal_string(const char *templat, va_list parg, int allow_r
     psize = vsprintf(buffer[my_idx],templat,parg);
 #endif
 
+#if defined(TRACE_BUFFER_USAGE)
+    printf("Printed into global buffer #%i ('%s')\n", my_idx, buffer[my_idx]);
+#endif /* TRACE_BUFFER_USAGE */
+
     if (psize == -1 || psize >= GBS_GLOBAL_STRING_SIZE) {
-        ad_assert(0);              // buffer overflow (increase GBS_GLOBAL_STRING_SIZE or use your own buffer)
+        fprintf(stderr, "Internal buffer overflow (psize=%i)\n", psize);
+        ad_assert(0);           // buffer overflow (you better use your own buffer)
         GB_CORE;
     }
 
-    if (!allow_reuse) idx = my_idx;
+    if (!allow_reuse) {
+        idx           = my_idx;
+        lifetime[idx] = 1;
+    }
+#if defined(TRACE_BUFFER_USAGE)
+    else {
+        printf("Allow reuse of buffer #%i\n", my_idx);
+    }
+#endif /* TRACE_BUFFER_USAGE */
 
     return buffer[my_idx];
 }
 
+void GBS_reuse_buffer(GB_CSTR global_buffer) {
+    /* If you've just shortely used a buffer, you can put it back here */
+
+    gbs_vglobal_string(global_buffer, 0, -1);
+}
+
 GB_CSTR GBS_global_string(const char *templat, ...) {
     /* goes to header: __attribute__((format(printf, 1, 2)))  */
 
@@ -384,6 +454,22 @@ int gbs_stricmp(const char *s1, const char *s2) {
     return cmp;
 }
 
+int gbs_strnicmp(const char *s1, const char *s2, size_t len) {
+    int c1, c2, cmp = 0;
+
+    if (len>0) {
+        do {
+            --len;
+            c1  = *s1++;
+            c2  = *s2++;
+            cmp = toupper(c1)-toupper(c2);
+        }
+        while (cmp == 0 && c1 != 0 && len>0);
+    }
+
+    return cmp;
+}
+
 void gbs_memcopy(char *dest, const char *source, long len)
 {
     register long    i;
@@ -532,12 +618,6 @@ GB_ERROR GB_check_hkey(const char *key)
 
 /*  return 0; */
 
-
-char *GBS_strdup(const char *s){
-    if (!s) return NULL;
-    return GB_STRDUP(s);
-}
-
 #define UPPERCASE(c) if ( (c>='a') && (c<='z')) c+= 'A'-'a'
 
 /* search a substring in another string
@@ -641,8 +721,8 @@ long GBS_string_cmp(const char *str,const char *search,long upper_case)
      /* *   Wildcard in search string */
      /* ?   any Charakter   */
      /* if uppercase    change all letters to uppercase */
-     /* returns 0 if strings are equal -1 or +1 if left strings is
-        less/greater than right string */
+     /* returns 0 if strings are equal
+        -int/+int if left string is less/greater than right string */
 {
     register const char *p1,*p2;
     register char a,b,*d;
@@ -652,12 +732,13 @@ long GBS_string_cmp(const char *str,const char *search,long upper_case)
     p1 = str;
     p2 = search;
     while (1) {
-        a = *p1;b=*p2;
+        a = *p1;
+        b = *p2;
         if (b == '*')   {
             if (!p2[1]) break; /* -> return 0; */
             i = 0;
             d = fsbuf;
-            for (p2++;(b=*p2)&&(b!='*');){
+            for (p2++; (b=*p2)&&(b!='*'); ) {
                 *(d++) = b;
                 p2++;
                 i++;
@@ -667,21 +748,30 @@ long GBS_string_cmp(const char *str,const char *search,long upper_case)
                 p1 += strlen(p1)-i;     /* check the end of the string */
                 if (p1 < str) return -1;    /* neither less or greater */
                 p2 -= i;
-            }else{
+            }
+            else {
                 *d=0;
                 p1 = GBS_find_string(p1,fsbuf,upper_case+2);
                 if (!p1) return -1;
                 p1 += i;
             }
             continue;
-        }else{
-            if (!a) return b;
-            if (!b) return a;
-            if (b != '?')   {
-                if (upper_case) {
-                    if (toupper(a) !=toupper(b) ) return 1;
-                }else{
-                    if (a!=b) return 1;
+        }
+        else {
+            if (!a) return -b;  /* return 0 or -int */
+            if (a != b) {
+                if (b != '?') {
+                    if (!b) return a; /* return +int */
+                    if (upper_case) {
+                        a = toupper(a);
+                        b = toupper(b);
+                        if (a != b) {
+                            return a-b; /* return -int if a<b, +int otherwise */
+                        }
+                    }
+                    else {
+                        return a-b; /* return -int if a<b, +int otherwise */
+                    }
                 }
             }
         }
@@ -1121,10 +1211,11 @@ GB_ERROR gbs_build_replace_string(void *strstruct,
                                     break;
                                 case '|':
                                     h = GB_command_interpreter(GB_get_root(gb_container), entry,psym+1,gb_container, 0);
-                                    if (h){
+                                    if (h) {
                                         GBS_strcat(strstruct,h);
                                         free(h);
-                                    }else{
+                                    }
+                                    else {
                                         return GB_get_error();
                                     }
                                     break;
@@ -1144,7 +1235,10 @@ GB_ERROR gbs_build_replace_string(void *strstruct,
                         break;
                     }
                     c = '*';
-                }else{
+                    GBS_chrcat(strstruct,c);
+                    GBS_chrcat(strstruct,d);
+                }
+                else {
                     wildcard_num = d - '1';
                     if (c == GBS_WILD) {
                         c = '?';
@@ -1154,10 +1248,12 @@ GB_ERROR gbs_build_replace_string(void *strstruct,
                         }
                         if (wildcard_num>=max_wildcard) {
                             GBS_chrcat(strstruct,c);
-                        }else{
+                        }
+                        else {
                             GBS_chrcat(strstruct,wildcards[wildcard_num]);
                         }
-                    }else{
+                    }
+                    else {
                         c = '*';
                         if ( (wildcard_num<0)||(wildcard_num>9) ) {
                             p--;            /* use this character */
@@ -1169,13 +1265,11 @@ GB_ERROR gbs_build_replace_string(void *strstruct,
                             GBS_strcat(strstruct,mwildcards[wildcard_num]);
                         }
                     }
-                    break;
                 }
-                GBS_chrcat(strstruct,c);
-                GBS_chrcat(strstruct,d);
                 break;
             default:
                 GBS_chrcat(strstruct,c);
+                break;
         }       /* switch c */
     }
     return 0;
@@ -1199,12 +1293,15 @@ char *GBS_string_eval(const char *insource, const char *icommand, GBDATA *gb_con
         the replace is repeated as many times as possible
         '\' is the escape character: e.g. \n is newline; '\\' is '\'; '\=' is '='; ....
 
-    eg:
+        eg:
         print first three characters of first word and the whole second word:
 
-        *(arb_key)  is the value of the a database entry arb key
-
-    */
+        *(arb_key)          is the value of the a database entry arb key
+        *(arb_key#string)   value of the database entry or 'string' if the entry does not exist
+        *(arb_key\:SRT)     runs SRT recursively on the value of the database entry 
+        *([arb_key]|ACI)    runs the ACI command interpreter on the value of the database entry (or on an empty string)
+        
+     */
 
 {
     register char *source;      /* pointer into the current string when parsed */
@@ -1419,129 +1516,14 @@ char *GBS_eval_env(const char *p){
     return p2;
 }
 
-/********************************************************************************************
-            Create an entry representing pt-server 'i'
-            (reads from arb_tcp.dat)
-********************************************************************************************/
-
-char *GBS_ptserver_id_to_choice(int i) {
-    char  search_for[256];
-    char  choice[256];
-    char *fr;
-    char *file;
-    char *server;
-    char empty[] = "";
-
-    sprintf(search_for,"ARB_PT_SERVER%i",i);
-
-    server = GBS_read_arb_tcp(search_for);
-    if (!server) return 0;
-
-    fr               = server;
-    file             = server;  /* i got the machine name of the server */
-    if (*file) file += strlen(file)+1; /* now i got the command string */
-    if (*file) file += strlen(file)+1; /* now i got the file */
-
-    if (strrchr(file,'/'))              file   = strrchr(file,'/')-1;
-    if (!(server = strtok(server,":"))) server = empty;
-
-    sprintf(choice,"%-8s: %s",server,file+2);
-    free(fr);
-
-    return strdup(choice);
-}
-
-
-/********************************************************************************************
-            Find an entry in the $ARBHOME/lib/arb_tcp.dat file
-********************************************************************************************/
-
-char *GBS_read_arb_tcp(const char *env)
-{
-    char       *filename;
-    char       *result = 0;
-
-    if (strchr(env,':')){
-        return GB_STRDUP(env);
-    }
-
-    filename = GBS_find_lib_file("arb_tcp.dat","", 1);
-    if (!filename) {
-        GB_export_error("File $ARBHOME/lib/arb_tcp.dat not found");
-    }
-    else {
-        FILE       *arb_tcp = fopen(filename,"r");
-        int         envlen  = strlen(env);
-        const char *user    = GB_getenvUSER();
-
-        if (!user) {
-            GB_export_error("Enviroment Variable 'USER' not found");
-            GB_print_error();
-        }
-        else {
-            int   envuserlen = strlen(user)+envlen+1;
-            char *envuser    = (char *)GB_calloc(sizeof(char),envuserlen+1);
-            char  buffer[256];
-            char *p;
-            char *nl;
-            char *tok1;
-
-            sprintf(envuser,"%s:%s",user,env);
-
-            for (p = fgets(buffer,255,arb_tcp);
-                 p;
-                 p = fgets(p,255,arb_tcp))
-            {
-                if (!strncmp(envuser,buffer,envuserlen)) {
-                    envlen = envuserlen;
-                    break;
-                }
-                if (!strncmp(env,buffer,envlen)) break;
-            }
-
-            if (p) {
-                if ( (nl = strchr(p,'\n')) ) *nl = 0;
-                p = p+envlen;
-                while ((*p == ' ') || (*p == '\t')) p++;
-                p = GBS_eval_env(p);
-                strncpy(buffer,p,200);
-                free(p);
-                p = buffer;
-
-                tok1 = strtok(p," \t");
-                p = buffer;
-                while (tok1) {
-                    int len = strlen(tok1);
-                    memmove(p, tok1, len+1);
-                    p += len+1;
-                    tok1 = strtok(0," \t");
-                }
-
-                result = (char*)GB_calloc(sizeof(char),p-buffer);
-                GB_MEMCPY(result,buffer,p-buffer);
-            }
-            else {
-                GB_export_error("Missing '%s' in '%s'",env,filename);
-            }
-
-            free(envuser);
-        }
-        fclose(arb_tcp);
-    }
-    free(filename);
-
-    return result;
-}
-
 char* GBS_find_lib_file(const char *filename,const char *libprefix, int warn_when_not_found)
-     /*
- * Searches in $CURRENTDIR $HOME $ARBHOME/lib
- */
 {
-    char            buffer[256];
-    const char           *arbhome;
-    const char           *home;
-    FILE           *in;
+    /* Searches files in $CURRENTDIR, $HOME, $ARBHOME/lib */
+    
+    char        buffer[256];
+    const char *arbhome;
+    const char *home;
+    FILE       *in;
 
     arbhome = GB_getenvARBHOME();
     home = GB_getenvHOME();
@@ -1870,9 +1852,9 @@ void GB_internal_error(const char *templat, ...) {
 
     full_message = GBS_global_string("Internal ARB Error: %s", message);
     gb_error_handler(full_message);
-    gb_error_handler("  ARB may be unstable now (due to this error).\n"
-                     "  Consider saving your database (maybe under a different name),\n"
-                     "  try to fix the error and restart ARB.");
+    gb_error_handler("ARB is most likely unstable now (due to this error).\n"
+                     "If you've made changes to the database, consider to save it using a different name.\n"
+                     "Try to fix the cause of the error and restart ARB.");
 
     if (GBS_do_core()) {
         GB_CORE;
@@ -2054,7 +2036,10 @@ char *GBS_regreplace(const char *in, const char *regexprin, GBDATA *gb_species){
     while (subs && subs > regexpr && subs[-1] == '\\') subs = strrchr(subs,'/');
     if (!subs || subs == regexpr){
         free(regexpr);
-        GB_export_error("no '/' found in regexpr"); /* dont change this error message (or change it in adquery/GB_command_interpreter too) */
+        
+        /* dont change this error message (or change it in adquery/GB_command_interpreter too) : */
+        GB_export_error("no '/' found in regexpr");
+        
         return 0;
     }
     *(subs++) = 0;
