@@ -536,8 +536,7 @@ long GB_read_ints_count(GBDATA *gbd)
 
 GB_UINT4 *GB_read_ints(GBDATA *gbd)
 {
-    GB_CUINT4 *i;
-    i = GB_read_ints_pntr(gbd);
+    GB_CUINT4 *i = GB_read_ints_pntr(gbd);
     if (!i) return 0;
     return  (GB_UINT4 *)gbs_malloc_copy((char *)i,GB_GETSIZE(gbd)*sizeof(GB_UINT4));
 }
@@ -589,13 +588,12 @@ float *GB_read_floats(GBDATA *gbd)
 
 char *GB_read_as_string(GBDATA *gbd)
 {
-    char buffer[256];
     switch (GB_TYPE(gbd)) {
         case GB_STRING: return GB_read_string(gbd);
         case GB_LINK:   return GB_read_link(gbd);
-        case GB_BYTE:   sprintf(buffer,"%i",GB_read_byte(gbd)); return GB_STRDUP(buffer);
-        case GB_INT:    sprintf(buffer,"%li",GB_read_int(gbd)); return GB_STRDUP(buffer);
-        case GB_FLOAT:  sprintf(buffer,"%.4g",GB_read_float(gbd)); return GB_STRDUP(buffer);
+        case GB_BYTE:   return GBS_global_string_copy("%i",GB_read_byte(gbd));
+        case GB_INT:    return GBS_global_string_copy("%li",GB_read_int(gbd));
+        case GB_FLOAT:  return GBS_global_string_copy("%.4g",GB_read_float(gbd));
         case GB_BITS:   return GB_read_bits(gbd,'0','1');
             /* Be careful : When adding new types here, you have to make sure that
              * GB_write_as_string is able to write them back and that this makes sense.
@@ -604,6 +602,42 @@ char *GB_read_as_string(GBDATA *gbd)
     }
 }
 
+/* array type access functions (intended for perl use) */ 
+
+long GB_read_from_ints(GBDATA *gbd, long index) {
+    static GBDATA    *last_gbd = 0;
+    static long       count    = 0;
+    static GB_CUINT4 *i        = 0;
+
+    if (gbd != last_gbd) {
+        count    = GB_read_ints_count(gbd);
+        i        = GB_read_ints_pntr(gbd);
+        last_gbd = gbd;
+    }
+
+    if (index >= 0 && index < count) {
+        return i[index];
+    }
+    return -1;
+}
+
+double GB_read_from_floats(GBDATA *gbd, long index) {
+    static GBDATA    *last_gbd = 0;
+    static long       count    = 0;
+    static GB_CFLOAT *f        = 0;
+
+    if (gbd != last_gbd) {
+        count    = GB_read_floats_count(gbd);
+        f        = GB_read_floats_pntr(gbd);
+        last_gbd = gbd;
+    }
+
+    if (index >= 0 && index < count) {
+        return f[index];
+    }
+    return -1;
+}
+
 
 /********************************************************************************************
                     WRITE DATA
@@ -1227,7 +1261,7 @@ GB_ERROR gb_delete_force(GBDATA *source)    /* delete always */
 ********************************************************************************************/
 GB_ERROR GB_copy(GBDATA *dest, GBDATA *source)
 {
-    long type;
+    GB_TYPES type;
     GB_ERROR error = 0;
     GBDATA *gb_p;
     GBDATA *gb_d;
@@ -1238,8 +1272,8 @@ GB_ERROR GB_copy(GBDATA *dest, GBDATA *source)
     type = GB_TYPE(source);
     if (GB_TYPE(dest) != type)
     {
-        return GB_export_error("incompatible types in GB_copy (source %s:%li != %s:%i",
-                               GB_read_key_pntr(source),type, GB_read_key_pntr(dest), GB_TYPE(dest));
+        return GB_export_error("incompatible types in GB_copy (source %s:%u != %s:%u",
+                               GB_read_key_pntr(source), type, GB_read_key_pntr(dest), GB_TYPE(dest));
     }
 
     switch (type)
@@ -1795,7 +1829,7 @@ GB_CPNTR gb_read_pntr_ts(GBDATA *gbd, struct gb_transaction_save *ts){
 }
 
 /* get last array value in callbacks */
-void *GB_read_old_value(){
+NOT4PERL void *GB_read_old_value(){
     char *data;
 
     if (!g_b_old_callback_list) {
