@@ -172,6 +172,14 @@ void ParseMenu()
             crop(in_line,head,temp);
             thisitem->meta = temp[0];
         }
+        /*
+         *       Sequence type restriction
+         */
+        else if(Find(in_line,"seqtype:"))
+        {
+            crop(in_line,head,temp);
+            thisitem->seqtype = toupper(temp[0]); /* 'A' -> amino acids, 'N' -> nucleotides, otherwise both */
+        }
         /*
          *  arg: defines the symbol for a command line arguement.
          *      this is used for substitution into the itemmethod
