@@ -214,7 +214,7 @@ void awt_delete_species_in_list(void *dummy, struct adaqbsstruct *cbs)
     }
 }
 
-GB_HASH *awt_generate_species_hash(GBDATA *gb_main, char *key,int split)
+GB_HASH *awt_create_species_hash(GBDATA *gb_main, char *key,int split)
 {
     GB_HASH *hash = GBS_create_hash(GBT_get_species_hash_size(gb_main),1);
     GBDATA  *gb_species;
@@ -396,7 +396,7 @@ void awt_do_query(void *dummy, struct adaqbsstruct *cbs,AW_CL ext_query)
 
     if (cbs->gb_ref && ( ext_query == AWT_EXT_QUERY_COMPARE_LINES || ext_query == AWT_EXT_QUERY_COMPARE_WORDS)) {
         GB_push_transaction(cbs->gb_ref);
-        ref_hash = awt_generate_species_hash(cbs->gb_ref,first_key, ext_query == AWT_EXT_QUERY_COMPARE_WORDS);
+        ref_hash = awt_create_species_hash(cbs->gb_ref,first_key, ext_query == AWT_EXT_QUERY_COMPARE_WORDS);
     }
 
     for (GBDATA *gb_item_container = cbs->selector->get_first_item_container(cbs->gb_main, aw_root, range);
@@ -445,7 +445,7 @@ void awt_do_query(void *dummy, struct adaqbsstruct *cbs,AW_CL ext_query)
                                 all_fields = 1;
                             }
                             else {
-                                gb_key = GB_find_sub_by_quark(gb_item,this_query->getKeyquark(),0,0);
+                                gb_key = GB_find_sub_by_quark(gb_item, this_query->getKeyquark(), GB_NONE, 0, 0);
                             }
 
                             while (gb_key) {
@@ -2111,7 +2111,7 @@ struct adaqbsstruct *awt_create_query_box(AW_window *aws, awt_query_struct *awtq
             if (xpos_calc[1] == -1) aws->get_at_position(&xpos_calc[1], &ypos_dummy);
 
             aws->at(xpos_calc[1], ypos+key*KEY_Y_OFFSET);
-            aws->create_toggle(cbs->awar_not[key], "matches.bitmap", "not.bitmap");
+            aws->create_toggle(cbs->awar_not[key], "#equal.xpm", "#notEqual.xpm");
 
             if (xpos_calc[2] == -1) aws->get_at_position(&xpos_calc[2], &ypos_dummy);
         }
@@ -2159,7 +2159,7 @@ struct adaqbsstruct *awt_create_query_box(AW_window *aws, awt_query_struct *awtq
     if (awtqs->count_pos_fig){
         aws->at(awtqs->count_pos_fig);
         aws->label("Hits:");
-        aws->create_button(0,cbs->awar_count);
+        aws->create_button(0,cbs->awar_count,0,"+");
     }
 
     if (awtqs->config_pos_fig){
@@ -2167,12 +2167,12 @@ struct adaqbsstruct *awt_create_query_box(AW_window *aws, awt_query_struct *awtq
         aws->at(awtqs->config_pos_fig);
         AWT_insert_config_manager(aws, AW_ROOT_DEFAULT, "query_box", query_box_store_config, query_box_restore_config, (AW_CL)cbs, 0);
     }
-    aws->button_length(19);
+
+    aws->button_length(18);
     if (awtqs->do_query_pos_fig){
         aws->at(awtqs->do_query_pos_fig);
         aws->callback((AW_CB)awt_do_query,(AW_CL)cbs,AWT_EXT_QUERY_NONE);
-        aws->highlight();
-        aws->create_button("SEARCH", "SEARCH","Q");
+        aws->create_button("SEARCH", "Search");
     }
     if (awtqs->do_refresh_pos_fig && !GB_NOVICE){
         aws->at(awtqs->do_refresh_pos_fig);
@@ -2183,19 +2183,19 @@ struct adaqbsstruct *awt_create_query_box(AW_window *aws, awt_query_struct *awtq
         aws->at(awtqs->do_mark_pos_fig);
         aws->help_text("mark_list.hlp");
         aws->callback((AW_CB)awt_do_mark_list,(AW_CL)cbs,1);
-        aws->create_button("MARK_LISTED_UNMARK_REST", "MARK LISTED\nUNMARK REST","M");
+        aws->create_button("MARK_LISTED_UNMARK_REST", "Mark Listed\nUnmark Rest","M");
     }
     if (awtqs->do_unmark_pos_fig){
         aws->at(awtqs->do_unmark_pos_fig);
         aws->help_text("unmark_list.hlp");
         aws->callback((AW_CB)awt_do_mark_list,(AW_CL)cbs,0);
-        aws->create_button("UNMARK_LISTED_MARK_REST","UNMARK LISTED\nMARK REST","U");
+        aws->create_button("UNMARK_LISTED_MARK_REST","Unmark Listed\nMark Rest","U");
     }
     if (awtqs->do_delete_pos_fig){
         aws->at(awtqs->do_delete_pos_fig);
         aws->help_text("del_list.hlp");
         aws->callback((AW_CB)awt_delete_species_in_list,(AW_CL)cbs,0);
-        aws->create_button("DELETE_LISTED","DELETE LISTED","D");
+        aws->create_button("DELETE_LISTED","Delete Listed","D");
     }
     if (awtqs->do_set_pos_fig && !GB_NOVICE){
         sprintf(buffer,"tmp/arbdb_query_%i/set_key",query_id);
@@ -2213,7 +2213,7 @@ struct adaqbsstruct *awt_create_query_box(AW_window *aws, awt_query_struct *awtq
         aws->at(awtqs->do_set_pos_fig);
         aws->help_text("mod_field_list.hlp");
         aws->callback(AW_POPUP,(AW_CL)create_awt_do_set_list,(AW_CL)cbs);
-        aws->create_button("WRITE_TO_FIELDS_OF_LISTED", "WRITE TO FIELDS\nOF LISTED","S");
+        aws->create_button("WRITE_TO_FIELDS_OF_LISTED", "Write to Fields\nof Listed","S");
     }
 
     char *Items = strdup(cbs->selector->items_name);
