@@ -12,7 +12,7 @@
 #include <awt_tree.hxx>
 #include <awt_dtree.hxx>
 #include <awt_tree_cb.hxx>
-// #include <awt_assert.hxx>
+#include <awt_advice.hxx>
 #include <awt.hxx>
 
 
@@ -119,7 +119,7 @@ static int nt_species_has_alignment(GBDATA *gb_species, void *cd_use) {
 }
 
 static int nt_sequence_is_partial(GBDATA *gb_species, void *cd_partial) {
-    int wanted  = (int)cd_partial;
+    long wanted  = (long)cd_partial;
     awt_assert(wanted == 0 || wanted == 1);
     int partial = GBT_is_partial(gb_species, 1-wanted, 0);
 
@@ -246,10 +246,11 @@ void NT_mark_nontree_cb(AW_window *, AW_CL cl_ntw, AW_CL cl_mark_mode)
     struct mark_nontree_cb_data  cd;
 
     if ((mark_mode&MARK_MODE_LOWER_BITS) == 0) {   // unmark is much faster
-        cd.hash = GBT_generate_marked_species_hash(ntw->gb_main); // because it only hashes marked species
+        cd.hash = GBT_create_marked_species_hash(ntw->gb_main); // because it only hashes marked species
     }
     else {
-        cd.hash = GBT_generate_species_hash(ntw->gb_main, 0); // otherwise we have to hash ALL species
+        cd.hash = GBT_create_species_hash(ntw->gb_main/*, 1*/); // otherwise we have to hash ALL species
+        // Note: changed to ignore case (ralf 2007-07-06)
     }
 
     NT_remove_species_in_tree_from_hash(gtree->tree_root, cd.hash);
@@ -638,6 +639,11 @@ void NT_move_boot_branch(AW_window*, AW_CL cl_ntw, AW_CL cl_direction) // copy b
 
         tree_root->compute_tree(ntw->gb_main);
         AWT_TREE(ntw)->save(ntw->gb_main,0,0,0);
+
+        char *adviceText = GBS_global_string_copy("Please note, that you just overwrote your existing %s.",
+                                                  direction ? "bootstrap values" : "branchlengths");
+        AWT_advice(adviceText, AWT_ADVICE_TOGGLE|AWT_ADVICE_HELP, 0, "tbl_boot2len.hlp");
+        free(adviceText);
     }
     ntw->zoom_reset();
     ntw->refresh();
@@ -780,6 +786,9 @@ void NT_jump_cb_auto(AW_window *dummy, AWT_canvas *ntw){    // jump only if auto
     ntw->refresh();
 }
 
+inline const char *plural(int val) {
+    return "s"+(val == 1);
+}
 
 void NT_reload_tree_event(AW_root *awr, AWT_canvas *ntw, GB_BOOL set_delete_cbs)
 {
@@ -790,6 +799,23 @@ void NT_reload_tree_event(AW_root *awr, AWT_canvas *ntw, GB_BOOL set_delete_cbs)
     if (error) {
         aw_message(error);
     }
+    else {
+        int zombies, duplicates;
+        ((AWT_graphic_tree*)ntw->tree_disp)->get_zombies_and_duplicates(zombies, duplicates);
+
+        if (zombies || duplicates) {
+            const char *msg = 0;
+            if (duplicates) {
+                if (zombies) msg = GBS_global_string("%i zombie%s and %i duplicate%s", zombies, plural(zombies), duplicates, plural(duplicates));
+                else msg         = GBS_global_string("%i duplicate%s", duplicates, plural(duplicates));
+            }
+            else {
+                awt_assert(zombies);
+                msg = GBS_global_string("%i zombie%s", zombies, plural(zombies));
+            }
+            aw_message(GBS_global_string("%s in '%s'", msg, tree_name));
+        }
+    }
     free(tree_name);
     ntw->zoom_reset();
     AWT_expose_cb(0,ntw,0);
