@@ -273,7 +273,7 @@ char *PHMATRIX::load(char *usei,AP_filter *filter,AP_weights *weights,AP_smatrix
     tree_size++;
     GB_CSTR *species_in_sort_tree = 0;
 
-    int no_of_species = all ? GBT_count_species(gb_main) : GBT_count_marked_species(gb_main);
+    int no_of_species = all ? GBT_get_species_count(gb_main) : GBT_count_marked_species(gb_main);
     int in_tree_species = 0;
     int out_tree_species = no_of_species;
     int unknown_species_in_tree = 0;
@@ -426,8 +426,8 @@ long *PHMATRIX::create_helix_filter(BI_helix *helix,AP_filter *filter){
     for (i=0;i<filter->filter_len;i++) {
         if (filter->filter_mask[i]) {
             temp[i] = count;
-            if (helix->entries[i].pair_type== HELIX_PAIR) {
-                result[count] = helix->entries[i].pair_pos;
+            if (helix->pairtype(i)== HELIX_PAIR) {
+                result[count] = helix->opposite_position(i);
             }else{
                 result[count] = -1;
             }
@@ -1294,6 +1294,7 @@ static void ph_autodetect_callback(AW_window *aww)
     free(filter_str);
 }
 
+static void ap_exit(AW_window *) __attribute__((noreturn));
 static void ap_exit(AW_window *)
 {
     if (gb_main) GB_exit(gb_main);
@@ -1445,7 +1446,7 @@ AW_window *create_matrix_window(AW_root *aw_root) {
 
     aws->at("cancel_select");
     aws->callback((AW_CB1)AW_POPUP,(AW_CL)awt_create_select_cancel_window);
-    aws->create_button("SELECT_CANCEL_CHARS", "Select","C");
+    aws->create_button("SELECT_CANCEL_CHARS", "Info","C");
 
     aws->at("change_matrix");
     aws->callback(AW_POPUP,(AW_CL)create_dna_matrix_window,0);
