@@ -30,9 +30,9 @@ int ED4_Edit_String::nrepeat_is_already_set = 0;    // nrepeat was zero (and was
 unsigned char ED4_is_align_character[255];
 void ED4_init_is_align_character(GB_CSTR gap_chars)
 {
-    int c;
-    for (c=0; c<=255; c++) {
-        ED4_is_align_character[c] = (strchr(gap_chars, c)!=NULL);
+    memset(ED4_is_align_character, 0, 256);
+    for (int p = 0; gap_chars[p]; ++p) {
+        ED4_is_align_character[(unsigned char)(gap_chars[p])] = 1;
     }
 }
 
@@ -408,7 +408,7 @@ unsigned char ED4_Edit_String::get_gap_type(long pos, int direction)
 }
 
 ED4_ERROR *ED4_Edit_String::command( AW_key_mod keymod, AW_key_code keycode, char key, int direction, ED4_EDITMODI mode, bool is_consensus,
-                                     long &seq_pos, bool &changed_flag, bool &center_cursor, bool &cannot_handle, bool &write_fault, GBDATA* gb_data, bool is_sequence)
+                                     long &seq_pos, bool &changed_flag, ED4_CursorJumpType& cursor_jump, bool &cannot_handle, bool &write_fault, GBDATA* gb_data, bool is_sequence)
 {
     changed_flag = 0;
     write_fault = 0;
@@ -519,7 +519,7 @@ ED4_ERROR *ED4_Edit_String::command( AW_key_mod keymod, AW_key_code keycode, cha
 
                         // Ctrl+Cursor = move cursor to next end of word (or to end of next word)
 
-                        if (keymod&AW_KEY_CONTROL) {
+                        if (keymod & AW_KEYMODE_CONTROL) {
                             if (adjacent_scr_pos>=0) {
                                 long pos = adjacent_seq_pos;
 
@@ -549,7 +549,7 @@ ED4_ERROR *ED4_Edit_String::command( AW_key_mod keymod, AW_key_code keycode, cha
 
                         // ALT/META+Cursor = jump & fetch
 
-                        if (keymod & (AW_KEY_META|AW_KEY_ALT)) {
+                        if (keymod & (AW_KEYMODE_ALT)) {
                             if (is_consensus) { cannot_handle = 1; return 0; }
 
                             if (ADPP_IS_ALIGN_CHARACTER(seq[adjacent_seq_pos])) { // there's a _gap_ next to the cursor -> let's fetch
@@ -599,65 +599,6 @@ ED4_ERROR *ED4_Edit_String::command( AW_key_mod keymod, AW_key_code keycode, cha
                                     else {
                                         ad_err = GBS_global_string("You can only jump single bases.");
                                     }
-#if 0
-                                    if (gap_behind<0 || gap_behind>seq_len) {
-                                        ad_err = GBS_global_string("Cannot jump out of sequence!");
-                                    }
-                                    else {
-                                        if (ADPP_IS_ALIGN_CHARACTER(seq[gap_behind])) { // jump only one base
-                                            long dest_pos = get_next_base(gap_behind, direction); // destination pos of jumped base
-
-                                            if (dest_pos<0) {
-                                                dest_pos = direction<0 ? 0 : seq_len-1;
-                                                if (!ADPP_IS_ALIGN_CHARACTER(seq[dest_pos])) {
-                                                    dest_pos = -1;
-                                                }
-                                            }
-                                            else {
-                                                dest_pos -= direction;
-                                            }
-
-                                            if (dest_pos>=0) {
-                                                ad_err = moveBase(adjacent_seq_pos, dest_pos, get_gap_type(adjacent_seq_pos, -direction));
-
-                                                if (!ad_err) { // move cursor into opposite direction (to jump next base)
-                                                    seq_pos = get_next_base(adjacent_seq_pos, -direction);
-                                                    if (seq_pos==-1) {
-                                                        seq_pos = direction>0 ? 0 : seq_len; // if no next base -> cursor to beg or end of seq
-                                                    }
-                                                    else {
-                                                        seq_pos += (direction<0);
-                                                    }
-                                                }
-                                            }
-                                            else {
-                                                n = 0;
-                                            }
-                                        }
-                                        else { // jump two or more bases
-                                            long next_gap = get_next_gap(adjacent_seq_pos, direction);
-
-                                            if (next_gap>=0) {
-                                                long dest_pos = get_next_base(next_gap, direction);
-                                                long source_pos = get_next_base(next_gap, -direction);
-                                                long last_source = direction>0 ? seq_pos : seq_pos-1;
-
-                                                dest_pos = dest_pos==-1 ? (direction<0 ? 0 : seq_len-1) : dest_pos-direction;
-                                                e4_assert(source_pos>=0);
-
-                                                ad_err = shiftBases(source_pos, last_source, dest_pos, direction, &dest_pos,
-                                                                    get_gap_type(adjacent_seq_pos, -direction));
-
-                                                if (!ad_err) {
-                                                    seq_pos = dest_pos + (direction<0);
-                                                }
-                                            }
-                                            else {
-                                                n = 0;
-                                            }
-                                        }
-                                    }
-#endif
                                 }
                                 else {
                                     n = 0;
@@ -726,60 +667,60 @@ ED4_ERROR *ED4_Edit_String::command( AW_key_mod keymod, AW_key_code keycode, cha
                     } // end-while
                     break;
                 }
-            case AW_KEY_DELETE:
+                
             case AW_KEY_BACKSPACE:
-                {
-                    if (is_consensus) { cannot_handle = 1; return 0; };
-                    if (keymod) { return 0; }
+                h = seq_pos;
 
-                    if (mode==AD_NOWRITE) { write_fault = 1; break; }
-
-                    h = seq_pos;
-
-                    if (direction>0) {
-                        seq_pos -= nrepeat;
-                    }else{
-                        seq_pos += nrepeat;
-                    }
-                    if (seq_pos <0 || seq_pos >= seq_len) {
-                        seq_pos = h;
-                        break;
-                    }
-
-                    h = seq_pos;
-
-                    switch (mode) {
-                        case AD_ALIGN:
-                            {
-                                int len;
-                                int offset;
-
-                                ad_err = 0;
-                                if (direction>=0)   offset = 0;
-                                else        offset = -nrepeat;
-
-                                for (len = nrepeat-1; len>=0; len--) {
-                                    if (!ADPP_IS_ALIGN_CHARACTER(seq[h+offset+len])){
-                                        ad_err = GBS_global_string("You cannot remove bases in align mode");
-                                        break;
-                                    }
-                                }
-                                if (ad_err) break;
-                            }
-                        case AD_REPLACE:
-                        case AD_INSERT:
-                            ad_err = remove(nrepeat, h, direction, !is_sequence);
-                            if (!ad_err) {
-                                changed_flag = 1;
-                            }
-                            break;
-                        default:
-                            break;
-                    }
-
-                    //      nrepeat_zero_requested = 1;
+                if (direction>0) {
+                    seq_pos -= nrepeat;
+                }else{
+                    seq_pos += nrepeat;
+                }
+                if (seq_pos <0 || seq_pos >= seq_len) {
+                    seq_pos = h;
                     break;
                 }
+                // fall-through
+
+            case AW_KEY_DELETE:
+                h = seq_pos;
+
+                if (is_consensus) { cannot_handle = 1; return 0; };
+                if (keymod) { return 0; }
+
+                if (mode==AD_NOWRITE) { write_fault = 1; break; }
+
+                switch (mode) {
+                    case AD_ALIGN:
+                        {
+                            int len;
+                            int offset;
+
+                            ad_err = 0;
+                            if (direction>=0)   offset = 0;
+                            else        offset = -nrepeat;
+
+                            for (len = nrepeat-1; len>=0; len--) {
+                                if (!ADPP_IS_ALIGN_CHARACTER(seq[h+offset+len])){
+                                    ad_err = GBS_global_string("You cannot remove bases in align mode");
+                                    break;
+                                }
+                            }
+                            if (ad_err) break;
+                        }
+                    case AD_REPLACE:
+                    case AD_INSERT:
+                        ad_err = remove(nrepeat, h, direction, !is_sequence);
+                        if (!ad_err) {
+                            changed_flag = 1;
+                        }
+                        break;
+                    default:
+                        break;
+                }
+
+                //      nrepeat_zero_requested = 1;
+                break;
             case AW_KEY_ASCII: {
 
                 //      Tastaturbelegung:
@@ -803,7 +744,7 @@ ED4_ERROR *ED4_Edit_String::command( AW_key_mod keymod, AW_key_code keycode, cha
                             break;
                         }
                         case 'A': { // CTRL-A = Start Fast-Aligner
-                            AW_window *aw_tmp = ED4_ROOT->temp_aww;
+                            AW_window *aw_tmp = ED4_ROOT->get_aww();
                             if (is_consensus) { cannot_handle = 1; return 0; };
                             if (mode==AD_NOWRITE) { write_fault = 1; return 0; }
 
@@ -821,7 +762,7 @@ ED4_ERROR *ED4_Edit_String::command( AW_key_mod keymod, AW_key_code keycode, cha
                             int basesLeftOf = 0;
                             int pos;
 
-                            for (pos=0; pos<seq_pos && pos<seq_len; pos++) {    // count # of bases left of cursorpos
+                            for (pos=0; pos < seq_pos && pos<seq_len; pos++) {    // count # of bases left of cursorpos
                                 if (!ADPP_IS_ALIGN_CHARACTER(seq[pos])) {
                                     basesLeftOf++;
                                 }
@@ -833,13 +774,33 @@ ED4_ERROR *ED4_Edit_String::command( AW_key_mod keymod, AW_key_code keycode, cha
                             seq = aligned_seq;                  // set new sequence
                             changed_flag=1;                     // and mark changed
 
-                            for (pos=0; basesLeftOf>=0 && pos<seq_len; pos++) {
-                                if (!ADPP_IS_ALIGN_CHARACTER(seq[pos])) {
-                                    basesLeftOf--;
+                            {
+                                int basesLeftOf2   = 0;
+                                int lastCorrectPos = -1;
+                                for (pos=0; pos<seq_pos && pos<seq_len; pos++) {    // count # of bases left of cursorpos
+                                    if (!ADPP_IS_ALIGN_CHARACTER(seq[pos])) {
+                                        basesLeftOf2++;
+                                        if (basesLeftOf2 == basesLeftOf) lastCorrectPos = pos;
+                                    }
+                                }
+
+                                if (basesLeftOf != basesLeftOf2) { // old cursor-position has different basepos
+                                    if (lastCorrectPos != -1) { // already seen position with same basepos
+                                        seq_len = lastCorrectPos+1;
+                                    }
+                                    else {
+                                        for (; pos<seq_len; pos++) {
+                                            if (!ADPP_IS_ALIGN_CHARACTER(seq[pos])) {
+                                                basesLeftOf2++;
+                                                if (basesLeftOf2 == basesLeftOf) {
+                                                    seq_pos = pos+1;
+                                                    break; // stop after first matching position
+                                                }
+                                            }
+                                        }
+                                    }
                                 }
                             }
-
-                            seq_pos = pos ? pos-1 : pos;
                             break;
                         }
                         case 'E': { // CTRL-E = Toggle Edit/Align-Mode
@@ -857,15 +818,15 @@ ED4_ERROR *ED4_Edit_String::command( AW_key_mod keymod, AW_key_code keycode, cha
                         case 'J': { // CTRL-J = Jump to opposite helix position
                             AW_helix *helix = ED4_ROOT->helix;
 
-                            if (size_t(seq_pos)<helix->size && helix->entries[seq_pos].pair_type) {
-                                int pairing_pos = helix->entries[seq_pos].pair_pos;
-
-                                e4_assert(helix->entries[pairing_pos].pair_pos==seq_pos);
-                                seq_pos = pairing_pos;
-                                center_cursor = true;
+                            if (!helix->has_entries()) {
+                                ad_err = GB_strdup("Got no helix information");
+                            }
+                            else if (helix->pairtype(seq_pos) != HELIX_NONE) {
+                                seq_pos = helix->opposite_position(seq_pos);
+                                cursor_jump = ED4_JUMP_KEEP_POSITION;
                             }
                             else {
-                                ad_err = GBS_global_string("No helix position");
+                                ad_err = GB_strdup("Not at helix position");
                             }
                             break;
                         }
@@ -888,13 +849,13 @@ ED4_ERROR *ED4_Edit_String::command( AW_key_mod keymod, AW_key_code keycode, cha
                         }
                         case 'L':  { // CTRL-L = Refresh
                             ED4_refresh_window(0, 0, 0);
-                            center_cursor = true;
+                            cursor_jump = ED4_JUMP_CENTERED;
                             break;
                         }
                         case 'S': { // CTRL-S = Repeat last search
-                            ad_err = ED4_repeat_last_search();
-                            seq_pos = ED4_ROOT->temp_ed4w->cursor.get_sequence_pos();
-                            center_cursor = true;
+                            ad_err      = ED4_repeat_last_search();
+                            seq_pos     = ED4_ROOT->get_ed4w()->cursor.get_sequence_pos();
+                            cursor_jump = ED4_JUMP_KEEP_POSITION;
                             break;
                         }
                         case 'U': {
@@ -903,13 +864,13 @@ ED4_ERROR *ED4_Edit_String::command( AW_key_mod keymod, AW_key_code keycode, cha
                         }
                         case 'O': { //  for ALT-left
                             keycode = AW_KEY_LEFT;
-                            keymod = AW_KEY_META;
+                            keymod = AW_KEYMODE_ALT;
                             reinterpret_key = 1;
                             break;
                         }
                         case 'P': { // for ALT-right
                             keycode = AW_KEY_RIGHT;
-                            keymod = AW_KEY_META;
+                            keymod = AW_KEYMODE_ALT;
                             reinterpret_key = 1;
                             break;
                         }
@@ -1069,9 +1030,13 @@ void ED4_Edit_String::edit(ED4_work_info *info)
 {
     e4_assert(info->working_terminal != 0);
 
+    if (info->direction == 0) { // 5'<-3'
+        info->char_position++;
+    }
+
     info->out_seq_position = remap->screen_to_sequence(info->char_position);
     info->refresh_needed   = false;
-    info->center_cursor    = false;
+    info->cursor_jump      = ED4_JUMP_KEEP_VISIBLE;
     info->cannot_handle    = false;
     old_seq                = 0;
 
@@ -1084,9 +1049,6 @@ void ED4_Edit_String::edit(ED4_work_info *info)
         int seq_len_int;
         seq = info->working_terminal->resolve_pointer_to_string_copy(&seq_len_int);
         seq_len = seq_len_int;
-
-        //         seq     = GB_read_string(info->gb_data);
-        //         seq_len = GB_read_string_count(info->gb_data);
     }
 
     int map_key = ED4_ROOT->edk->map_key(info->event.character);
@@ -1151,7 +1113,7 @@ void ED4_Edit_String::edit(ED4_work_info *info)
                       (int)(info->string!=0),
                       info->out_seq_position,
                       info->refresh_needed,
-                      info->center_cursor,
+                      info->cursor_jump,
                       info->cannot_handle,
                       write_fault,
                       info->gb_data,
@@ -1190,6 +1152,13 @@ void ED4_Edit_String::edit(ED4_work_info *info)
             seq = 0;
         }
     }
+
+    if (info->direction == 0) {
+        info->char_position = remap->sequence_to_screen(info->out_seq_position);
+        info->char_position--;
+        info->out_seq_position = remap->screen_to_sequence(info->char_position);
+    }
+
     info->error = (char *)err;
 }
 
