@@ -19,13 +19,16 @@
 #include <AW_helix.hxx>
 #include <st_window.hxx>
 #include <gde.hxx>
+#include<awt_pro_a_nucs.hxx>
+
+#include <ed4_extern.hxx>
 
 #include "ed4_class.hxx"
 #include "ed4_awars.hxx"
-#include "ed4_defs.hxx"
 #include "ed4_edit_string.hxx"
 #include "ed4_nds.hxx"
 #include "ed4_visualizeSAI.hxx"
+#include "ed4_ProteinViewer.hxx"
 
 #include "edit_naligner.hxx"
 
@@ -43,7 +46,6 @@ int SEQ_TERM_TEXT_YOFFSET;
 int  MAXSEQUENCECHARACTERLENGTH; // greatest # of characters in a sequence string terminal
 int  MAXSPECIESWIDTH;
 int  MAXINFOWIDTH;              // # of pixels used to display sequence info ("CONS", "4data", etc.)
-int  MARGIN;                    // sets margin for cursor moves in characters
 
 long          ED4_counter = 0;
 long          all_found;        // nr of species which haven't been found
@@ -246,7 +248,7 @@ static char *add_area_for_gde(ED4_area_manager *area_man, uchar **&the_names, uc
 
                 if (seq) {
                     set_and_realloc_gde_array(the_names, the_sequences, allocated, numberspecies, maxalign, name, name_len, seq, seq_len);
-                    if (show_helix && !is_SAI && ED4_ROOT->helix->size) {
+                    if (show_helix && !is_SAI && ED4_ROOT->helix->size()) {
                         char *helix = ED4_ROOT->helix->seq_2_helix(seq, '.');
                         set_and_realloc_gde_array(the_names, the_sequences, allocated, numberspecies, maxalign, name, name_len, helix, seq_len);
                     }
@@ -318,14 +320,8 @@ static void ED4_gap_chars_changed(AW_root *root) {
     ED4_init_is_align_character(gap_chars);
 }
 
-void ED4_create_global_awars(AW_root *root) {
-    root->awar_string(AWAR_ITARGET_STRING, "", gb_main);
-    ED4_create_search_awars(root);
-}
-
 static void ED4_edit_direction_changed(AW_root */*awr*/) {
-    ED4_ROOT->temp_ed4w->cursor.redraw();
-
+    ED4_ROOT->get_ed4w()->cursor.redraw();
 }
 
 void ED4_expose_all_windows() {
@@ -338,9 +334,40 @@ static void ED4_do_expose(AW_root *) {
     ED4_expose_all_windows();
 }
 
-void ED4_create_all_awars(AW_root *root, const char *config_name) { // cursor awars are created in window constructor
+static void ed4_bind_mainDB_awar_callbacks(AW_root *root) {
+    // callbacks to main DB awars are bound later
+    // (otherwise its easy to crash the editor by clicking around in ARB_NTREE during editor startup)
 
-    ED4_create_global_awars(root);
+    root->awar(AWAR_SET_CURSOR_POSITION)->add_callback(ED4_remote_set_cursor_cb, 0, 0);
+    root->awar(AWAR_SPECIES_NAME)->add_callback(ED4_selected_species_changed_cb);
+    root->awar(AWAR_SAI_NAME)->add_callback(ED4_selected_SAI_changed_cb);
+}
+
+static void ed4_create_mainDB_awars(AW_root *root, const char *config_name) {
+    // WARNING: do not bind callbacks here -> do it in ed4_bind_mainDB_awar_callbacks()
+
+    root->awar_string(AWAR_ITARGET_STRING, "", gb_main);
+
+    root->awar_int(AWAR_CURSOR_POSITION, 1, gb_main);
+    root->awar_int( AWAR_CURSOR_POSITION_LOCAL, 0, gb_main);
+    root->awar_int(AWAR_SET_CURSOR_POSITION, 1, gb_main);
+
+    root->awar_string(AWAR_FIELD_CHOSEN, "", gb_main);
+
+    root->awar_string(AWAR_SPECIES_NAME, "", gb_main);
+    root->awar_string(AWAR_SAI_NAME, "", gb_main);
+    root->awar_string(AWAR_SAI_GLOBAL, "", gb_main);
+    //    root->awar_string(AWAR_SAI_COLOR_STR, "", gb_main); //sai visualization in probe match
+
+    root->awar_string(AWAR_EDIT_CONFIGURATION, config_name, gb_main);
+
+    ED4_create_search_awars(root);
+}
+
+static void ed4_create_all_awars(AW_root *root, const char *config_name) {
+    // Note: cursor awars are created in window constructor
+
+    ed4_create_mainDB_awars(root, config_name);
     ARB_init_global_awars(root, AW_ROOT_DEFAULT, gb_main);
     
     AWT_create_db_browser_awars(root, AW_ROOT_DEFAULT);
@@ -368,18 +395,6 @@ void ED4_create_all_awars(AW_root *root, const char *config_name) { // cursor aw
     root->awar_int( AWAR_EDIT_TERMINAL_SPACING, 0)->add_target_var(&ED4_ROOT->terminal_add_spacing)->add_callback(ED4_do_expose);
     root->awar_int( AWAR_EDIT_TITLE_MODE, 0);
 
-    root->awar_int(AWAR_CURSOR_POSITION, 1, gb_main);
-    root->awar_int( AWAR_CURSOR_POSITION_LOCAL, 0, gb_main);
-    root->awar_int(AWAR_SET_CURSOR_POSITION, 1, gb_main)->add_callback(ED4_remote_set_cursor_cb, 0, 0);
-
-    root->awar_string(AWAR_FIELD_CHOSEN, "", gb_main);
-
-    root->awar_string(AWAR_SPECIES_NAME, "", gb_main)->add_callback(ED4_selected_species_changed_cb);
-    root->awar_string(AWAR_SAI_NAME, "", gb_main)->add_callback(ED4_selected_SAI_changed_cb);
-    root->awar_string(AWAR_SAI_GLOBAL, "", gb_main);
-    //    root->awar_string(AWAR_SAI_COLOR_STR, "", gb_main); //sai visualization in probe match
-
-    root->awar_string(AWAR_EDIT_CONFIGURATION,config_name,gb_main);
     ed4_changesecurity(root,0);
 
     root->awar_int(ED4_AWAR_COMPRESS_SEQUENCE_GAPS,0)->add_callback(ED4_compression_toggle_changed_cb, AW_CL(0), 0);
@@ -398,8 +413,9 @@ void ED4_create_all_awars(AW_root *root, const char *config_name) { // cursor aw
     root->awar_string(ED4_AWAR_REP_SEARCH_PATTERN, ".");
     root->awar_string(ED4_AWAR_REP_REPLACE_PATTERN, "-");
 
-    root->awar_int( ED4_AWAR_SCROLL_SPEED_X, 40);
-    root->awar_int( ED4_AWAR_SCROLL_SPEED_Y, 20);
+    root->awar_int(ED4_AWAR_SCROLL_SPEED_X, 40);
+    root->awar_int(ED4_AWAR_SCROLL_SPEED_Y, 20);
+    root->awar_int(ED4_AWAR_SCROLL_MARGIN, 5);
 
     root->awar_string(ED4_AWAR_SPECIES_TO_CREATE, "");
 
@@ -416,6 +432,11 @@ void ED4_create_all_awars(AW_root *root, const char *config_name) { // cursor aw
     root->awar_int(ED4_AWAR_CREATE_FROM_CONS_DATA_SOURCE, 0);
 
     ED4_createVisualizeSAI_Awars(root,AW_ROOT_DEFAULT);
+
+    // Create Awars To Be Used In Protein Viewer
+    if(ED4_ROOT->alignment_type == GB_AT_DNA) {
+        PV_CreateAwars(root,AW_ROOT_DEFAULT);
+    }
 }
 
 const char *ED4_propertyName(int mode) {
@@ -464,7 +485,7 @@ static void openProperties() {
 
 static char *ED4_find_protein_structure_SAI(GBDATA *gb_main, const char *alignment_name) {
     GB_transaction dummy(gb_main);
-	GBDATA *gb_extended_data = GB_search(gb_main,"extended_data",GB_CREATE_CONTAINER);
+    GBDATA *gb_extended_data = GB_search(gb_main,"extended_data",GB_CREATE_CONTAINER);
     GBDATA *gb_protstruct    = GBT_find_SAI_rel_exdata(gb_extended_data, "protstruct");
 
     if (gb_protstruct) {
@@ -532,7 +553,7 @@ int main(int argc,char **argv)
 
     ED4_ROOT->database = new EDB_root_bact;
     ED4_ROOT->init_alignment();
-    ED4_create_all_awars(ED4_ROOT->aw_root, config_name);
+    ed4_create_all_awars(ED4_ROOT->aw_root, config_name);
 
     ED4_ROOT->st_ml = new_ST_ML(gb_main);
     ED4_ROOT->sequence_colors = new AWT_seq_colors((GBDATA *)ED4_ROOT->aw_root->application_database,(int)ED4_G_SEQUENCES, ED4_refresh_window, 0,0);
@@ -559,7 +580,9 @@ int main(int argc,char **argv)
                 ED4_ROOT->protstruct_len = strlen(ED4_ROOT->protstruct);
             }
             else {
+#if defined(DEBUG)
                 fprintf(stderr, "No SAI \"protstruct\" found. Protein structure information will not be displayed.\n");
+#endif // DEBUG
             }
             break;
 
@@ -568,9 +591,9 @@ int main(int argc,char **argv)
             break;
     }
 
-    if (err) aw_message(err);
+    if (err) aw_message(err); // write to console
     ED4_ROOT->sequence_colors->aww = ED4_ROOT->create_new_window(); // create first editor window
-    if (err) aw_message(err);
+    if (err) aw_message(err); // write again to status window
 
     {
         int found_config = 0;
@@ -612,6 +635,9 @@ int main(int argc,char **argv)
         }
     }
 
+    // now bind DB depending callbacks
+
+    ed4_bind_mainDB_awar_callbacks(ED4_ROOT->aw_root);
     {
         GB_transaction dummy(gb_main);
         GBDATA *species_container = GB_search(gb_main, "species_data", GB_FIND);
@@ -622,6 +648,11 @@ int main(int argc,char **argv)
         printf("Species container contains %i species (at startup)\n", ED4_elements_in_species_container);
 #endif
     }
+    
+    // Create Additional sequence (aminoacid) terminals to be used in Protein Viewer
+    if(ED4_ROOT->alignment_type == GB_AT_DNA) {
+        PV_CallBackFunction(ED4_ROOT->aw_root);
+    }
 
     ED4_ROOT->aw_root->main_loop(); // enter main-loop
 }
