@@ -19,12 +19,14 @@
 #include <awtc_fast_aligner.hxx>
 #include <awtc_constructSequence.hxx>
 
-#include "../NTREE/nt_date.h"
+#include <arb_version.h>
 
 #include <AW_helix.hxx>
 #include <st_window.hxx>
 #include <gde.hxx>
 
+#include <ed4_extern.hxx>
+
 #include "ed4_class.hxx"
 #include "ed4_awars.hxx"
 #include "ed4_tools.hxx"
@@ -33,6 +35,7 @@
 #include "ed4_secedit.hxx"
 #include "ed4_visualizeSAI.hxx"
 #include "edit_naligner.hxx"
+#include "ed4_ProteinViewer.hxx"
 
 #if defined(ARB_OPENGL)
 #include "ed4_RNA3D.hxx"
@@ -107,16 +110,7 @@ ED4_returncode ED4_root::refresh_window(int redraw) // this function should only
 
 ED4_returncode ED4_root::refresh_all_windows(int redraw)
 {
-#ifdef DEBUG
-    //    printf("ED4_root::refresh_all_windows this=%p\n", this);
-#endif
-    ED4_window  *window, *ed4w_temp;
-    AW_window   *aw_temp;
-    AW_device   *dev_temp;
-
-    ed4w_temp = ED4_ROOT->temp_ed4w;
-    aw_temp   = ED4_ROOT->temp_aww;
-    dev_temp  = ED4_ROOT->temp_device;
+    ED4_window *old_window = get_ed4w();
 
     last_window_reached = 0;
 
@@ -130,33 +124,17 @@ ED4_returncode ED4_root::refresh_all_windows(int redraw)
         redraw = 1;
     }
 
-#ifdef DEBUG
-    //    printf("ED4_root::refresh_all_windows this=%p [mid]\n", this);
-#endif
-
-    window = ED4_ROOT->first_window;
+    ED4_window *window = first_window;
     GB_transaction dummy(gb_main);
     while (window) {
-        if (window->next == NULL) {
-            last_window_reached = 1;
-        }
-
-        ED4_ROOT->temp_aww  = window->aww;
-        ED4_ROOT->temp_device   = window->aww->get_device(AW_MIDDLE_AREA);
-        ED4_ROOT->temp_ed4w     = window;
-
+        if (!window->next) last_window_reached = 1;
+        use_window(window);
         refresh_window_simple(redraw);
-
         window = window->next;
     }
 
-    ED4_ROOT->temp_ed4w = ed4w_temp;
-    ED4_ROOT->temp_aww = aw_temp;
-    ED4_ROOT->temp_device = dev_temp;
+    use_window(old_window);
 
-#ifdef DEBUG
-    //    printf("ED4_root::refresh_all_windows this=%p [done]\n", this);
-#endif
     return ( ED4_R_OK );
 }
 
@@ -167,7 +145,7 @@ ED4_returncode ED4_root::win_to_world_coords( AW_window *aww, AW_pos *x, AW_pos
     ED4_folding_line  *current_fl;
     AW_pos             temp_x, temp_y;
 
-    current_window = ED4_ROOT->first_window->get_matching_ed4w( aww );
+    current_window = first_window->get_matching_ed4w( aww );
 
     if ( current_window == NULL )
         return ( ED4_R_WARNING );
@@ -216,10 +194,10 @@ short ED4_root::is_primary_selection(ED4_terminal *object )
 
 ED4_returncode ED4_root::deselect_all( void )
 {
-    ED4_multi_species_manager *main_multi_man = ED4_ROOT->middle_area_man->get_defined_level(ED4_L_MULTI_SPECIES)->to_multi_species_manager();
+    ED4_multi_species_manager *main_multi_man = middle_area_man->get_defined_level(ED4_L_MULTI_SPECIES)->to_multi_species_manager();
 
     main_multi_man->deselect_all_species();
-    main_multi_man = ED4_ROOT->top_area_man->get_defined_level(ED4_L_MULTI_SPECIES)->to_multi_species_manager();
+    main_multi_man = top_area_man->get_defined_level(ED4_L_MULTI_SPECIES)->to_multi_species_manager();
     main_multi_man->deselect_all_species();
 
     return ED4_R_OK;
@@ -282,6 +260,11 @@ ED4_returncode  ED4_root::remove_from_selected( ED4_terminal *object )
                 seq_term->parent->refresh_requested_by_child();
             }
 
+            //ProtView: Refresh corresponding AA_sequence terminals
+            if (alignment_type == GB_AT_DNA) {
+                PV_CallBackFunction(this->aw_root); 
+            }
+
             ED4_multi_species_manager *multi_man = object->get_parent(ED4_L_MULTI_SPECIES)->to_multi_species_manager();
             multi_man->invalidate_species_counters();
         }
@@ -297,7 +280,7 @@ ED4_returncode  ED4_root::remove_from_selected( ED4_terminal *object )
 void ED4_root::announce_deletion(ED4_base *object) {
     // remove any links which might point to the object
 
-    ED4_cursor& cursor = temp_ed4w->cursor;
+    ED4_cursor& cursor = get_ed4w()->cursor;
     if (cursor.owner_of_cursor == object) { // about to delete owner_of_cursor
         cursor.HideCursor();
         e4_assert(cursor.owner_of_cursor != object);
@@ -305,7 +288,7 @@ void ED4_root::announce_deletion(ED4_base *object) {
 }
 
 
-ED4_returncode  ED4_root::add_to_selected( ED4_terminal *object )
+ED4_returncode ED4_root::add_to_selected( ED4_terminal *object )
 {
     ED4_base            *tmp_object;
     ED4_level            mlevel;
@@ -389,6 +372,11 @@ ED4_returncode  ED4_root::add_to_selected( ED4_terminal *object )
                 seq_term->parent->refresh_requested_by_child();
             }
 
+            //ProtView: Refresh corresponding AA_sequence terminals
+            if (alignment_type == GB_AT_DNA) {
+                PV_CallBackFunction(this->aw_root); 
+            }
+
             ED4_multi_species_manager *multi_man = object->get_parent(ED4_L_MULTI_SPECIES)->to_multi_species_manager();
             multi_man->invalidate_species_counters();
         }
@@ -445,7 +433,8 @@ void ED4_alignment_length_changed(GBDATA *gb_alignment_len, int */*cl*/, GB_CB_T
         if (err) { aw_message(err); err = 0; }
 
         if (was_increased) {
-            ED4_ROOT->main_manager->route_down_hierarchy((void**)&new_length, 0, change_char_table_length);
+            int *new_length_ptr = &new_length;
+            ED4_ROOT->main_manager->route_down_hierarchy((void**)new_length_ptr, 0, change_char_table_length);
             ED4_ROOT->root_group_man->remap()->mark_compile_needed_force();
         }
     }
@@ -458,7 +447,7 @@ ED4_returncode ED4_root::init_alignment() {
     alignment_type = GBT_get_alignment_type(gb_main,alignment_name);
     if (alignment_type==GB_AT_UNKNOWN) aw_message("You have to select a valid alignment before you can start ARB_EDIT4", "EXIT");
 
-    GBDATA *gb_alignment = GBT_get_alignment(gb_main,ED4_ROOT->alignment_name);
+    GBDATA *gb_alignment = GBT_get_alignment(gb_main, alignment_name);
     if (!gb_alignment) aw_message("You can't edit without an existing alignment", "EXIT");
 
     GBDATA *gb_alignment_len = GB_search(gb_alignment,"alignment_len",GB_FIND);
@@ -467,7 +456,7 @@ ED4_returncode ED4_root::init_alignment() {
 
     GB_add_callback(gb_alignment_len, (GB_CB_TYPE)GB_CB_CHANGED, (GB_CB)ED4_alignment_length_changed, 0);
 
-    ED4_ROOT->aw_root->awar_string(AWAR_EDITOR_ALIGNMENT, ED4_ROOT->alignment_name);
+    aw_root->awar_string(AWAR_EDITOR_ALIGNMENT, alignment_name);
 
     return ED4_R_OK;
 }
@@ -475,7 +464,7 @@ ED4_returncode ED4_root::init_alignment() {
 void ED4_root::recalc_font_group() {
     font_group.unregisterAll();
     for (int f=ED4_G_FIRST_FONT; f<=ED4_G_LAST_FONT; f++) {
-        font_group.registerFont(temp_device, f);
+        font_group.registerFont(get_device(), f);
     }
 }
 
@@ -487,11 +476,20 @@ ED4_returncode ED4_root::create_hierarchy(char *area_string_middle, char *area_s
     ED4_window *new_window;
     long total_no_of_species, total_no_of_groups, group_count, species_count;
 
+    /*****YKADI COMMENT****************/
+    // this is the sequence reference region - it counts the species and related info (e.g. helix) displayed
+    //in the REFERENCE region -- TOP REGION
+    /*****YKADI COMMENT****************/
+
     database->calc_no_of_all(area_string_top, &group_count, &species_count);
     total_no_of_groups = group_count;
     total_no_of_species = species_count;
     loading = 1;
 
+    /*****YKADI COMMENT****************/
+    //  MIDDLE  REGION -- counts no of species and sais including groups
+    /*****YKADI COMMENT****************/
+
     database->calc_no_of_all(area_string_middle, &group_count, &species_count);
     total_no_of_groups += group_count;
     total_no_of_species += species_count;
@@ -545,7 +543,7 @@ ED4_returncode ED4_root::create_hierarchy(char *area_string_middle, char *area_s
         {
             ED4_area_manager *top_area_manager = new ED4_area_manager("Top_Area_Manager", 0, y, 0, 0, device_manager);
             device_manager->children->append_member(top_area_manager);
-            ED4_ROOT->top_area_man = top_area_manager;
+            top_area_man = top_area_manager;
 
             top_spacer_terminal = new ED4_spacer_terminal( "Top_Spacer" , 0, 0, 100, 10, top_area_manager);
             top_area_manager->children->append_member( top_spacer_terminal );
@@ -565,9 +563,9 @@ ED4_returncode ED4_root::create_hierarchy(char *area_string_middle, char *area_s
             }
 
 
-            ED4_ROOT->reference = new AWT_reference(gb_main);
-            ED4_ROOT->database->scan_string(top_multi_species_manager, ref_terminals.get_ref_sequence_info(), ref_terminals.get_ref_sequence(),
-                                            area_string_top, &index, &y);
+            reference = new AWT_reference(gb_main);
+            database->scan_string(top_multi_species_manager, ref_terminals.get_ref_sequence_info(), ref_terminals.get_ref_sequence(),
+                                  area_string_top, &index, &y);
             GB_pop_transaction( gb_main );
 
             const int TOP_MID_LINE_HEIGHT   = 3;
@@ -602,7 +600,7 @@ ED4_returncode ED4_root::create_hierarchy(char *area_string_middle, char *area_s
             //  printf("Middle Area y : %d\n",y);
             middle_area_manager = new ED4_area_manager("Middle_Area_Manager", 0, y, 0, 0, device_manager);
             device_manager->children->append_member(middle_area_manager);
-            ED4_ROOT->middle_area_man = middle_area_manager;
+            middle_area_man = middle_area_manager;
 
             tree_terminal = new ED4_tree_terminal( "Tree", 0, 0, 2, change, middle_area_manager);
             middle_area_manager->children->append_member( tree_terminal );
@@ -621,8 +619,8 @@ ED4_returncode ED4_root::create_hierarchy(char *area_string_middle, char *area_s
             index = 0;
             {
                 GB_transaction dummy(gb_main);
-                ED4_ROOT->database->scan_string(mid_multi_species_manager, ref_terminals.get_ref_sequence_info(), ref_terminals.get_ref_sequence(),
-                                                area_string_middle, &index, &y);
+                database->scan_string(mid_multi_species_manager, ref_terminals.get_ref_sequence_info(), ref_terminals.get_ref_sequence(),
+                                      area_string_middle, &index, &y);
             }
 
             if (total_no_of_species > MINSPECFORSTATWIN) {
@@ -637,7 +635,7 @@ ED4_returncode ED4_root::create_hierarchy(char *area_string_middle, char *area_s
             tree_terminal->extension.size[HEIGHT] = y - help;
 
             mid_multi_species_manager->generate_id_for_groups();
-            y += 10;                                                // add top-mid_spacer_terminal height
+            y += 10;                                                // add top-mid_spacer_terminal height 
 
             mid_bot_line_terminal = new ED4_line_terminal( "Mid_Bot_Line_Terminal" ,0, y, 0, 3, device_manager );   // width will be set below
             device_manager->children->append_member( mid_bot_line_terminal );
@@ -650,8 +648,8 @@ ED4_returncode ED4_root::create_hierarchy(char *area_string_middle, char *area_s
 
         // ********** Middle Area end **********
 
-        if (ED4_ROOT->scroll_links.link_for_hor_slider) {
-            long ext_width = long(ED4_ROOT->scroll_links.link_for_hor_slider->extension.size[WIDTH]);
+        if (scroll_links.link_for_hor_slider) {
+            long ext_width = long(scroll_links.link_for_hor_slider->extension.size[WIDTH]);
 
             top_multi_spacer_terminal_beg->extension.size[WIDTH] = ext_width + MAXSPECIESWIDTH + SEQUENCEINFOSIZE;
             mid_multi_spacer_terminal_beg->extension.size[WIDTH] = ext_width + MAXSPECIESWIDTH + SEQUENCEINFOSIZE;
@@ -718,7 +716,7 @@ ED4_returncode ED4_root::create_hierarchy(char *area_string_middle, char *area_s
         new_window = new_window->next;
     }
 
-    ED4_ROOT->aw_root->add_timed_callback(2000,ED4_timer,(AW_CL)0,(AW_CL)0);
+    aw_root->add_timed_callback(2000,ED4_timer,(AW_CL)0,(AW_CL)0);
 
     char *out_message = GBS_strclose(not_found_message);
     not_found_message = 0;
@@ -732,10 +730,10 @@ ED4_returncode ED4_root::get_area_rectangle(AW_rectangle *rect, AW_pos x, AW_pos
     ED4_folding_line *flv, *flh;
     int x1, x2, y1, y2;
     AW_rectangle area_rect;
-    temp_device->get_area_size(&area_rect);
+    get_device()->get_area_size(&area_rect);
 
     x1 = area_rect.l;
-    for (flv=temp_ed4w->vertical_fl; ; flv = flv->next) {
+    for (flv=get_ed4w()->vertical_fl; ; flv = flv->next) {
         if (flv) {
             e4_assert(flv->length==INFINITE);
             x2 = int(flv->window_pos[X_POS]);
@@ -748,7 +746,7 @@ ED4_returncode ED4_root::get_area_rectangle(AW_rectangle *rect, AW_pos x, AW_pos
         }
 
         y1 = area_rect.t;
-        for (flh=temp_ed4w->horizontal_fl; ; flh = flh->next) {
+        for (flh=get_ed4w()->horizontal_fl; ; flh = flh->next) {
             if (flh) {
                 e4_assert(flh->length==INFINITE);
                 y2 = int(flh->window_pos[Y_POS]);
@@ -782,8 +780,8 @@ ED4_returncode ED4_root::world_to_win_coords(AW_window *IF_DEBUG(aww), AW_pos *x
     ED4_folding_line    *current_fl;
     AW_pos      temp_x, temp_y, x, y;
 
-    current_window = ED4_ROOT->temp_ed4w;
-    e4_assert(current_window==ED4_ROOT->first_window->get_matching_ed4w(aww));
+    current_window = get_ed4w();
+    e4_assert(current_window==first_window->get_matching_ed4w(aww));
 
     if (! current_window) {
         e4_assert(0);
@@ -1124,7 +1122,7 @@ static void title_mode_changed(AW_root *aw_root, AW_window *aww)
     int title_mode = aw_root->awar(AWAR_EDIT_TITLE_MODE)->read_int();
 
     if (title_mode==0) {
-        aww->set_info_area_height(55);
+        aww->set_info_area_height(57);
     }
     else {
         aww->set_info_area_height(170);
@@ -1152,7 +1150,7 @@ void ED4_undo_redo(AW_window*, AW_CL undo_type)
     else {
         GB_begin_transaction(gb_main);
         GB_commit_transaction(gb_main);
-        ED4_cursor *cursor = &ED4_ROOT->temp_ed4w->cursor;
+        ED4_cursor *cursor = &ED4_ROOT->get_ed4w()->cursor;
         if (cursor->owner_of_cursor) {
             ED4_terminal *terminal = cursor->owner_of_cursor->to_terminal();
 
@@ -1168,7 +1166,7 @@ void aw_clear_message_cb(AW_window *aww);
 void ED4_clear_errors(AW_window*, AW_CL)
 {
     //    ED4_ROOT->aw_root->awar(AWAR_ERROR_MESSAGES)->write_string("");
-    aw_clear_message_cb(ED4_ROOT->temp_aww);
+    aw_clear_message_cb(ED4_ROOT->get_aww());
 }
 
 AW_window *ED4_zoom_message_window(AW_root *root, AW_CL)
@@ -1234,7 +1232,7 @@ static void insert_search_fields(AW_window_menu_modes *awmm,
 
 void ED4_set_protection(AW_window */*aww*/, AW_CL cd1, AW_CL /*cd2*/)
 {
-    ED4_cursor *cursor = &ED4_ROOT->temp_ed4w->cursor;
+    ED4_cursor *cursor = &ED4_ROOT->get_ed4w()->cursor;
 
     if (cursor->owner_of_cursor) {
         int wanted_protection = int(cd1);
@@ -1366,7 +1364,7 @@ void ED4_no_dangerous_modes(void)
     switch (ED4_ROOT->species_mode) {
         case ED4_SM_KILL: {
             ED4_ROOT->species_mode = ED4_SM_MOVE;
-            ED4_ROOT->temp_aww->select_mode(0);
+            ED4_ROOT->get_aww()->select_mode(0);
             break;
         }
         default: {
@@ -1421,7 +1419,7 @@ static void ED4_save_defaults(AW_window *aw, AW_CL cl_mode, AW_CL) {
 ED4_returncode ED4_root::generate_window( AW_device **device,   ED4_window **new_window)
 {
     AW_window_menu_modes *awmm;
-    AW_gc_manager         aw_gc_manager; //every window has its own gc_manager
+    AW_gc_manager         win_gc_manager; //every window has its own gc_manager
     ED4_window           *ed4w = first_window;
 
     while (ed4w)                                        // before creating a window look for a hidden window
@@ -1452,14 +1450,15 @@ ED4_returncode ED4_root::generate_window( AW_device **device,   ED4_window **new
     *device     = awmm->get_device(AW_MIDDLE_AREA); //Points to Middle Area device
     *new_window = ED4_window::insert_window( awmm ); //append to window list
 
-    if (ED4_window::no_of_windows >= 1)                         // this is the first window
-    {
+    e4_assert(ED4_window::no_of_windows >= 1);
+    if (ED4_window::no_of_windows == 1) { // this is the first window
+        AW_init_color_group_defaults("arb_edit4");
+    }
+    else { // additional edit windows
         copy_window_struct( first_window, *new_window );
     }
 
-    AW_init_color_group_defaults("arb_edit4");
-
-    aw_gc_manager   = AW_manage_GC( awmm,                           //Window
+    win_gc_manager   = AW_manage_GC( awmm,                           //Window
                                     *device,                //device-handle of window
                                     ED4_G_STANDARD,                     //GC_Standard configuration
                                     ED4_G_DRAG,
@@ -1471,7 +1470,7 @@ ED4_returncode ED4_root::generate_window( AW_device **device,   ED4_window **new
 
                                     "#f8f8f8",
                                     "STANDARD$black", // Standard Color showing sequences
-                                    "#SEQUENCES$#808080", // default color for sequences (color 0)
+                                    "#SEQUENCES (0)$#505050", // default color for sequences (color 0)
                                     "+-HELIX (1)$#8E0000",  "+-COLOR 2$#0000dd",    "-COLOR 3$#00AA55",
                                     "+-COLOR 4$#80f",       "+-COLOR 5$#c0a020",    "-COLOR 6$grey",
                                     "+-COLOR 7$#ff0000",    "+-COLOR 8$#44aaff",    "-COLOR 9$#ffaa00",
@@ -1492,6 +1491,9 @@ ED4_returncode ED4_root::generate_window( AW_device **device,   ED4_window **new
 
                                     0);
 
+    aw_gc_manager = win_gc_manager;
+
+
 #define SEP________________________SEP awmm->insert_separator()
 
     // ------------------------------
@@ -1570,7 +1572,7 @@ ED4_returncode ED4_root::generate_window( AW_device **device,   ED4_window **new
     awmm->insert_menu_topic("align_sequence","Old Aligner From ARB_EDIT", "O","ne_align_seq.hlp", AWM_ALL,AW_POPUP, (AW_CL)create_naligner_window, 0 );
     awmm->insert_menu_topic("del_ali_tmp", "Remove All Aligner Entries", "R", 0, AWM_ALL, ED4_remove_faligner_entries, 1, 0);
     SEP________________________SEP;
-    awmm->insert_menu_topic("sec_edit", "Edit Secondary Structure", "", 0, AWM_ALL, ED4_SECEDIT_start, 0, 0);
+    awmm->insert_menu_topic("sec_edit", "Edit Secondary Structure", "", 0, AWM_ALL, ED4_SECEDIT_start, (AW_CL)gb_main, 0);
 
     // ------------------------------
     //  View
@@ -1622,6 +1624,10 @@ ED4_returncode ED4_root::generate_window( AW_device **device,   ED4_window **new
     awmm->insert_menu_topic( "dcs_threshold", "Set threshold for D.c.s.", "f", "st_ml.hlp",AWM_ALL, ED4_set_col_stat_threshold, 1, 0);
     SEP________________________SEP;
     awmm->insert_menu_topic( "visulize_SAI", "Visualize SAIs", "z", "visualizeSAI.hlp", AWM_ALL,AW_POPUP,(AW_CL)ED4_createVisualizeSAI_window, 0 );
+    // Enable ProteinViewer only for DNA sequence type
+    if (alignment_type == GB_AT_DNA) {
+        awmm->insert_menu_topic( "Protein_Viewer", "Protein Viewer", "w", "proteinViewer.hlp", AWM_ALL,AW_POPUP,(AW_CL)ED4_CreateProteinViewer_window, 0 );
+    }
 
     // ------------------------------
     //  Block
@@ -1666,8 +1672,8 @@ ED4_returncode ED4_root::generate_window( AW_device **device,   ED4_window **new
     awmm->insert_menu_topic( "props_consensus", "Consensus Definition " , 0, "e4_consensus.hlp",    AWM_ALL, AW_POPUP, (AW_CL)ED4_create_consensus_definition_window, 0 );
     SEP________________________SEP;
 
-    awmm->insert_menu_topic( "props_data", "Change Colors & Fonts "  ,  "C", 0,     AWM_ALL, AW_POPUP, (AW_CL)AW_create_gc_window, (AW_CL)aw_gc_manager );
-    awmm->insert_menu_topic( "props_seq_colors", "Set Sequence Colors "  , "S", "no help",   AWM_ALL, AW_POPUP, (AW_CL)create_seq_colors_window, (AW_CL)ED4_ROOT->sequence_colors );
+    awmm->insert_menu_topic( "props_data", "Change Colors & Fonts "  ,  "C", 0,     AWM_ALL, AW_POPUP, (AW_CL)AW_create_gc_window, (AW_CL)win_gc_manager );
+    awmm->insert_menu_topic( "props_seq_colors", "Set Sequence Colors "  , "S", "no help",   AWM_ALL, AW_POPUP, (AW_CL)create_seq_colors_window, (AW_CL)sequence_colors);
     SEP________________________SEP;
     awmm->insert_menu_topic( "props_helix_sym", "Helix Settings ","H","helixsym.hlp", AWM_ALL, AW_POPUP, (AW_CL)create_helix_props_window, (AW_CL)new AW_cb_struct(awmm,(AW_CB)ED4_expose_cb,0,0) );
     awmm->insert_menu_topic( "props_key_map", "Key Mappings ", "K","nekey_map.hlp", AWM_ALL, AW_POPUP, (AW_CL)create_key_map_window, 0 );
@@ -1681,13 +1687,13 @@ ED4_returncode ED4_root::generate_window( AW_device **device,   ED4_window **new
         // check what is the default mode
         int default_mode = -1;
         for (int mode = 0; mode <= 2; ++mode) {
-            if (0 == strcmp(ED4_propertyName(mode), ED4_ROOT->db_name)) {
+            if (0 == strcmp(ED4_propertyName(mode), db_name)) {
                 default_mode = mode;
                 break;
             }
         }
 
-        const char *entry = GBS_global_string("Save loaded Properties (~/%s)", ED4_ROOT->db_name);
+        const char *entry = GBS_global_string("Save loaded Properties (~/%s)", db_name);
         awmm->insert_menu_topic("save_loaded_props", entry, "l", "e4_defaults.hlp", AWM_EXP, ED4_save_defaults, (AW_CL)default_mode, 0);
         SEP________________________SEP;
 
@@ -1700,6 +1706,8 @@ ED4_returncode ED4_root::generate_window( AW_device **device,   ED4_window **new
     }
     awmm->close_sub_menu();
 
+    awmm->insert_help_topic("help_arb_edit", "ARB_EDIT4 help",     "E", "e4.hlp", AWM_ALL, (AW_CB)AW_POPUP_HELP, (AW_CL)"e4.hlp", 0);
+
     // ----------------------------------------------------------------------------------------------------
 
 #undef SEP________________________SEP
@@ -1712,104 +1720,119 @@ ED4_returncode ED4_root::generate_window( AW_device **device,   ED4_window **new
 
     int db_pathx,db_pathy;
     awmm->get_at_position( &db_pathx,&db_pathy );
-    awmm->callback( ED4_quit_editor, 0, 0);
 
     awmm->shadow_width(1);
     awmm->load_xfig("edit4/editmenu.fig", AW_FALSE);
 
-    // ----------------------------------------------------------------------------------------------------
-    awmm->button_length(7);
-    awmm->help_text("quit.hlp");
+    // -------------------------
+    //      help /quit /fold
+    // -------------------------
+    
+    awmm->button_length(0);
+    
     awmm->at("quit");
-    if (ED4_window::no_of_windows == 1)     awmm->create_button("QUIT","QUIT"); // this is the first window
-    else                    awmm->create_button("CLOSE","CLOSE");
+    awmm->callback(ED4_quit_editor, 0, 0);
+    awmm->help_text("quit.hlp");
+     if (ED4_window::no_of_windows == 1)  awmm->create_button("QUIT","#quit.xpm"); // this is the first window
+     else                                 awmm->create_button("CLOSE","#close.xpm");
 
     awmm->at("help");
-    awmm->callback(AW_POPUP_HELP,(AW_CL)"e4.hlp");
-    awmm->create_button("HELP","HELP");
+    awmm->callback(AW_help_entry_pressed);
+    awmm->help_text("e4.hlp");
+    awmm->create_button("HELP","#help.xpm");
+
+    awmm->at("fold");
+    awmm->help_text("e4.hlp");
+    awmm->create_toggle(AWAR_EDIT_TITLE_MODE, "#more.bitmap", "#less.bitmap");
+
+    // ------------------
+    //      positions
+    // ------------------
+    
+    awmm->button_length(0);
+    
+    awmm->at("posTxt");     awmm->create_button(0, "Position");
+
+    awmm->button_length(6+1);
+    
+    awmm->at("ecoliTxt");   awmm->create_button(0, ED4_AWAR_NDS_ECOLI_NAME, 0, "+");
 
     awmm->button_length(0);
-    awmm->at("fold");
-    awmm->create_toggle(AWAR_EDIT_TITLE_MODE, "edit/folded.bitmap", "edit/unfolded.bitmap");
-
-    // ----------------------------------------------------------------------------------------------------
-    int db_treex,db_treey;
-    awmm->get_at_position( &db_treex,&db_treey );
-    awmm->button_length(200);
+    
+    awmm->at("baseTxt");    awmm->create_button(0, "Base");
+    awmm->at("iupacTxt");   awmm->create_button(0, "IUPAC");
+    awmm->at("helixnrTxt"); awmm->create_button(0, "Helix#");
 
     awmm->at("pos");
-    awmm->callback((AW_CB)ED4_jump_to_cursor_position, (AW_CL) ED4_ROOT->temp_ed4w->awar_path_for_cursor, AW_CL(1));
-    awmm->create_input_field(ED4_ROOT->temp_ed4w->awar_path_for_cursor,8);
+    awmm->callback((AW_CB)ED4_jump_to_cursor_position, (AW_CL) get_ed4w()->awar_path_for_cursor, AW_CL(1));
+    awmm->create_input_field(get_ed4w()->awar_path_for_cursor,7);
 
     awmm->at("ecoli");
-    awmm->callback((AW_CB)ED4_jump_to_cursor_position,(AW_CL) ED4_ROOT->temp_ed4w->awar_path_for_Ecoli, AW_CL(0));
-    awmm->create_input_field(ED4_ROOT->temp_ed4w->awar_path_for_Ecoli,8);
+    awmm->callback((AW_CB)ED4_jump_to_cursor_position,(AW_CL) get_ed4w()->awar_path_for_Ecoli, AW_CL(0));
+    awmm->create_input_field(get_ed4w()->awar_path_for_Ecoli,6);
 
     awmm->at("base");
-    awmm->callback((AW_CB)ED4_jump_to_cursor_position,(AW_CL) ED4_ROOT->temp_ed4w->awar_path_for_basePos, AW_CL(0));
-    awmm->create_input_field(ED4_ROOT->temp_ed4w->awar_path_for_basePos,8);
+    awmm->callback((AW_CB)ED4_jump_to_cursor_position,(AW_CL) get_ed4w()->awar_path_for_basePos, AW_CL(0));
+    awmm->create_input_field(get_ed4w()->awar_path_for_basePos,6);
 
     awmm->at("iupac");
-    awmm->callback((AW_CB)ED4_set_iupac,(AW_CL) ED4_ROOT->temp_ed4w->awar_path_for_IUPAC, AW_CL(0));
-    awmm->create_input_field(ED4_ROOT->temp_ed4w->awar_path_for_IUPAC,5);
+    awmm->callback((AW_CB)ED4_set_iupac,(AW_CL) get_ed4w()->awar_path_for_IUPAC, AW_CL(0));
+    awmm->create_input_field(get_ed4w()->awar_path_for_IUPAC,4);
 
     awmm->at("helixnr");
-    awmm->callback((AW_CB)ED4_set_helixnr,(AW_CL) ED4_ROOT->temp_ed4w->awar_path_for_helixNr, AW_CL(0));
-    awmm->create_input_field(ED4_ROOT->temp_ed4w->awar_path_for_helixNr,6);
+    awmm->callback((AW_CB)ED4_set_helixnr,(AW_CL) get_ed4w()->awar_path_for_helixNr, AW_CL(0));
+    awmm->create_input_field(get_ed4w()->awar_path_for_helixNr,5);
 
-    // ----------------------------------------------------------------------------------------------------
-    awmm->button_length(8);
-    awmm->at("insert");
-    awmm->create_toggle(AWAR_INSERT_MODE, "edit/replace.bitmap", "edit/insert.bitmap");
-
-    awmm->button_length(8);
-    awmm->at("edit");
-    awmm->create_toggle(AWAR_EDIT_MODE, "edit/align.bitmap", "edit/noalign.bitmap");
-
-    awmm->button_length(8);
-    awmm->at("direct");
-    awmm->create_toggle(AWAR_EDIT_DIRECTION,"edit/r2l.bitmap","edit/l2r.bitmap");
-
-    awmm->at("cons");
-    awmm->create_toggle(ED4_AWAR_CONSENSUS_SHOW, "edit/nocons.bitmap", "edit/cons.bitmap");
-
-    awmm->at("num");
-    awmm->create_toggle(ED4_AWAR_DIGITS_AS_REPEAT, "edit/norepeat.bitmap", "edit/repeat.bitmap");
-
-    awmm->at("key");
-    awmm->create_toggle("key_mapping/enable", "edit/nokeymap.bitmap", "edit/keymap.bitmap");
-
-    // ----------------------------------------------------------------------------------------------------
-    awmm->button_length(0);
-
-#if defined(ARB_OPENGL)
-    awmm->at("rna3d");
-    awmm->callback((void(*)(AW_window*, AW_CL))ED4_RNA3D_Start, 0);
-    awmm->create_button("RNA3D", "#edit/rna3d.xpm");
-#endif // ARB_OPENGL
-
-    awmm->at("secedit");
-    awmm->callback((void(*)(AW_window*, AW_CL))ED4_SECEDIT_start, 0);
-    awmm->create_button("SECEDIT", "#edit/edit_sec.bitmap");
-
-    awmm->button_length(6);
-    awmm->at("undo");
-    awmm->callback(ED4_undo_redo, GB_UNDO_UNDO);
-    awmm->create_button("UNDO", "UNDO");
-
-    awmm->at("redo");
-    awmm->callback(ED4_undo_redo, GB_UNDO_REDO);
-    awmm->create_button("REDO", "REDO");
+    // ---------------------------
+    //      jump/get/undo/redo
+    // ---------------------------
 
+    awmm->button_length(4);
+    
     awmm->at("jump");
     awmm->callback(ED4_jump_to_current_species, 0);
-    awmm->create_button("JUMP", "JUMP");
+    awmm->help_text("e4.hlp");
+    awmm->create_button("JUMP", "Jump");
 
     awmm->at("get");
     awmm->callback(ED4_get_and_jump_to_actual, 0);
-    awmm->create_button("GET", "GET");
+    awmm->help_text("e4.hlp");
+    awmm->create_button("GET", "Get");
 
-    awmm->button_length(5);
+    awmm->button_length(0);
+    
+    awmm->at("undo");
+    awmm->callback(ED4_undo_redo, GB_UNDO_UNDO);
+    awmm->help_text("undo.hlp");
+    awmm->create_button("UNDO", "#undo.bitmap");
+
+    awmm->at("redo");
+    awmm->callback(ED4_undo_redo, GB_UNDO_REDO);
+    awmm->help_text("undo.hlp");
+    awmm->create_button("REDO", "#redo.bitmap");
+
+    // -------------------------
+    //      aligner / SAIviz
+    // -------------------------
+    
+    awmm->button_length(7);
+    
+    awmm->at("aligner");
+    awmm->callback(AW_POPUP, (AW_CL)ED4_create_faligner_window, (AW_CL)&faligner_client_data);
+    awmm->help_text("faligner.hlp");
+    awmm->create_button("ALIGNER", "Aligner");
+
+    awmm->at("saiviz");
+    awmm->callback(AW_POPUP,(AW_CL)ED4_createVisualizeSAI_window, 0 );
+    awmm->help_text("visualizeSAI.hlp");
+    awmm->create_button("SAIVIZ", "SAIviz");
+
+    // ------------------------------------------
+    //      align/insert/protection/direction
+    // ------------------------------------------
+
+    awmm->button_length(0);
+    
     awmm->at("protect");
     awmm->create_option_menu(AWAR_EDIT_SECURITY_LEVEL);
     awmm->insert_option("0",0,0);
@@ -1821,7 +1844,85 @@ ED4_returncode ED4_root::generate_window( AW_device **device,   ED4_window **new
     awmm->insert_default_option("6",0,6);
     awmm->update_option_menu();
 
+    // draw protection icon AFTER protection!!
+    awmm->at("pico");
+    awmm->create_button("PROTECT","#protect.xpm");
+
+    // draw align/edit-button AFTER protection!!
+    awmm->at("edit");
+    awmm->create_toggle(AWAR_EDIT_MODE, "edit/align.xpm", "edit/editseq.xpm", 7);
+
+    awmm->at("insert");
+    awmm->create_text_toggle(AWAR_INSERT_MODE, "Replace", "Insert", 7);
+
+    awmm->at("direct");
+    awmm->create_toggle(AWAR_EDIT_DIRECTION,"edit/3to5.bitmap","edit/5to3.bitmap", 7);
+
+    // ------------------------
+    //      secedit / rna3d
+    // ------------------------
+
+    int xoffset = 0;
+
+    // Enable RNA3D and SECEDIT programs if the database contains valid alignment of rRNA sequences.
+    if (alignment_type == GB_AT_RNA) {
+        awmm->button_length(0);
+        
+        awmm->at("secedit");
+        awmm->callback(ED4_SECEDIT_start, (AW_CL)gb_main, 0);
+        awmm->help_text("arb_secedit.hlp");
+        awmm->create_button("SECEDIT", "#edit/secedit.xpm");
+        
+#if defined(ARB_OPENGL)
+        awmm->at("rna3d");
+        awmm->callback(ED4_RNA3D_Start, 0, 0);
+        awmm->help_text("rna3d_general.hlp");
+        awmm->create_button("RNA3D", "#edit/rna3d.xpm");
+#endif // ARB_OPENGL
+    }
+    else {
+        awmm->at("secedit");
+        int xsecedit = awmm->get_at_xposition();
+        awmm->at("zoom");
+        int xzoom    = awmm->get_at_xposition();
+        xoffset      = xsecedit-xzoom; // shift message stuff to the left by xoffset
+    }
+
+    {
+        int x, y;
+
+        awmm->at("zoom");
+        if (xoffset) { awmm->get_at_position(&x, &y); awmm->at(x+xoffset, y); }
+        awmm->callback(AW_POPUP, (AW_CL)ED4_zoom_message_window, (AW_CL)0);
+        awmm->create_button("ZOOM", "#edit/zoom.bitmap");
+
+        awmm->at("clear");
+        if (xoffset) { awmm->get_at_position(&x, &y); awmm->at(x+xoffset, y); }
+        awmm->callback(ED4_clear_errors, (AW_CL)0);
+        awmm->create_button("CLEAR", "#edit/clear.bitmap");
+
+        awmm->at("errortext");
+        if (xoffset) { awmm->get_at_position(&x, &y); awmm->at(x+xoffset, y); }
+        aw_root->awar_string(AWAR_ERROR_MESSAGES, "This is ARB Edit4 [Build " ARB_VERSION "]");
+        awmm->create_text_field(AWAR_ERROR_MESSAGES);
+        aw_set_local_message();
+    }
+
+    // --------------------
+    //      'more' area
+    // --------------------
+
+    awmm->at("cons");
+    awmm->create_toggle(ED4_AWAR_CONSENSUS_SHOW, "edit/nocons.bitmap", "edit/cons.bitmap");
+
+    awmm->at("num");
+    awmm->create_toggle(ED4_AWAR_DIGITS_AS_REPEAT, "edit/norepeat.bitmap", "edit/repeat.bitmap");
+
+    awmm->at("key");
+    awmm->create_toggle("key_mapping/enable", "edit/nokeymap.bitmap", "edit/keymap.bitmap");
+
     // search
+    
     awmm->button_length(0);
 #define INSERT_SEARCH_FIELDS(Short, label_prefix, prefix) insert_search_fields(awmm, #label_prefix, #prefix, ED4_AWAR_##prefix##_SEARCH_PATTERN, ED4_##prefix##_PATTERN, ED4_AWAR_##prefix##_SEARCH_SHOW, Short)
 
@@ -1845,19 +1946,6 @@ ED4_returncode ED4_root::generate_window( AW_device **device,   ED4_window **new
     awmm->callback(ED4_search, ED4_encodeSearchDescriptor(+1, ED4_ANY_PATTERN));
     awmm->create_button("ALL_SEARCH_NEXT", "#edit/next.bitmap");
 
-    awmm->at("zoom");
-    awmm->callback(AW_POPUP, (AW_CL)ED4_zoom_message_window, (AW_CL)0);
-    awmm->create_button("ZOOM", "#edit/zoom.bitmap");
-
-    awmm->at("clear");
-    awmm->callback(ED4_clear_errors, (AW_CL)0);
-    awmm->create_button("CLEAR", "#edit/clear.bitmap");
-
-    awmm->at("errortext");
-    aw_root->awar_string(AWAR_ERROR_MESSAGES, "This is ARB Edit4 [Build " DATE "]");
-    awmm->create_text_field(AWAR_ERROR_MESSAGES);
-    aw_set_local_message();
-
     title_mode_changed(aw_root, awmm);
 
     // Buttons at left window border
@@ -1866,7 +1954,7 @@ ED4_returncode ED4_root::generate_window( AW_device **device,   ED4_window **new
     awmm->create_mode( 0, "edit/kill.bitmap",  "kill.hlp",   ((AW_active)-1), (AW_CB)modes_cb, (AW_CL)1, (AW_CL)0);
     awmm->create_mode( 0, "edit/mark.bitmap",  "mark.hlp",   ((AW_active)-1), (AW_CB)modes_cb, (AW_CL)2, (AW_CL)0);
 
-    AWTC_create_faligner_variables(awmm->get_root(), ED4_ROOT->db);
+    AWTC_create_faligner_variables(awmm->get_root(), db);
 
     return ( ED4_R_OK );
 }
@@ -1881,7 +1969,6 @@ AW_window *ED4_root::create_new_window( void )                      // only the
     ED4_calc_terminal_extentions();
 
     last_window_reached     = 1;
-    MARGIN                  = 5;
     DRAW                    = 1;
     move_cursor             = 0;
     all_found               = 0;
@@ -1911,28 +1998,30 @@ ED4_index ED4_root::pixel2pos(AW_pos click_x) {
 ED4_root::ED4_root()
 {
     memset((char *)this,0,sizeof(*this));
-    aw_root             = new AW_root;
-    first_window            = NULL;
-    main_manager            = NULL;
-    database                = NULL;
-    temp_ed4w               = NULL;
-    temp_aww            =    NULL;
-    temp_device         = NULL;
-    temp_gc             = 0;
-    scroll_links.link_for_hor_slider= NULL;
-    scroll_links.link_for_ver_slider= NULL;
-    folding_action          = 0;
-    //     ref_terminals.ref_sequence_info  = NULL;
-    //     ref_terminals.ref_sequence   = NULL;
+
+    aw_root      = new AW_root;
+    first_window = NULL;
+    main_manager = NULL;
+    database     = NULL;
+    tmp_ed4w     = NULL;
+    tmp_aww      = NULL;
+    tmp_device   = NULL;
+    temp_gc      = 0;
+
+    scroll_links.link_for_hor_slider = NULL;
+    scroll_links.link_for_ver_slider = NULL;
+
+    folding_action = 0;
+    
     species_mode            = ED4_SM_MOVE;
-    scroll_picture.scroll       = 0;
-    scroll_picture.old_x        = 0;
-    scroll_picture.old_y        = 0;
+    scroll_picture.scroll   = 0;
+    scroll_picture.old_x    = 0;
+    scroll_picture.old_y    = 0;
     middle_area_man         = NULL;
     top_area_man            = NULL;
     root_group_man          = NULL;
-    ecoli_ref           = NULL;
-    column_stat_activated = 0;
+    ecoli_ref               = NULL;
+    column_stat_activated   = 0;
     column_stat_initialized = 0;
     visualizeSAI            = 0;
     visualizeSAI_allSpecies = 0;
