@@ -16,6 +16,7 @@
 #include <awt.hxx>
 #include <awt_config_manager.hxx>
 
+#include <ed4_extern.hxx>
 #include "ed4_awars.hxx"
 #include "ed4_class.hxx"
 #include "ed4_tools.hxx"
@@ -99,21 +100,21 @@ public:
         }
     }
 
-    SearchSettings(SearchAwarList awarList) 	{ pattern = 0; update(awarList); }
-    ~SearchSettings() 				{ delete pattern; }
+    SearchSettings(SearchAwarList awarList)     { pattern = 0; update(awarList); }
+    ~SearchSettings()                           { delete pattern; }
 
-    GB_CSTR get_pattern() const 		{ return pattern; }
-    int get_min_mismatches() const 		{ return min_mismatches; }
-    int get_max_mismatches() const 		{ return max_mismatches; }
-    ED4_SEARCH_CASE get_case_sensitive() const	{ return case_sensitive; }
-    ED4_SEARCH_TU get_tu() const 		{ return tu; }
-    ED4_SEARCH_GAPS get_pat_gaps() const 	{ return pat_gaps; }
-    ED4_SEARCH_GAPS get_seq_gaps() const 	{ return seq_gaps; }
-    int get_open_folded() const 		{ return open_folded; }
-    int get_autoJump() const 			{ return autoJump; }
-    int get_reverse() const 			{ return reverse; }
-    int get_complement() const 			{ return complement; }
-    int get_exact() const 			{ return exact; }
+    GB_CSTR get_pattern() const                 { return pattern; }
+    int get_min_mismatches() const              { return min_mismatches; }
+    int get_max_mismatches() const              { return max_mismatches; }
+    ED4_SEARCH_CASE get_case_sensitive() const  { return case_sensitive; }
+    ED4_SEARCH_TU get_tu() const                { return tu; }
+    ED4_SEARCH_GAPS get_pat_gaps() const        { return pat_gaps; }
+    ED4_SEARCH_GAPS get_seq_gaps() const        { return seq_gaps; }
+    int get_open_folded() const                 { return open_folded; }
+    int get_autoJump() const                    { return autoJump; }
+    int get_reverse() const                     { return reverse; }
+    int get_complement() const                  { return complement; }
+    int get_exact() const                       { return exact; }
 };
 
 // --------------------------------------------------------------------------------
@@ -123,17 +124,17 @@ typedef void (*reportMatch)(int start, int end, GB_CSTR comment, int mismatches[
 
 class SearchTreeNode
 {
-    char c; 			// character
-    SearchTreeNode *son;	// son!=0 (exception: FOUND)
+    char            c;          // character
+    SearchTreeNode *son;        // son != 0 (exception: FOUND)
     SearchTreeNode *brother;
-    char *comment;		// 0 or comment given in search pattern
+    char           *comment;    // 0 or comment given in search pattern
 
-    static SearchTreeNode FOUND;
-    static int start_offset;
-    static reportMatch report;
-    static int min_mismatches;
-    static int max_mismatches;
-    static int *uni2real;	// transform unified positions to real sequence positions
+    static SearchTreeNode  FOUND;
+    static int             start_offset;
+    static reportMatch     report;
+    static int             min_mismatches;
+    static int             max_mismatches;
+    static int            *uni2real; // transform unified positions to real sequence positions
 
     SearchTreeNode(const SearchTreeNode &) { e4_assert(0); } // forbidden
 
@@ -147,19 +148,19 @@ public:
 
     // you must call the following functions before calling findMatches():
 
-    static void set_start_offset(int off) 		{ start_offset = off; }
-    static void set_report(reportMatch r, int *u2r)	{ report = r; uni2real = u2r; }
-    static void set_mismatches(int minMis, int maxMis) 	{ min_mismatches = minMis; max_mismatches = maxMis; }
+    static void set_start_offset(int off)               { start_offset = off; }
+    static void set_report(reportMatch r, int *u2r)     { report = r; uni2real = u2r; }
+    static void set_mismatches(int minMis, int maxMis)  { min_mismatches = minMis; max_mismatches = maxMis; }
 };
 
 // --------------------------------------------------------------------------------
 
-SearchTreeNode 	SearchTreeNode::FOUND(0, 0);
-int 		SearchTreeNode::start_offset;
-reportMatch 	SearchTreeNode::report;
-int 		SearchTreeNode::min_mismatches;
-int 		SearchTreeNode::max_mismatches;
-int 		*SearchTreeNode::uni2real;
+SearchTreeNode  SearchTreeNode::FOUND(0, 0);
+int             SearchTreeNode::start_offset;
+reportMatch     SearchTreeNode::report;
+int             SearchTreeNode::min_mismatches;
+int             SearchTreeNode::max_mismatches;
+int            *SearchTreeNode::uni2real;
 
 SearchTreeNode::SearchTreeNode(GB_CSTR pattern, GB_CSTR pattern_comment)
 {
@@ -372,7 +373,7 @@ SearchTree::SearchTree(const SearchSettings *s)
         }
     }
 
-#define ROOT(tok,com)                                       \
+#define INSERT_ROOT_PATTERN(tok,com)                        \
     do {                                                    \
         if (root) {                                         \
             root = root->insert_unified_pattern(tok, com);  \
@@ -384,12 +385,13 @@ SearchTree::SearchTree(const SearchSettings *s)
 
 
     {
-        char       *pattern      = strdup(sett->get_pattern());
-        const char *trenner      = "\n,";
-        char       *tok          = strtok(pattern, trenner);
+        char       *pattern           = strdup(sett->get_pattern());
+        const char *trenner           = "\n,";
+        char       *tok               = strtok(pattern, trenner);
         char       *comment;
         char        T_or_U;
-        GB_ERROR    T_or_U_error = GBT_determine_T_or_U(ED4_ROOT->alignment_type, &T_or_U, "complement");
+        GB_ERROR    T_or_U_error      = GBT_determine_T_or_U(ED4_ROOT->alignment_type, &T_or_U, "complement");
+        bool        show_T_or_U_error = false;
 
         while (tok) {
             splitTokComment(&tok, &comment);
@@ -406,63 +408,61 @@ SearchTree::SearchTree(const SearchSettings *s)
                 }
 
                 if (!s_exact || (!s_reverse && !s_complement)) {
-                    ROOT(uni_tok, comment);
+                    // insert original search pattern if all patterns shall be used (!s_exact)
+                    // or if neither s_reverse nor s_complement
+                    INSERT_ROOT_PATTERN(uni_tok, comment);
                 }
                 int commentLen = comment ? strlen(comment) : 0;
 
                 if (s_reverse) {
                     char *reverse = GBT_reverseNucSequence(uni_tok, uni_tok_len);
-                    char *reverseComment = appendComment(comment, commentLen, "(reverse)");
 
-                    if (!s_exact || (s_reverse && !s_complement)) {
-                        ROOT(reverse, reverseComment);
+                    if (!s_exact || !s_complement) {
+                        char *reverseComment = appendComment(comment, commentLen, "(reverse)");
+
+                        INSERT_ROOT_PATTERN(reverse, reverseComment); // insert reverse pattern
+                        free(reverseComment);
                     }
                     if (s_complement) {
                         e4_assert(IS_NUCLEOTIDE());
                         // char T_or_U = ED4_ROOT->alignment_type==GB_AT_DNA ? 'T' : 'U';
                         if (T_or_U) {
-                            char *revcomp = GBT_complementNucSequence(reverse, uni_tok_len, T_or_U);
+                            char *revcomp        = GBT_complementNucSequence(reverse, uni_tok_len, T_or_U);
                             char *revcompComment = appendComment(comment, commentLen, "(reverse complement)");
+                            char *uni_revcomp    = unify_pattern(revcomp, 0);
 
-                            if (!s_exact || (s_reverse && s_complement)) {
-                                ROOT(revcomp, revcompComment);
-                            }
+                            INSERT_ROOT_PATTERN(uni_revcomp, revcompComment); // insert reverse complement pattern
+
+                            free(uni_revcomp);
                             free(revcompComment);
                             free(revcomp);
                         }
                         else {
-                            if (T_or_U_error) {
-                                aw_message(T_or_U_error);
-                                T_or_U_error = 0; // only once
-                            }
+                            show_T_or_U_error = true; // only show error if it matters
                         }
                     }
 
-                    free(reverseComment);
                     free(reverse);
                 }
-                else if (s_complement) {
+                
+                if (s_complement) {
                     GB_transaction dummy(gb_main);
 
-//                     GB_alignment_type ali_type = GBT_get_alignment_type(gb_main, GBT_get_default_alignment(gb_main));
-//                     e4_assert(ali_type==GB_AT_DNA || ali_type==GB_AT_RNA);
-//                     char              T_or_U   = ali_type==GB_AT_DNA ? 'T' : 'U';
-
                     if (T_or_U) {
-                        char *complement        = GBT_complementNucSequence(uni_tok, uni_tok_len, T_or_U);
-                        char *complementComment = appendComment(comment, commentLen, "(complement)");
+                        if (!s_exact || !s_reverse) {
+                            char *complement        = GBT_complementNucSequence(uni_tok, uni_tok_len, T_or_U);
+                            char *complementComment = appendComment(comment, commentLen, "(complement)");
+                            char *uni_complement    = unify_pattern(complement, 0);
 
-                        if (!s_exact || (!s_reverse && s_complement)) {
-                            ROOT(complement, complementComment);
+                            INSERT_ROOT_PATTERN(uni_complement, complementComment); // insert complement pattern
+
+                            free(uni_complement);
+                            free(complementComment);
+                            free(complement);
                         }
-                        free(complementComment);
-                        free(complement);
                     }
                     else {
-                        if (T_or_U_error) {
-                            aw_message(T_or_U_error);
-                            T_or_U_error = 0; // only once
-                        }
+                        show_T_or_U_error = true; // only show error if it matters
                     }
                 }
 
@@ -472,9 +472,11 @@ SearchTree::SearchTree(const SearchSettings *s)
             free(uni_tok);
         }
         free(pattern);
+
+        if (show_T_or_U_error && T_or_U_error) aw_message(T_or_U_error);
     }
 
-#undef ROOT
+#undef INSERT_ROOT_PATTERN
 }
 
 SearchTree::~SearchTree()
@@ -516,23 +518,6 @@ char *SearchTree::unify_str(const char *data, int len, ED4_SEARCH_GAPS gaps, int
     int   nlen    = 0;
     int   realPos = 0;
 
-    // Original-Version:
-    //
-    //     while (realPos<len) {
-    // 	char c = *data++;
-    //
-    // 	if (gaps==ED4_SG_CONSIDER_GAPS || (c!='-' && c!='.')) { // use character in unified string?
-    // 	    *pp++ = unified[c];
-    // 	    if (uni2real) {
-    // 		(*uni2real)[nlen] = realPos;
-    // 	    }
-    // 	    nlen++;
-    // 	}
-    // 	realPos++;
-    //     }
-    //
-    // Optimized Version:
-
     if (uni2real) {
         int *u2r = *uni2real;
 
@@ -632,7 +617,7 @@ void SearchTree::findMatches(const char *seq, int len, reportMatch report)
 #define AWAR_NAME(t,s)  ED4_AWAR_##t##_SEARCH_##s
 
 #define AWAR_LIST(t)                            \
-AWAR_NAME(t,PATTERN),                           \
+    AWAR_NAME(t,PATTERN),                       \
     AWAR_NAME(t,MIN_MISMATCHES),                \
     AWAR_NAME(t,MAX_MISMATCHES),                \
     AWAR_NAME(t,CASE),                          \
@@ -658,24 +643,23 @@ static struct S_SearchAwarList awar_list[SEARCH_PATTERNS] = {
     { AWAR_LIST(SIG3) },
 };
 
-static inline int resultsAreShown(ED4_SearchPositionType type)
-{
+static inline int resultsAreShown(ED4_SearchPositionType type) {
     return ED4_ROOT->aw_root->awar(awar_list[type].show)->read_int();
 }
 
 enum search_params_changed_action {
     REFRESH_IF_SHOWN   = 1,
-    REFRESH_ALWAYS	   = 2,
+    REFRESH_ALWAYS     = 2,
     RECALC_SEARCH_TREE = 4,
     TEST_MIN_MISMATCH  = 8,
     TEST_MAX_MISMATCH  = 16,
-    DO_AUTO_JUMP	   = 32
+    DO_AUTO_JUMP       = 32
 };
 
 // --------------------------------------------------------------------------------
 
-static SearchSettings *settings[SEARCH_PATTERNS]; 	// last searched settings for each type
-static SearchTree *tree[SEARCH_PATTERNS]; 		// Search trees for each type
+static SearchSettings *settings[SEARCH_PATTERNS]; // last searched settings for each type
+static SearchTree     *tree[SEARCH_PATTERNS]; // Search trees for each type
 
 // --------------------------------------------------------------------------------
 
@@ -716,12 +700,17 @@ static void searchParamsChanged(AW_root *root, AW_CL cl_type, AW_CL cl_action)
     }
 
     if (action & (RECALC_SEARCH_TREE|TEST_MIN_MISMATCH|TEST_MAX_MISMATCH)) {
-        if (tree[type]->get_shortestPattern() < 4*root->awar(awar_list[type].max_mismatches)->read_int()) {
-            aw_message("Too many mismatches!");
-            int mami = tree[type]->get_shortestPattern()/4;
-            root->awar(awar_list[type].max_mismatches)->write_int(mami);
-            if (root->awar(awar_list[type].min_mismatches)->read_int() > mami) {
-                root->awar(awar_list[type].min_mismatches)->write_int(mami);
+        int patLen = tree[type]->get_shortestPattern();
+        int maxMis = root->awar(awar_list[type].max_mismatches)->read_int();
+
+        if (patLen < 3*maxMis) {
+            int maxMaxMis = tree[type]->get_shortestPattern()/3;
+
+            aw_message(GBS_global_string("Too many mismatches (patterns of length=%i allow only %i max. mismatches)", patLen, maxMaxMis));
+
+            root->awar(awar_list[type].max_mismatches)->write_int(maxMaxMis);
+            if (root->awar(awar_list[type].min_mismatches)->read_int() > maxMaxMis) {
+                root->awar(awar_list[type].min_mismatches)->write_int(maxMaxMis);
             }
             goto recalc;
         }
@@ -733,9 +722,9 @@ static void searchParamsChanged(AW_root *root, AW_CL cl_type, AW_CL cl_action)
         }
     }
 
-    if (settings[type]->get_autoJump() && (action & DO_AUTO_JUMP)) {		// auto jump
-        ED4_cursor *cursor = &ED4_ROOT->temp_ed4w->cursor;
-        int jumped = 0;
+    if (settings[type]->get_autoJump() && (action & DO_AUTO_JUMP)) { // auto jump
+        ED4_cursor *cursor = &ED4_ROOT->get_ed4w()->cursor;
+        bool jumped = false;
 
         if (cursor->owner_of_cursor && cursor->owner_of_cursor->is_sequence_terminal()) {
             int pos = cursor->get_sequence_pos();
@@ -762,9 +751,11 @@ static void searchParamsChanged(AW_root *root, AW_CL cl_type, AW_CL cl_action)
             }
 
             if (bestPos!=-1) {
-                jumped = 1;
-                if (bestPos!=pos) {
-                    seq_term->setCursorTo(cursor, bestPos, 1);
+                if (bestPos == pos) {
+                    jumped = true; // already there
+                }
+                else {
+                    jumped = seq_term->setCursorTo(cursor, bestPos, 1, ED4_JUMP_KEEP_POSITION);
                 }
             }
         }
@@ -777,9 +768,11 @@ static void searchParamsChanged(AW_root *root, AW_CL cl_type, AW_CL cl_action)
     if (action & REFRESH_ALWAYS) {
         bool old_update                        = ED4_update_global_cursor_awars_allowed;
         ED4_update_global_cursor_awars_allowed = false;
-        ED4_refresh_window(ED4_ROOT->temp_aww, 0, 0);
+        ED4_refresh_window(ED4_ROOT->get_aww(), 0, 0);
         ED4_update_global_cursor_awars_allowed = old_update;
     }
+
+    root->awar(ED4_AWAR_SEARCH_RESULT_CHANGED)->touch(); // trigger refresh in SECEDIT
 }
 
 void ED4_create_search_awars(AW_root *root)
@@ -788,29 +781,31 @@ void ED4_create_search_awars(AW_root *root)
 
     int i;
     for (i=0; i<SEARCH_PATTERNS; i++) {
-        root->awar_string(awar_list[i].pattern, "", gb_main) 				->cb(REFRESH_IF_SHOWN | RECALC_SEARCH_TREE | DO_AUTO_JUMP);
-        root->awar_int(awar_list[i].case_sensitive, ED4_SC_CASE_INSENSITIVE, gb_main)	->cb(REFRESH_IF_SHOWN | RECALC_SEARCH_TREE | DO_AUTO_JUMP);
-        root->awar_int(awar_list[i].tu, ED4_ST_T_EQUAL_U, gb_main)			->cb(REFRESH_IF_SHOWN | RECALC_SEARCH_TREE | DO_AUTO_JUMP);
-        root->awar_int(awar_list[i].pat_gaps, ED4_SG_IGNORE_GAPS, gb_main)		->cb(REFRESH_IF_SHOWN | RECALC_SEARCH_TREE | DO_AUTO_JUMP);
-        root->awar_int(awar_list[i].reverse, 0, gb_main)				->cb(REFRESH_IF_SHOWN | RECALC_SEARCH_TREE | DO_AUTO_JUMP);
-        root->awar_int(awar_list[i].complement, 0, gb_main)				->cb(REFRESH_IF_SHOWN | RECALC_SEARCH_TREE | DO_AUTO_JUMP);
-        root->awar_int(awar_list[i].exact, 0, gb_main)					->cb(REFRESH_IF_SHOWN | RECALC_SEARCH_TREE | DO_AUTO_JUMP);
-        root->awar_int(awar_list[i].min_mismatches, 0, gb_main)				->cb(REFRESH_IF_SHOWN | TEST_MAX_MISMATCH | DO_AUTO_JUMP);
-        root->awar_int(awar_list[i].max_mismatches, 0, gb_main)				->cb(REFRESH_IF_SHOWN | TEST_MIN_MISMATCH | DO_AUTO_JUMP);
-        root->awar_int(awar_list[i].seq_gaps, ED4_SG_IGNORE_GAPS, gb_main)		->cb(REFRESH_IF_SHOWN | DO_AUTO_JUMP);
-        root->awar_int(awar_list[i].show, 1, gb_main)					->cb(REFRESH_ALWAYS);
-        root->awar_int(awar_list[i].openFolded, 1, gb_main)				->cb(0);
-        root->awar_int(awar_list[i].autoJump, 1, gb_main)				->cb(DO_AUTO_JUMP);
+        root->awar_string(awar_list[i].pattern, "", gb_main)                            ->cb(REFRESH_IF_SHOWN | RECALC_SEARCH_TREE | DO_AUTO_JUMP);
+        root->awar_int(awar_list[i].case_sensitive, ED4_SC_CASE_INSENSITIVE, gb_main)   ->cb(REFRESH_IF_SHOWN | RECALC_SEARCH_TREE | DO_AUTO_JUMP);
+        root->awar_int(awar_list[i].tu, ED4_ST_T_EQUAL_U, gb_main)                      ->cb(REFRESH_IF_SHOWN | RECALC_SEARCH_TREE | DO_AUTO_JUMP);
+        root->awar_int(awar_list[i].pat_gaps, ED4_SG_IGNORE_GAPS, gb_main)              ->cb(REFRESH_IF_SHOWN | RECALC_SEARCH_TREE | DO_AUTO_JUMP);
+        root->awar_int(awar_list[i].reverse, 0, gb_main)                                ->cb(REFRESH_IF_SHOWN | RECALC_SEARCH_TREE | DO_AUTO_JUMP);
+        root->awar_int(awar_list[i].complement, 0, gb_main)                             ->cb(REFRESH_IF_SHOWN | RECALC_SEARCH_TREE | DO_AUTO_JUMP);
+        root->awar_int(awar_list[i].exact, 0, gb_main)                                  ->cb(REFRESH_IF_SHOWN | RECALC_SEARCH_TREE | DO_AUTO_JUMP);
+        root->awar_int(awar_list[i].min_mismatches, 0, gb_main)                         ->cb(REFRESH_IF_SHOWN | TEST_MAX_MISMATCH | DO_AUTO_JUMP);
+        root->awar_int(awar_list[i].max_mismatches, 0, gb_main)                         ->cb(REFRESH_IF_SHOWN | TEST_MIN_MISMATCH | DO_AUTO_JUMP);
+        root->awar_int(awar_list[i].seq_gaps, ED4_SG_IGNORE_GAPS, gb_main)              ->cb(REFRESH_IF_SHOWN | DO_AUTO_JUMP);
+        root->awar_int(awar_list[i].show, 1, gb_main)                                   ->cb(REFRESH_ALWAYS);
+        root->awar_int(awar_list[i].openFolded, 1, gb_main)                             ->cb(0);
+        root->awar_int(awar_list[i].autoJump, 1, gb_main)                               ->cb(DO_AUTO_JUMP);
 
         settings[i] = new SearchSettings(&awar_list[i]);
         tree[i] = new SearchTree(settings[i]);
     }
 
+    root->awar_int(ED4_AWAR_SEARCH_RESULT_CHANGED, 0, gb_main);
+
 #undef cb
 
     // awars to save/load search parameters:
     {
-        char *dir = GBS_global_string_copy("%s/lib/search_settings", GB_getenvARBHOME());
+        char *dir = GBS_global_string_copy("%s/.arb_prop/search_settings", GB_getenvHOME());
         aw_create_selection_box_awars(root, ED4_SEARCH_SAVE_BASE, dir, ".asp", "noname.asp");
         root->awar(ED4_SEARCH_SAVE_BASE"/directory")->write_string(dir);
         free(dir);
@@ -885,20 +880,6 @@ ED4_SearchPosition *ED4_SearchPosition::remove(ED4_SearchPositionType typeToRemo
     }
 
     return head;
-
-//     if (whatsFound==typeToRemove) {
-//         ED4_SearchPosition *rest = next ? next->remove(typeToRemove) : 0;
-
-//         next = 0;
-//         delete this;
-
-//         return rest;
-//     }
-
-//     if (next) {
-//         next = next->remove(typeToRemove);
-//     }
-//     return this;
 }
 
 #ifdef TEST_SEARCH_POSITION
@@ -923,7 +904,7 @@ int ED4_SearchPosition::ok() const
 GB_CSTR ED4_SearchPosition::get_comment() const
 {
     if (!comment) return 0;
-    if (lastShownComment && strcmp(lastShownComment, comment)==0) return 0;	// do not show comment twice
+    if (lastShownComment && strcmp(lastShownComment, comment)==0) return 0; // do not show comment twice
 
     delete lastShownComment;
     lastShownComment = strdup(comment);
@@ -940,24 +921,15 @@ ED4_SearchPosition *ED4_SearchPosition::get_next_at(int pos) const
         self = self->next;
     }
     return 0;
-
-
-//     if (!next || next->start_pos>pos) {
-//         return 0;
-//     }
-//     if (next->containsPos(pos)) {
-//         return next;
-//     }
-//     return next->get_next_at(pos);
 }
 
 // --------------------------------------------------------------------------------
 
-int ED4_SearchResults::initialized = 0;
-int ED4_SearchResults::timeOfLastSearch[SEARCH_PATTERNS];
-int ED4_SearchResults::timeOfNextSearch[SEARCH_PATTERNS];
-int ED4_SearchResults::shown[SEARCH_PATTERNS];
-int ED4_SearchResults::bufferSize;
+int   ED4_SearchResults::initialized = 0;
+int   ED4_SearchResults::timeOfLastSearch[SEARCH_PATTERNS];
+int   ED4_SearchResults::timeOfNextSearch[SEARCH_PATTERNS];
+int   ED4_SearchResults::shown[SEARCH_PATTERNS];
+int   ED4_SearchResults::bufferSize;
 char *ED4_SearchResults::buffer;
 
 ED4_SearchResults::ED4_SearchResults()
@@ -993,8 +965,8 @@ ED4_SearchResults::~ED4_SearchResults() {
 
 // --------------------------------------------------------------------------------
 
-static ED4_SearchResults *reportToResult = 0;
-static ED4_SearchPositionType reportType;
+static ED4_SearchResults      *reportToResult = 0;
+static ED4_SearchPositionType  reportType;
 
 static void reportSearchPosition(int start, int end, GB_CSTR comment, int mismatches[MAX_MISMATCHES])
 {
@@ -1034,7 +1006,7 @@ void ED4_SearchResults::addSearchPosition(ED4_SearchPosition *pos)
     }
 }
 
-void ED4_SearchResults::search(ED4_sequence_terminal *seq_terminal)
+void ED4_SearchResults::search(const ED4_sequence_terminal *seq_terminal)
 {
     int i;
     int needed[SEARCH_PATTERNS];
@@ -1200,7 +1172,7 @@ void ED4_SearchResults::searchAgain()
     }
 }
 
-char *ED4_SearchResults::buildColorString(ED4_sequence_terminal *seq_terminal, int start, int end)
+char *ED4_SearchResults::buildColorString(const ED4_sequence_terminal *seq_terminal, int start, int end) 
     // builds a char buffer (access is only valid from result[start] till result[end])
 {
     int i;
@@ -1235,9 +1207,6 @@ char *ED4_SearchResults::buildColorString(ED4_sequence_terminal *seq_terminal, i
     // search first search-result that goes in between start-end
 
     ED4_SearchPosition *pos = get_first_at(ED4_ANY_PATTERN, start, end);
-    //     while (pos && pos->get_end_pos()<start) {
-    // 	pos = pos->get_next();
-    //     }
     e4_assert(!pos || (pos->get_start_pos()<=end && pos->get_end_pos()>=start));
 
     int correct_neg_values = 0;
@@ -1284,8 +1253,12 @@ char *ED4_SearchResults::buildColorString(ED4_sequence_terminal *seq_terminal, i
     return buffer-start;
 }
 
+const char *ED4_buildColorString(const ED4_sequence_terminal *seq_terminal, int start, int end) {
+    return seq_terminal->results().buildColorString(seq_terminal, start, end);
+}
+
 ED4_SearchPosition *ED4_SearchResults::get_shown_at(int pos) const
-    // used by ED4_cursor::updateAwars to get search-comment
+// used by ED4_cursor::updateAwars to get search-comment
 {
     ED4_SearchPosition *p = get_last_starting_before(ED4_ANY_PATTERN, pos, 0); // @@@ tofix: should find last_ending before
     ED4_SearchPosition *best = 0;
@@ -1379,10 +1352,10 @@ void ED4_search(AW_window */*aww*/, AW_CL searchDescriptor)
 
     // detect position where to start searching
 
-    ED4_terminal *terminal = 0;	// start search at this terminal
+    ED4_terminal *terminal = 0; // start search at this terminal
     int pos; // ... and at this position
 
-    ED4_cursor *cursor = &ED4_ROOT->temp_ed4w->cursor;
+    ED4_cursor *cursor = &ED4_ROOT->get_ed4w()->cursor;
     if (cursor->owner_of_cursor) { // if cursor is shown -> use cursor position
         terminal = cursor->owner_of_cursor->to_terminal();
         pos = cursor->get_sequence_pos();
@@ -1405,12 +1378,10 @@ void ED4_search(AW_window */*aww*/, AW_CL searchDescriptor)
     while (terminal) {
         if (terminal->is_sequence_terminal()) {
             ED4_sequence_terminal *seq_terminal = terminal->to_sequence_terminal();
-            ED4_SearchResults& results = seq_terminal->results();
-            ED4_SearchPosition *found = 0;
+            ED4_SearchResults&     results      = seq_terminal->results();
+            ED4_SearchPosition    *found        = 0;
 
             if (pattern==ED4_ANY_PATTERN || settings[pattern]->get_open_folded() || !terminal->is_in_folded_group()) {
-                //		int done = 0;
-
                 results.search(seq_terminal);
 
                 if (direction==1) {
@@ -1431,7 +1402,7 @@ void ED4_search(AW_window */*aww*/, AW_CL searchDescriptor)
                         }
                     }
                     else {
-                        seq_terminal->setCursorTo(&ED4_ROOT->temp_ed4w->cursor, pos, 1);
+                        seq_terminal->setCursorTo(&ED4_ROOT->get_ed4w()->cursor, pos, true, ED4_JUMP_KEEP_POSITION);
                     }
                     break;
                 }
@@ -1464,6 +1435,39 @@ void ED4_search(AW_window */*aww*/, AW_CL searchDescriptor)
     }
 }
 
+void ED4_mark_matching_species(AW_window */*aww*/, AW_CL cl_pattern) {
+    ED4_SearchPositionType  pattern  = ED4_SearchPositionType(cl_pattern);
+    ED4_terminal           *terminal = ED4_ROOT->root_group_man->get_first_terminal();
+    GB_ERROR                error    = 0;
+    GB_transaction          ta(gb_main);
+
+    while (terminal && !error) {
+        if (terminal->is_sequence_terminal()) {
+            ED4_sequence_terminal *seq_terminal = terminal->to_sequence_terminal();
+            ED4_SearchResults&     results      = seq_terminal->results();
+
+            results.search(seq_terminal);
+            ED4_SearchPosition *found = results.get_first_starting_after(pattern, INT_MIN, false);
+
+            if (found) {
+                ED4_species_manager *species_man = seq_terminal->get_parent(ED4_L_SPECIES)->to_species_manager();;
+                if (!species_man->flag.is_consensus) {
+                    GBDATA *gbd = species_man->get_species_pointer();
+                    e4_assert(gbd);
+                    error = GB_write_flag(gbd, 1);
+                }            
+            }
+        }
+
+        terminal = terminal->get_next_terminal();
+    }
+
+    if (error) {
+        aw_message(error);
+        ta.abort();
+    }
+}
+
 #define ESC '\\'
 
 static char *pattern2str(GB_CSTR p) {
@@ -1620,17 +1624,17 @@ static void load_search_paras_from_file(AW_window *aw, AW_CL cl_type) {
         else {
             int value = atoi(content);
 
-            if (strcmp(buffer, "minmis")==0)		root->awar(al->min_mismatches)->write_int(value);
-            else if (strcmp(buffer, "maxmis")==0)	root->awar(al->max_mismatches)->write_int(value);
-            else if (strcmp(buffer, "case")==0)		root->awar(al->case_sensitive)->write_int(value);
-            else if (strcmp(buffer, "tu")==0)		root->awar(al->tu)->write_int(value);
-            else if (strcmp(buffer, "patgaps")==0)	root->awar(al->pat_gaps)->write_int(value);
-            else if (strcmp(buffer, "seqgaps")==0)	root->awar(al->seq_gaps)->write_int(value);
-            else if (strcmp(buffer, "openfolded")==0)	root->awar(al->openFolded)->write_int(value);
-            else if (strcmp(buffer, "autojump")==0)	root->awar(al->autoJump)->write_int(value);
-            else if (strcmp(buffer, "reverse")==0)	root->awar(al->reverse)->write_int(value);
-            else if (strcmp(buffer, "complement")==0)	root->awar(al->complement)->write_int(value);
-            else if (strcmp(buffer, "exact")==0)	root->awar(al->exact)->write_int(value);
+            if (strcmp(buffer, "minmis")==0)            root->awar(al->min_mismatches)->write_int(value);
+            else if (strcmp(buffer, "maxmis")==0)       root->awar(al->max_mismatches)->write_int(value);
+            else if (strcmp(buffer, "case")==0)         root->awar(al->case_sensitive)->write_int(value);
+            else if (strcmp(buffer, "tu")==0)           root->awar(al->tu)->write_int(value);
+            else if (strcmp(buffer, "patgaps")==0)      root->awar(al->pat_gaps)->write_int(value);
+            else if (strcmp(buffer, "seqgaps")==0)      root->awar(al->seq_gaps)->write_int(value);
+            else if (strcmp(buffer, "openfolded")==0)   root->awar(al->openFolded)->write_int(value);
+            else if (strcmp(buffer, "autojump")==0)     root->awar(al->autoJump)->write_int(value);
+            else if (strcmp(buffer, "reverse")==0)      root->awar(al->reverse)->write_int(value);
+            else if (strcmp(buffer, "complement")==0)   root->awar(al->complement)->write_int(value);
+            else if (strcmp(buffer, "exact")==0)        root->awar(al->exact)->write_int(value);
             else {
                 aw_message(GBS_global_string("Unknown tag '%s' in '%s' - ignored!", buffer, filename));
             }
@@ -1758,6 +1762,10 @@ AW_window *ED4_create_search_window(AW_root *root, AW_CL cl) {
     aws->callback(ED4_search, (AW_CL)ED4_encodeSearchDescriptor(-1, type));
     aws->create_button("SEARCH_LAST", "#edit/last.bitmap", "L");
 
+    aws->at("mark");
+    aws->callback(ED4_mark_matching_species, (AW_CL)type);
+    aws->create_autosize_button("MARK_SPECIES", "Mark species with matches", "M");
+
     aws->at("show");
     aws->create_toggle(awarList->show);
 
@@ -1810,9 +1818,9 @@ AW_window *ED4_create_search_window(AW_root *root, AW_CL cl) {
 
 static int has_species_name(ED4_base *base, AW_CL cl_species_name) {
     if (base->is_sequence_terminal()) {
-	ED4_sequence_terminal *seq_term = base->to_sequence_terminal();
-	const char *species_name = (const char *)cl_species_name;
-	return species_name && seq_term && seq_term->species_name && strcmp(species_name, seq_term->species_name)==0;
+        ED4_sequence_terminal *seq_term = base->to_sequence_terminal();
+        const char *species_name = (const char *)cl_species_name;
+        return species_name && seq_term && seq_term->species_name && strcmp(species_name, seq_term->species_name)==0;
     }
     return 0;
 }
