@@ -21,14 +21,21 @@
 #include <awt_attributes.hxx>
 #include <st_window.hxx>
 
+#include <ed4_extern.hxx>
+
 #include "ed4_class.hxx"
 #include "ed4_awars.hxx"
 #include "ed4_edit_string.hxx"
 #include "ed4_block.hxx"
 #include "ed4_nds.hxx"
 #include "ed4_visualizeSAI.hxx"
+#include "ed4_ProteinViewer.hxx"
 #include "ed4_protein_2nd_structure.hxx"
 
+int ED4_consensus_sequence_terminal::get_length() const {
+    return get_char_table().size();
+}
+
 ED4_returncode ED4_consensus_sequence_terminal::draw( int /*only_text*/ )
 {
     AW_pos x, y;
@@ -38,8 +45,8 @@ ED4_returncode ED4_consensus_sequence_terminal::draw( int /*only_text*/ )
     static char   *buffer      = 0;
     static size_t  buffer_size = 0;
 
-    calc_world_coords( &x, &y );
-    ED4_ROOT->world_to_win_coords( ED4_ROOT->temp_aww, &x, &y );
+    calc_world_coords(&x, &y);
+    ED4_ROOT->world_to_win_coords(ED4_ROOT->get_aww(), &x, &y);
     calc_update_intervall(&left_index, &right_index );
 
     text_x = x + CHARACTEROFFSET; // don't change
@@ -49,7 +56,7 @@ ED4_returncode ED4_consensus_sequence_terminal::draw( int /*only_text*/ )
     int paint_back = 1;
     if (paint_back) {
         paint_back = 3-paint_back;
-        ED4_ROOT->temp_device->box(ED4_G_FIRST_COLOR_GROUP+paint_back-1, x, y, extension.size[WIDTH], extension.size[HEIGHT], -1, 0, 0);
+        ED4_ROOT->get_device()->box(ED4_G_FIRST_COLOR_GROUP+paint_back-1, x, y, extension.size[WIDTH], extension.size[HEIGHT], -1, 0, 0);
     }
 #endif // DEBUG
     
@@ -80,7 +87,7 @@ ED4_returncode ED4_consensus_sequence_terminal::draw( int /*only_text*/ )
         }
 
         char *cons = GB_give_buffer(seq_end+1);
-        cons = get_parent( ED4_L_GROUP )->to_group_manager()->table().build_consensus_string(seq_start, seq_end, cons);
+        cons = get_char_table().build_consensus_string(seq_start, seq_end, cons);
 
         if ( size_t(right_index) >= buffer_size){
             delete [] buffer;
@@ -109,13 +116,13 @@ ED4_returncode ED4_consensus_sequence_terminal::draw( int /*only_text*/ )
 
 
     if (buffer_size) {
-        ED4_ROOT->temp_device->top_font_overlap    = 1;
-        ED4_ROOT->temp_device->bottom_font_overlap = 1;
+        ED4_ROOT->get_device()->top_font_overlap    = 1;
+        ED4_ROOT->get_device()->bottom_font_overlap = 1;
     
-        ED4_ROOT->temp_device->text( ED4_G_SEQUENCES, buffer, text_x, text_y, 0, 1, 0, 0, (long) right_index+1);
+        ED4_ROOT->get_device()->text( ED4_G_SEQUENCES, buffer, text_x, text_y, 0, 1, 0, 0, (long) right_index+1);
         
-        ED4_ROOT->temp_device->top_font_overlap    = 0;
-        ED4_ROOT->temp_device->bottom_font_overlap = 0;
+        ED4_ROOT->get_device()->top_font_overlap    = 0;
+        ED4_ROOT->get_device()->bottom_font_overlap = 0;
     }
 
     return ( ED4_R_OK );
@@ -125,23 +132,23 @@ int ED4_show_helix_on_device(AW_device *device, int gc, const char *opt_string,
                              AW_pos x,AW_pos y, AW_pos opt_ascent,AW_pos opt_descent,
                              AW_CL cduser, AW_CL real_sequence_length, AW_CL cd2){
     AWUSE(opt_ascent);AWUSE(opt_descent);AWUSE(opt_string_size);
-    AW_helix *THIS = (AW_helix *)cduser;
+    AW_helix *helix = (AW_helix *)cduser;
     const ED4_remap *rm = ED4_ROOT->root_group_man->remap();
     char *buffer = GB_give_buffer(size+1);
     register long i,j,k;
 
     for (k=0; size_t(k)<size; k++) {
         i = rm->screen_to_sequence(k+start);
-        if ( size_t(i)<THIS->size && THIS->entries[i].pair_type) {
-            j = THIS->entries[i].pair_pos;
-            char pairing_character = '.';
-            if (j < real_sequence_length){
-                pairing_character = opt_string[j];
-            }
-            buffer[k] = THIS->get_symbol(opt_string[i],pairing_character, THIS->entries[i].pair_type);
-        }else{
+
+        BI_PAIR_TYPE pairType = helix->pairtype(i);
+        if (pairType == HELIX_NONE) {
             buffer[k] = ' ';
         }
+        else {
+            j             = helix->opposite_position(i);
+            char pairchar = j<real_sequence_length ? opt_string[j] : '.';
+            buffer[k]     = helix->get_symbol(opt_string[i], pairchar, pairType);
+        }
     }
     buffer[size] = 0;
     return device->text(gc,buffer,x,y,0.0,(AW_bitset)-1,0,cd2);
@@ -153,10 +160,10 @@ struct show_summary_parameters {
 };
 
 
-int ED4_show_summary_match_on_device(AW_device *device, int gc, const char *opt_string, size_t opt_string_size, size_t start, size_t size,
-									 AW_pos x,AW_pos y, AW_pos opt_ascent,AW_pos opt_descent,
-									 AW_CL cd_summary_paramters, AW_CL real_sequence_length, AW_CL cd2){
-    AWUSE(opt_ascent);AWUSE(opt_descent);AWUSE(opt_string_size);
+int ED4_show_summary_match_on_device(AW_device *device, int gc, const char */*opt_string*/, size_t /*opt_string_size*/, size_t start, size_t size,
+                                     AW_pos x,AW_pos y, AW_pos /*opt_ascent*/,AW_pos /*opt_descent*/,
+                                     AW_CL cd_summary_paramters, AW_CL /*real_sequence_length*/, AW_CL cd2)
+{
     show_summary_parameters *ssp    = (show_summary_parameters*)cd_summary_paramters;
     const ED4_remap         *rm     = ED4_ROOT->root_group_man->remap();
     char                    *buffer = GB_give_buffer(size+1);
@@ -175,7 +182,151 @@ int ED4_show_summary_match_on_device(AW_device *device, int gc, const char *opt_
     return device->text(gc,buffer,x,y,0.0,(AW_bitset)-1,0,cd2);
 }
 
+//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+//  ProteinViewer: Drawing AminoAcid sequence parallel to the DNA sequence
+//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+#define is_upper(c) ('A'<=(c) && (c)<='Z')
 
+ED4_returncode ED4_AA_sequence_terminal::draw( int /*only_text*/ )
+{
+    AW_pos        text_x, text_y;
+    int           max_seq_len            = 0;
+    static int    color_is_used[ED4_G_DRAG];
+    static char **colored_strings        = 0;
+    static int    len_of_colored_strings = 0;
+    AW_device    *device                 = ED4_ROOT->get_device();
+
+    max_seq_len = strlen(this->aaSequence);
+
+    AW_pos world_x, world_y;
+
+    calc_world_coords(&world_x, &world_y);
+    ED4_ROOT->world_to_win_coords(ED4_ROOT->get_aww(), &world_x, &world_y);
+
+    text_x    = world_x + CHARACTEROFFSET; // don't change
+    text_y    = world_y + SEQ_TERM_TEXT_YOFFSET;
+
+    const ED4_remap *rm = ED4_ROOT->root_group_man->remap();
+
+    long left,right;
+    calc_update_intervall(&left, &right );
+    rm->clip_screen_range(&left, &right);
+
+    {
+        int max_seq_pos = rm->sequence_to_screen_clipped(max_seq_len-1);
+
+        if (right>max_seq_len) right = max_seq_pos;
+        if (left>right) {
+            const char *no_data = "No sequence data";
+            size_t      len     = strlen(no_data);
+
+            device->text(ED4_G_STANDARD, no_data, text_x, text_y, 0, 1, 0, 0, len);
+            return ED4_R_OK;
+        }
+    }
+
+    if (right >= len_of_colored_strings){
+        len_of_colored_strings = right + 256;
+        if (!colored_strings) {
+            colored_strings = (char **)GB_calloc(sizeof(char *),ED4_G_DRAG);
+        }
+        int i;
+        for (i=0;i<ED4_G_DRAG;i++){
+            free(colored_strings[i]);
+            colored_strings[i] = (char *)malloc(sizeof(char) * (len_of_colored_strings+1));
+
+            memset(colored_strings[i],' ',len_of_colored_strings);
+            colored_strings[i][len_of_colored_strings] = 0;
+        }
+    }
+
+    // int seq_start = rm->screen_to_sequence(left); // real start of sequence
+    int seq_end = rm->screen_to_sequence(right);
+
+    // mark all strings as unused
+    memset(color_is_used,0,sizeof(color_is_used));
+
+    //    int    iDisplayAminoAcids = ED4_ROOT->aw_root->awar(AWAR_PROTVIEW_DISPLAY_AA)->read_int();
+    int iDisplayMode = ED4_ROOT->aw_root->awar(AWAR_PROTVIEW_DISPLAY_OPTIONS)->read_int();
+    unsigned char *aaSequence = (unsigned char *) strdup(this->aaSequence);
+
+    {     // transform strings, compress if needed
+        AWT_reference *ref        = ED4_ROOT->reference;
+        ref->expand_to_length(seq_end);
+        char *char_2_char = ED4_ROOT->sequence_colors->char_2_char_aa;
+        char *char_2_gc   = ED4_ROOT->sequence_colors->char_2_gc_aa;
+        int scr_pos;
+
+        for (scr_pos=left; scr_pos <= right; scr_pos++) {
+            int seq_pos = rm->screen_to_sequence(scr_pos);
+            int c = aaSequence[seq_pos];
+            int gc = char_2_gc[c];
+
+            color_is_used[gc] = scr_pos+1;
+            colored_strings[gc][scr_pos] = char_2_char[aaSequence[seq_pos]];
+        }
+    }
+
+    // painting background
+    if((iDisplayMode == PV_AA_CODE) || (iDisplayMode == PV_AA_BOX)) {
+        {
+            AW_pos       width = ED4_ROOT->font_group.get_width(ED4_G_HELIX);
+            int          real_left  = left;
+            int        real_right  = right;
+            AW_pos           x2  = text_x + width*real_left;
+            AW_pos            x1 = x2;
+            AW_pos           y1  = world_y;
+            AW_pos           y2  = text_y+1;
+            AW_pos       height = y2-y1+1;
+            int                color  = ED4_G_STANDARD;
+            char *char_2_gc_aa = ED4_ROOT->sequence_colors->char_2_gc_aa;
+
+            int i;
+            for ( i = real_left; i <= real_right; i++,x2 += width) {
+                int new_pos = rm->screen_to_sequence(i);  //getting the real position of the base in the sequence
+                int       c = aaSequence[new_pos];
+                char    base = aaSequence[new_pos];
+                if (is_upper(base) || (base=='*')) {
+                    x1   = x2-width;         // store current x pos to x1
+                    x2 += width*2; // add 2 char width to x2
+                    i    += 2;         //jump two pos 
+
+                    int gcChar =  char_2_gc_aa[c];
+                    if ((gcChar>=0) && (gcChar<ED4_G_DRAG)) {
+                        color = gcChar;  
+                        if (iDisplayMode == PV_AA_BOX) {
+                            device->box(color, AW_TRUE, x1, y1, width*3, height, -1, 0,0);
+                        }
+                        else {
+                            device->arc(ED4_G_STANDARD, false, x1+(width*3)/2, y1, (width*3)/2, height/2, 0, -180, -1, 0, 0);
+                        }
+                    }
+                }
+            }
+        }
+    }
+    free(aaSequence);
+
+    device->top_font_overlap    = 1;
+    device->bottom_font_overlap = 1;
+
+    // output strings -- when display aminoacid sequence selected
+    if((iDisplayMode == PV_AA_NAME) || (iDisplayMode == PV_AA_CODE)) {
+        //    if (!iDisplayAsColoredBox || iDisplayAminoAcids) { 
+        int gc;
+        for (gc = 0; gc < ED4_G_DRAG; gc++){
+            if (!color_is_used[gc]) continue;
+            if ((int)strlen(colored_strings[gc])>=color_is_used[gc]) {
+                device->text( gc, colored_strings[gc], text_x, text_y, 0, 1, 0, 0, color_is_used [gc]);
+                memset(colored_strings[gc] + left,' ', right-left+1); // clear string
+            }
+        }
+    }
+    device->top_font_overlap    = 0;
+    device->bottom_font_overlap = 0;
+
+    return ( ED4_R_OK );
+}
 
 ED4_returncode ED4_sequence_terminal::draw( int /*only_text*/ )
 {
@@ -184,9 +335,10 @@ ED4_returncode ED4_sequence_terminal::draw( int /*only_text*/ )
     static int    color_is_used[ED4_G_DRAG];
     static char **colored_strings        = 0;
     static int    len_of_colored_strings = 0;
-    AW_device    *device                 = ED4_ROOT->temp_device;
+    AW_device    *device                 = ED4_ROOT->get_device();
 
     resolve_pointer_to_char_pntr(&max_seq_len);
+    e4_assert(max_seq_len>0);
 
 #if defined(DEBUG) && 0
     int paint_back = 0;
@@ -197,8 +349,8 @@ ED4_returncode ED4_sequence_terminal::draw( int /*only_text*/ )
 
     AW_pos world_x, world_y;
 
-    calc_world_coords( &world_x, &world_y );
-    ED4_ROOT->world_to_win_coords( ED4_ROOT->temp_aww, &world_x, &world_y );
+    calc_world_coords(&world_x, &world_y);
+    ED4_ROOT->world_to_win_coords(ED4_ROOT->get_aww(), &world_x, &world_y);
 
 #if defined(DEBUG) && 0
     int paint_back = 1;
@@ -257,8 +409,11 @@ ED4_returncode ED4_sequence_terminal::draw( int /*only_text*/ )
         unsigned char *db_pointer = (unsigned char *)resolve_pointer_to_string_copy();
 
         ref->expand_to_length(seq_end);
-        char *char_2_char = ED4_ROOT->sequence_colors->char_2_char;
-        char *char_2_gc = ED4_ROOT->sequence_colors->char_2_gc;
+
+        GB_alignment_type aliType = GetAliType();
+        char *char_2_char = (aliType && (aliType==GB_AT_AA))? ED4_ROOT->sequence_colors->char_2_char_aa:ED4_ROOT->sequence_colors->char_2_char;
+        char *char_2_gc   = (aliType && (aliType==GB_AT_AA))? ED4_ROOT->sequence_colors->char_2_gc_aa:ED4_ROOT->sequence_colors->char_2_gc;
+
         int scr_pos;
         int is_ref = ref->reference_species_is(species_name);
 
@@ -301,7 +456,7 @@ ED4_returncode ED4_sequence_terminal::draw( int /*only_text*/ )
             !spec_man->flag.is_SAI                           &&
             (is_marked || ED4_ROOT->visualizeSAI_allSpecies))
         {
-            saiColors = getSaiColorString(ED4_ROOT->aw_root, seq_start, seq_end);
+            saiColors = ED4_getSaiColorString(ED4_ROOT->aw_root, seq_start, seq_end);
         }
 
         if (colors || searchColors || is_marked || is_selected || color_group || saiColors) {
@@ -351,7 +506,7 @@ ED4_returncode ED4_sequence_terminal::draw( int /*only_text*/ )
                 if (color != old_color) {   // draw till oldcolor
                     if (x2>old_x){
                         if (old_color!=ED4_G_STANDARD) {
-                            device->box(old_color,old_x, y1, x2-old_x, height, -1, 0,0); // paints the search pattern background
+                            device->box(old_color, AW_TRUE, old_x, y1, x2-old_x, height, -1, 0,0); // paints the search pattern background
                         }
                     }
                     old_x = x2;
@@ -361,7 +516,7 @@ ED4_returncode ED4_sequence_terminal::draw( int /*only_text*/ )
 
             if (x2>old_x){
                 if (color!=ED4_G_STANDARD) {
-                    device->box(color,old_x, y1, x2-old_x, height, -1, 0,0);
+                    device->box(color, AW_TRUE, old_x, y1, x2-old_x, height, -1, 0,0);
                 }
             }
         }
@@ -372,7 +527,7 @@ ED4_returncode ED4_sequence_terminal::draw( int /*only_text*/ )
 
     // output helix
     if (ED4_ROOT->helix->is_enabled()) { // should do a remap
-        int screen_length = rm->clipped_sequence_to_screen(ED4_ROOT->helix->size);
+        int screen_length = rm->clipped_sequence_to_screen(ED4_ROOT->helix->size());
         if ((right+1) < screen_length) {
             screen_length = right+1;
         }
@@ -436,8 +591,8 @@ ED4_returncode ED4_sequence_info_terminal::draw( int /*only_text*/ )
 {
     AW_pos x, y;
 
-    calc_world_coords( &x, &y );
-    ED4_ROOT->world_to_win_coords( ED4_ROOT->temp_aww, &x, &y );
+    calc_world_coords(&x, &y);
+    ED4_ROOT->world_to_win_coords(ED4_ROOT->get_aww(), &x, &y);
 
     AW_pos text_x = x + CHARACTEROFFSET; // don't change
     AW_pos text_y = y+INFO_TERM_TEXT_YOFFSET;
@@ -451,23 +606,23 @@ ED4_returncode ED4_sequence_info_terminal::draw( int /*only_text*/ )
         GB_pop_transaction(gbdata);
     }
     else {
-        buffer[0] = '-';
+        buffer[0] = ' ';
     }
     strncpy(&buffer[1],this->id,8);
     buffer[9] = 0;
 
     ED4_species_name_terminal *name_term = corresponding_species_name_terminal();
     if (name_term->flag.selected) {
-        ED4_ROOT->temp_device->box(ED4_G_SELECTED, x, y, extension.size[WIDTH], text_y-y+1, (AW_bitset)-1, 0, 0);
+        ED4_ROOT->get_device()->box(ED4_G_SELECTED, AW_TRUE, x, y, extension.size[WIDTH], text_y-y+1, (AW_bitset)-1, 0, 0);
     }
 
-    ED4_ROOT->temp_device->top_font_overlap    = 1;
-    ED4_ROOT->temp_device->bottom_font_overlap = 1;
+    ED4_ROOT->get_device()->top_font_overlap    = 1;
+    ED4_ROOT->get_device()->bottom_font_overlap = 1;
     
-    ED4_ROOT->temp_device->text( ED4_G_STANDARD, buffer, text_x, text_y, 0, 1, 0, 0, 0);
+    ED4_ROOT->get_device()->text( ED4_G_STANDARD, buffer, text_x, text_y, 0, 1, 0, 0, 0);
 
-    ED4_ROOT->temp_device->top_font_overlap    = 0;
-    ED4_ROOT->temp_device->bottom_font_overlap = 0;
+    ED4_ROOT->get_device()->top_font_overlap    = 0;
+    ED4_ROOT->get_device()->bottom_font_overlap = 0;
 
     return ( ED4_R_OK );
 
@@ -480,20 +635,20 @@ ED4_returncode ED4_sequence_info_terminal::draw( int /*only_text*/ )
 ED4_returncode ED4_text_terminal::Show(int IF_DEBUG(refresh_all), int is_cleared)
 {
     e4_assert(update_info.refresh || refresh_all);
-    ED4_ROOT->temp_device->push_clip_scale();
+    ED4_ROOT->get_device()->push_clip_scale();
     if (adjust_clipping_rectangle()) {
         if (update_info.clear_at_refresh && !is_cleared) {
             clear_background();
         }
         draw();
     }
-    ED4_ROOT->temp_device->pop_clip_scale();
+    ED4_ROOT->get_device()->pop_clip_scale();
 
-    ED4_cursor *cursor = &ED4_ROOT->temp_ed4w->cursor;
+    ED4_cursor *cursor = &ED4_ROOT->get_ed4w()->cursor;
     if (this == cursor->owner_of_cursor){
-        ED4_ROOT->temp_device->push_clip_scale();
+        ED4_ROOT->get_device()->push_clip_scale();
         cursor->ShowCursor(0, ED4_C_NONE, 0);
-        ED4_ROOT->temp_device->pop_clip_scale();
+        ED4_ROOT->get_device()->pop_clip_scale();
     }
 
     return ED4_R_OK;
@@ -504,14 +659,14 @@ ED4_returncode ED4_text_terminal::draw( int /*only_text*/ )
     AW_pos x, y;
     AW_pos text_x, text_y;
 
-    calc_world_coords( &x, &y );
-    ED4_ROOT->world_to_win_coords( ED4_ROOT->temp_aww, &x, &y );
+    calc_world_coords(&x, &y);
+    ED4_ROOT->world_to_win_coords(ED4_ROOT->get_aww(), &x, &y);
 
     text_x = x + CHARACTEROFFSET; // don't change
     text_y = y + INFO_TERM_TEXT_YOFFSET;
 
-    ED4_ROOT->temp_device->top_font_overlap    = 1;
-    ED4_ROOT->temp_device->bottom_font_overlap = 1;
+    ED4_ROOT->get_device()->top_font_overlap    = 1;
+    ED4_ROOT->get_device()->bottom_font_overlap = 1;
 
     if (is_species_name_terminal()) {
         GB_CSTR real_name      = to_species_name_terminal()->get_displayed_text();
@@ -540,9 +695,9 @@ ED4_returncode ED4_text_terminal::draw( int /*only_text*/ )
         }
 
         if (flag.selected) {
-            ED4_ROOT->temp_device->box(ED4_G_SELECTED, x, y, extension.size[WIDTH], text_y-y+1, (AW_bitset)-1, 0, 0);
+            ED4_ROOT->get_device()->box(ED4_G_SELECTED, AW_TRUE, x, y, extension.size[WIDTH], text_y-y+1, (AW_bitset)-1, 0, 0);
         }
-        ED4_ROOT->temp_device->text( ED4_G_STANDARD, real_name, text_x+width_of_char, text_y, 0, 1, 0, 0, 0);
+        ED4_ROOT->get_device()->text( ED4_G_STANDARD, real_name, text_x+width_of_char, text_y, 0, 1, 0, 0, 0);
 
         if (paint_box) {
             int xsize = (width_of_char*6)/10;
@@ -552,9 +707,9 @@ ED4_returncode ED4_text_terminal::draw( int /*only_text*/ )
             int bx    = int(text_x+xoff);
             int by    = int(text_y-(yoff+ysize));
 
-            ED4_ROOT->temp_device->box(ED4_G_STANDARD, bx, by, xsize, ysize, (AW_bitset)-1, 0, 0);
+            ED4_ROOT->get_device()->box(ED4_G_STANDARD, AW_TRUE, bx, by, xsize, ysize, (AW_bitset)-1, 0, 0);
             if (!is_marked && xsize>2 && ysize>2) {
-                ED4_ROOT->temp_device->clear_part(bx+1, by+1, xsize-2, ysize-2, (AW_bitset)-1);
+                ED4_ROOT->get_device()->clear_part(bx+1, by+1, xsize-2, ysize-2, (AW_bitset)-1);
             }
         }
     }
@@ -562,11 +717,11 @@ ED4_returncode ED4_text_terminal::draw( int /*only_text*/ )
         char *db_pointer = resolve_pointer_to_string_copy();
 
         if (is_sequence_info_terminal()) {
-            ED4_ROOT->temp_device->text( ED4_G_STANDARD, db_pointer, text_x, text_y, 0, 1, 0, 0, 4);
+            ED4_ROOT->get_device()->text( ED4_G_STANDARD, db_pointer, text_x, text_y, 0, 1, 0, 0, 4);
         }
         else if (is_pure_text_terminal()) { // normal text (i.e. remark)
             text_y += (SEQ_TERM_TEXT_YOFFSET-INFO_TERM_TEXT_YOFFSET);
-            ED4_ROOT->temp_device->text( ED4_G_SEQUENCES, db_pointer, text_x, text_y, 0, 1, 0, 0, 0);
+            ED4_ROOT->get_device()->text( ED4_G_SEQUENCES, db_pointer, text_x, text_y, 0, 1, 0, 0, 0);
         }
         else{
             GB_CORE;
@@ -574,8 +729,8 @@ ED4_returncode ED4_text_terminal::draw( int /*only_text*/ )
 
         free(db_pointer);
     }
-    ED4_ROOT->temp_device->top_font_overlap    = 0;
-    ED4_ROOT->temp_device->bottom_font_overlap = 0;
+    ED4_ROOT->get_device()->top_font_overlap    = 0;
+    ED4_ROOT->get_device()->bottom_font_overlap = 0;
 
     return ( ED4_R_OK );
 }
