@@ -22,6 +22,9 @@
 #include <awt.hxx>
 #include <awt_canvas.hxx>
 #include <awt_sel_boxes.hxx>
+#include <aw_preset.hxx>
+
+#include <ed4_extern.hxx>
 
 #include "ed4_class.hxx"
 #include "ed4_visualizeSAI.hxx"
@@ -389,10 +392,12 @@ void ED4_createVisualizeSAI_Awars(AW_root *aw_root, AW_default aw_def) {  // ---
     AW_awar *awar_defaults_created = aw_root->awar_int(AWAR_SAI_CLR_DEFAULTS_CREATED, 0,  aw_def);
 
     if (awar_defaults_created->read_int() == 0) {
-        addDefaultTransTable(aw_root, "numeric", "0;1;2;3;4;5;6;7;8;9;");
-        addDefaultTransTable(aw_root, "binary", ".;+;;;;;;;;;");
-        addDefaultTransTable(aw_root, "xstring", ";x;;;;;;;;;");
+        addDefaultTransTable(aw_root, "numeric",   "0;1;2;3;4;5;6;7;8;9;");
+        addDefaultTransTable(aw_root, "binary",    ".;+;;;;;;;;;");
         addDefaultTransTable(aw_root, "consensus", "=ACGTU;;acgtu;.;;;;;;;");
+        addDefaultTransTable(aw_root, "helix",     ";;<>;;;;;[];;;");
+        addDefaultTransTable(aw_root, "xstring",   ";x;;;;;;;;;");
+        addDefaultTransTable(aw_root, "gaps",      ";-.;;;;;;;;;");
 
         awar_defaults_created->write_int(1);
     }
@@ -494,7 +499,7 @@ static AW_selection_list *buildClrTransTabNamesList(AW_window *aws) {
     return id;
 }
 
-const char *getSaiColorString(AW_root *awr, int start, int end) {
+const char *ED4_getSaiColorString(AW_root *awr, int start, int end) {
     static int   seqBufferSize = 0;
     static char *saiColors     = 0;
     static int   lastStart     = -1;
@@ -680,6 +685,11 @@ static AW_window *create_editColorTranslationTable_window(AW_root *aw_root){  //
     aws->callback(reverseColorTranslationTable);
     aws->create_button("REVERSE", "Reverse", "R");
 
+    aws->at("colors");
+    aws->callback(AW_POPUP,(AW_CL)AW_create_gc_window, (AW_CL)ED4_ROOT->aw_gc_manager);
+    aws->button_length(0);
+    aws->create_button("COLORS","#colors.xpm");
+
     return (AW_window *)aws;
 }
 
@@ -709,7 +719,7 @@ AW_window *ED4_createVisualizeSAI_window(AW_root *aw_root) {
 
     aws = new AW_window_simple;
 
-    aws->init( aw_root, "VISUALIZE_SAI", "VISUALIZE SAI");
+    aws->init( aw_root, "VISUALIZE_SAI", "Visualize SAIs");
     aws->load_xfig("visualizeSAI.fig");
 
     aws->callback( AW_POPUP_HELP,(AW_CL)"visualizeSAI.hlp");
@@ -766,3 +776,6 @@ AW_window *ED4_createVisualizeSAI_window(AW_root *aw_root) {
     return (AW_window *)aws;
 }
 
+bool ED4_SAIs_visualized() {
+    return ED4_ROOT->visualizeSAI;
+}
