@@ -7,6 +7,7 @@
 #include <aw_device.hxx>
 #include <aw_window.hxx>
 #include <aw_awars.hxx>
+#include <AW_rename.hxx>
 #include <awt.hxx>
 #include <awt_changekey.hxx>
 #include <awt_sel_boxes.hxx>
@@ -256,7 +257,7 @@ char *MG_remap::remap(const char *sequence){
 
 void MG_transfer_fields_info(char *fieldname = 0);
 
-void MG_create_species_var(AW_root *aw_root, AW_default aw_def)
+void MG_create_species_awars(AW_root *aw_root, AW_default aw_def)
 {
     aw_root->awar_string( AWAR_SPECIES1, "" ,   aw_def);
     aw_root->awar_string( AWAR_SPECIES2, "" ,   aw_def);
@@ -443,9 +444,7 @@ GB_ERROR MG_transfer_sequence(MG_remaps *remaps, GBDATA *source_species, GBDATA
 
 
 
-void MG_transfer_selected_species(AW_window *aww)
-{
-
+void MG_transfer_selected_species(AW_window *aww) {
     if (MG_check_alignment(aww,1)) return;
     aw_openstatus("Transferring selected species");
     GB_ERROR error = 0;
@@ -453,6 +452,8 @@ void MG_transfer_selected_species(AW_window *aww)
     GB_begin_transaction(gb_merge);
     GB_begin_transaction(gb_dest);
 
+    bool is_genome_db = GEN_is_genome_db(gb_merge, -1);
+
     GBDATA *gb_species_data1 = GB_search(gb_merge,"species_data",GB_CREATE_CONTAINER);
     GBDATA *gb_species_data2 = GB_search(gb_dest,"species_data",GB_CREATE_CONTAINER);
 
@@ -472,7 +473,9 @@ void MG_transfer_selected_species(AW_window *aww)
     }
     if (!error) {
         error = GB_copy(gb_species2,gb_species1);
-        if (!error) error = MG_export_fields(aww->get_root(), gb_species1, gb_species2);
+        if (!error && is_genome_db && GEN_is_pseudo_gene_species(gb_species1)) {
+            error = MG_export_fields(aww->get_root(), gb_species1, gb_species2, 0, 0);
+        }
         GB_write_flag(gb_species2,1);
     }
     if (!error) {       // align sequence !!!!!!!!!
@@ -504,29 +507,55 @@ void MG_transfer_species_list(AW_window *aww)
     GB_begin_transaction(gb_merge);
     GB_begin_transaction(gb_dest);
 
-    GBDATA *gb_species_data2 = GB_search(gb_dest,"species_data",GB_CREATE_CONTAINER);
+    GBDATA *gb_dest_species_data = GB_search(gb_dest,"species_data",GB_CREATE_CONTAINER);
 
     bool is_genome_db = GEN_is_genome_db(gb_merge, -1);
 
+    GB_HASH   *error_suppressor     = GBS_create_hash(50, 1);
+    GB_HASH   *source_species_hash  = GBT_create_species_hash(gb_merge);
+    GB_HASH   *dest_species_hash    = GBT_create_species_hash(gb_dest);
+    GB_HASH   *source_organism_hash = is_genome_db ? GBT_create_organism_hash(gb_merge) : 0;
+    MG_remaps  rm(gb_merge,gb_dest,aww->get_root());
+
     GBDATA *gb_species1;
-    GBDATA *gb_species2;
-    GBDATA *gb_name1;
-    MG_remaps rm(gb_merge,gb_dest,aww->get_root());
+    int     queried = 0;
+    int     count   = 0;
+
+    for (gb_species1 = GBT_first_species(gb_merge); gb_species1; gb_species1 = GBT_next_species(gb_species1)) {
+        if (IS_QUERIED(gb_species1)) queried++;
+    }
+    
     for (gb_species1 = GBT_first_species(gb_merge); gb_species1; gb_species1 = GBT_next_species(gb_species1)) {
         if (IS_QUERIED(gb_species1)) {
-            gb_name1 = GB_find(gb_species1,"name",0,down_level);
-            if (!gb_name1) continue;    // no name what happend ???
-            gb_species2 = GB_find(gb_species_data2,"name", GB_read_char_pntr(gb_name1),down_2_level);
+            GBDATA *gb_name1 = GB_find(gb_species1,"name",0,down_level);
+            gb_assert(gb_name1);
+            if (!gb_name1) continue;    // no name what happened ???
+
+            const char *name        = GB_read_char_pntr(gb_name1);
+            GBDATA     *gb_species2 = (GBDATA*)GBS_read_hash(dest_species_hash, name);
+            if (gb_species2) error  = GB_delete(gb_species2);
+
+            if (!error) gb_species2 = GB_create_container(gb_dest_species_data,"species");
+            if (!error) error       = GB_copy(gb_species2,gb_species1);
+
+            if (!error && is_genome_db && GEN_is_pseudo_gene_species(gb_species1)) {
+                error = MG_export_fields(aww->get_root(), gb_species1, gb_species2, error_suppressor, source_organism_hash);
+            }
 
-            if (gb_species2) error = GB_delete(GB_get_father(gb_species2));
-            if (!error) gb_species2 = GB_create_container(gb_species_data2,"species");
-            if (!error) error = GB_copy(gb_species2,gb_species1);
-            if (!error && is_genome_db) error = MG_export_fields(aww->get_root(), gb_species1, gb_species2);
             if (!error) error = MG_transfer_sequence(&rm,gb_species1,gb_species2);
             if (error) break;
+            
             GB_write_flag(gb_species2,1);
+            GBS_write_hash(dest_species_hash, name, (long)gb_species2);
+            aw_status(++count/double(queried));
         }
     }
+
+    GBS_free_hash(dest_species_hash);
+    GBS_free_hash(source_species_hash);
+    if (source_organism_hash) GBS_free_hash(source_organism_hash);
+    GBS_free_hash(error_suppressor);
+
     if (error) {
         GB_abort_transaction(gb_merge);
         GB_abort_transaction(gb_dest);
@@ -560,7 +589,7 @@ void MG_transfer_fields_cb(AW_window *aww){
     GB_begin_transaction(gb_merge);
     GB_begin_transaction(gb_dest);
 
-    GBDATA *gb_species_data2 = GB_search(gb_dest,"species_data",GB_CREATE_CONTAINER);
+    GBDATA *gb_dest_species_data = GB_search(gb_dest,"species_data",GB_CREATE_CONTAINER);
 
     GBDATA *gb_species1;
     GBDATA *gb_species2;
@@ -583,10 +612,10 @@ void MG_transfer_fields_cb(AW_window *aww){
         if (IS_QUERIED(gb_species1)) {
             gb_name1 = GB_find(gb_species1,"name",0,down_level);
             if (!gb_name1) continue;    // no name what happend ???
-            gb_species2 = GB_find(gb_species_data2,"name", GB_read_char_pntr(gb_name1),down_2_level);
+            gb_species2 = GB_find(gb_dest_species_data,"name", GB_read_char_pntr(gb_name1),down_2_level);
 
             if (!gb_species2) {
-                gb_species2 = GB_create_container(gb_species_data2,"species");
+                gb_species2 = GB_create_container(gb_dest_species_data,"species");
                 if (!gb_species2) {
                     error = GB_get_error();
                     break;
@@ -827,7 +856,7 @@ void MG_merge_tagged_field_cb(AW_window *aww){
     char *tag_del1= awr->awar(AWAR_TAG_DEL1)->read_string();
 
 
-    GBDATA *gb_species_data2 = GB_search(gb_dest,"species_data",GB_CREATE_CONTAINER);
+    GBDATA *gb_dest_species_data = GB_search(gb_dest,"species_data",GB_CREATE_CONTAINER);
 
     GBDATA *gb_species1;
     GBDATA *gb_species2;
@@ -845,7 +874,7 @@ void MG_merge_tagged_field_cb(AW_window *aww){
             if (!gb_name1) continue;    // no name what happend ???
             delete name;
             name = GB_read_string(gb_name1);
-            gb_species2 = GBT_create_species_rel_species_data(gb_species_data2,name);
+            gb_species2 = GBT_create_species_rel_species_data(gb_dest_species_data,name);
 
             if (!gb_species2) {
                 error = GB_get_error();
@@ -1051,15 +1080,18 @@ GB_ERROR MG_equal_alignments(bool autoselect_equal_alignment_name) {
 
 /** Merge the sequences of two databases */
 GB_ERROR MG_simple_merge(AW_root *awr) {
-    static char *m_name = 0;
+    static char *m_name         = 0;
+    GB_ERROR     error          = 0;
+    GBDATA      *M_species_data = 0;
+    GBDATA      *D_species_data = 0;
+    int          overwriteall   = 0;
+    int          autorenameall  = 0;
+    GB_HASH     *D_species_hash = 0;
+
     GB_push_my_security(gb_merge);
     GB_push_my_security(gb_dest);
-    GB_ERROR error = 0;
     GB_begin_transaction(gb_merge);
     GB_begin_transaction(gb_dest);
-    GBDATA *M_species_data = 0;
-    GBDATA *D_species_data = 0;
-    int overwriteall = 0;
 
     error = MG_equal_alignments(true);
     if (error) goto end;
@@ -1070,20 +1102,44 @@ GB_ERROR MG_simple_merge(AW_root *awr) {
     GBDATA *M_species;
     GBDATA *D_species;
     free(m_name); m_name = 0;
-    for (   M_species   = GB_find(M_species_data,"species",0,down_level);
-            M_species;
-            M_species   = GB_find(M_species,"species",0,this_level | search_next)){
-        GBDATA *M_name = GB_search(M_species,"name",GB_STRING);
-        free(m_name);
-        m_name = GB_read_string(M_name);
+
+    {
+        long M_species_count = GB_number_of_subentries(M_species_data);
+        long D_species_count = GB_number_of_subentries(D_species_data);
+
+        // create hash containing all species from gb_dest,
+        // but sized to hold all species from both DBs: 
+        D_species_hash = GBT_create_species_hash_sized(gb_dest, M_species_count+D_species_count);
+    }
+
+    for (M_species = GB_find(M_species_data,"species",0,down_level);
+         M_species;
+         M_species = GB_find(M_species,"species",0,this_level | search_next))
+    {
+        GBDATA *M_name       = GB_search(M_species,"name",GB_STRING);
+        free(m_name); m_name = GB_read_string(M_name);
+
         int count = 1;
+
     new_try:
+
         count++;
-        D_species = GBT_find_species_rel_species_data(D_species_data,m_name);
+        
+        D_species = (GBDATA*)GBS_read_hash(D_species_hash, m_name);
         if (D_species){
-            if (overwriteall){
+            if (overwriteall) {
                 error = GB_delete(D_species);
-            }else{
+            }
+            else if (autorenameall) {
+                GB_ERROR  dummy;
+                char     *newname = AWTC_create_numbered_suffix(D_species_hash, m_name, dummy);
+
+                mg_assert(newname);
+
+                free(m_name);
+                m_name = newname;
+            }
+            else {
                 switch (aw_message(GBS_global_string(
                                                      "Warning:  There is a name conflict for species '%s'\n"
                                                      "  You may:\n"
@@ -1091,36 +1147,70 @@ GB_ERROR MG_simple_merge(AW_root *awr) {
                                                      "  - Overwrite all species with name conflicts\n"
                                                      "  - Not import species\n"
                                                      "  - Rename imported species\n"
+                                                     "  - Automatically rename species (append .NUM)\n"
                                                      "  - Abort everything", m_name),
-                                   "overwrite, overwrite all, don't import, rename, abort")){
+                                   "overwrite, overwrite all, don't import, rename, auto-rename, abort")){
                     case 1:
                         overwriteall = 1;
                     case 0:
                         GB_delete(D_species);
                         break;
+                        
                     case 2:
                         continue;
-                    case 3:
-                        awr->awar_string(NEW_NAME_AWAR)->write_string((char *)GBS_global_string("%s.%i",m_name,count));
+
+                    case 3: {
+                        GB_ERROR  dummy;
+                        char     *newname = AWTC_create_numbered_suffix(D_species_hash, m_name, dummy);
+                        awr->awar_string(NEW_NAME_AWAR)->write_string(newname ? newname : "???");
+                        free(newname);
+
                         free(m_name);
                         m_name = aw_input("New name of species",NEW_NAME_AWAR);
+
                         goto new_try;
+                    }
                     case 4:
+                        autorenameall = 1;
+                        goto new_try;
+                        
+                    case 5:
                         error = "Operation aborted";
                         goto end;
                 }
             }
         }
-        D_species = GB_create_container(D_species_data,"species");
-        GB_copy(D_species, M_species);
-        GB_write_flag(D_species,1); // mark species
-        GB_write_usr_private(D_species,255); // put in hitlist
-        GBDATA *D_name = GB_search(D_species,"name",GB_STRING);
-        GB_write_string(D_name,m_name);
+
+        if (!error) {
+            D_species      = GB_create_container(D_species_data,"species");
+            if (!D_species) {
+                error = GB_get_error();
+            }
+            else {
+                error             = GB_copy(D_species, M_species);
+                if (!error) error = GB_write_flag(D_species,1); // mark species
+                if (!error) error = GB_write_usr_private(D_species,255); // put in hitlist
+                if (!error) {
+                    GBDATA *D_name = GB_search(D_species,"name",GB_STRING);
+                    if (!D_name) {
+                        error = GB_get_error();
+                    }
+                    else {
+                        error = GB_write_string(D_name,m_name);
+                    }
+                }
+            }
+
+            GBS_write_hash(D_species_hash, m_name, (long)D_species);
+        }
+
         if (error) break;
     }
 
  end:
+
+    if (D_species_hash) GBS_free_hash(D_species_hash);
+    
     if (error) {
         GB_abort_transaction(gb_merge);
         GB_abort_transaction(gb_dest);
@@ -1133,6 +1223,7 @@ GB_ERROR MG_simple_merge(AW_root *awr) {
 
         awr->awar(AWAR_SPECIES_NAME)->write_string(m_name);
     }
+    
     GB_pop_my_security(gb_merge);
     GB_pop_my_security(gb_dest);
     return error;
