@@ -17,15 +17,33 @@
 #include <list>
 #include <string>
 
+using namespace std;
+
+// --------------------------------------------------------------------------------
+
+// overloaded functions to avoid problems with type-punning:
+inline long aisc_find_lib(dll_public *dll, char *key) { return aisc_find_lib(reinterpret_cast<dllpublic_ext*>(dll), key); }
+
+inline void aisc_link(dll_public *dll, AN_shorts *shorts)   { aisc_link(reinterpret_cast<dllpublic_ext*>(dll), reinterpret_cast<dllheader_ext*>(shorts)); }
+inline void aisc_link(dll_public *dll, AN_revers *revers)   { aisc_link(reinterpret_cast<dllpublic_ext*>(dll), reinterpret_cast<dllheader_ext*>(revers)); }
+
+// --------------------------------------------------------------------------------
+
 #if defined(DEBUG)
 // #define DUMP_NAME_CREATION
 #endif // DEBUG
 
 #define UPPERCASE(c) do{ (c) = toupper(c); }while(0)
 
+// --------------------------------------------------------------------------------
+
 struct AN_gl_struct  AN_global;
 AN_main             *aisc_main; /* muss so heissen */
 
+const int SERVER_VERSION = 4;
+
+// --------------------------------------------------------------------------------
+
 inline char *an_strlwr(char *str) {
     for (int i = 0; str[i]; i++) {
         str[i] = tolower(str[i]);
@@ -58,7 +76,7 @@ inline int an_stricmp(const char *s1, const char *s2) {
 
 static AN_revers *lookup_an_revers(AN_main *main, const char *shortname) {
     char      *key        = an_strlwr(strdup(shortname));
-    AN_revers *an_reverse = (AN_revers*)aisc_find_lib((struct_dllpublic_ext*)&(main->prevers), key);
+    AN_revers *an_reverse = (AN_revers*)aisc_find_lib(&main->prevers, key);
     
     free(key);
 
@@ -66,16 +84,68 @@ static AN_revers *lookup_an_revers(AN_main *main, const char *shortname) {
 }
 
 static AN_shorts *lookup_an_shorts(AN_main *main, const char *identifier) {
-    // 'identifier' is either '*acc' or 'name1*name2*S' (see get_short() for details)
+    // 'identifier' is either '*acc*add_id' or 'name1*name2*S' (see get_short() for details)
+    // 'add_id' is the value of an additional DB field and may be empty.
+
     char      *key       = an_strlwr(strdup(identifier));
-    AN_shorts *an_shorts = (AN_shorts*)aisc_find_lib((struct_dllpublic_ext*)&(main->pnames), key);
+    AN_shorts *an_shorts = (AN_shorts*)aisc_find_lib(&main->pnames, key);
 
     free(key);
 
     return an_shorts;
 }
 
-static void an_add_short(AN_local *locs,const char *new_name, const char *parsed_name, const char *parsed_sym, const char *shrt, const char *acc)
+// ----------------------------------------
+// prefix hash
+
+static size_t an_shorts_elems(AN_shorts *sin) {
+    size_t count = 0;
+    while (sin) {
+        sin = sin->next;
+        count++;
+    }
+    return count;
+}
+
+#define PREFIXLEN 3
+
+static GB_HASH *an_get_prefix_hash() {
+    if (!aisc_main->prefix_hash) {
+        AN_shorts *sin       = aisc_main->shorts1;
+        size_t     elems     = an_shorts_elems(sin);
+        if (elems<100) elems = 100;
+        
+        GB_HASH *hash = GBS_create_hash(2*elems, 1);
+
+        while (sin) {
+            GBS_write_hash_no_strdup(hash, GB_strndup(sin->shrt, PREFIXLEN), (long)sin);
+            sin = sin->next;
+        }
+
+        aisc_main->prefix_hash = (long)hash;
+    }
+    return (GB_HASH*)aisc_main->prefix_hash;
+}
+
+static const char *an_make_prefix(const char *str) {
+    static char buf[] = "xxx";
+
+    buf[0] = str[0];
+    buf[1] = str[1];
+    buf[2] = str[2];
+    
+    return buf;
+}
+
+static AN_shorts *an_find_shrt_prefix(const char *search) {
+    return (AN_shorts*)GBS_read_hash(an_get_prefix_hash(), an_make_prefix(search));
+}
+
+// ----------------------------------------
+
+static void an_add_short(AN_local *locs, const char *new_name,
+                         const char *parsed_name, const char *parsed_sym,
+                         const char *shrt, const char *acc, const char *add_id)
 {
     AN_shorts *an_shorts;
     AN_revers *an_revers;
@@ -83,26 +153,35 @@ static void an_add_short(AN_local *locs,const char *new_name, const char *parsed
     locs = locs;
 
     if (strlen(parsed_sym)){
-        full_name = (char *)calloc(sizeof(char),
-                                   strlen(parsed_name) + strlen(" sym")+1);
+        full_name = (char *)calloc(sizeof(char), strlen(parsed_name) + strlen(" sym")+1);
         sprintf(full_name,"%s sym",parsed_name);
-    }else{
+    }
+    else {
         full_name = strdup(parsed_name);
     }
 
     an_shorts = create_AN_shorts();
     an_revers = create_AN_revers();
 
-    an_shorts->mh.ident = an_strlwr(strdup(new_name));
-    an_shorts->shrt = strdup(shrt);
+    an_shorts->mh.ident  = an_strlwr(strdup(new_name));
+    an_shorts->shrt      = strdup(shrt);
     an_shorts->full_name = strdup(full_name);
-    an_shorts->acc = strdup(acc);
-    aisc_link((struct_dllpublic_ext*)&(aisc_main->pnames),(struct_dllheader_ext*)an_shorts);
+    an_shorts->acc       = strdup(acc);
+    an_shorts->add_id    = strdup(add_id);
 
-    an_revers->mh.ident = an_strlwr(strdup(shrt));
+    aisc_link(&aisc_main->pnames, an_shorts);
+
+    an_revers->mh.ident  = an_strlwr(strdup(shrt));
     an_revers->full_name = full_name;
-    an_revers->acc = strdup(acc);
-    aisc_link((struct_dllpublic_ext*)&(aisc_main->prevers),(struct_dllheader_ext*)an_revers);
+    an_revers->acc       = strdup(acc);
+    an_revers->add_id    = strdup(add_id);
+
+    aisc_link(&aisc_main->prevers, an_revers);
+
+    GB_HASH *phash = an_get_prefix_hash();
+    GBS_write_hash(phash, an_make_prefix(an_shorts->shrt), (long)an_shorts); // add an_shorts to hash
+    GBS_optimize_hash(phash);
+
     aisc_main->touched = 1;
 }
 
@@ -110,6 +189,8 @@ static void an_remove_short(AN_shorts *an_shorts) {
     /* this should only be used to remove illegal entries from name-server.
        normally removing names does make problems - so use it very rarely */
 
+    GBS_write_hash(an_get_prefix_hash(), an_make_prefix(an_shorts->shrt), 0); // delete an_shorts from hash
+    
     AN_revers *an_revers = lookup_an_revers(aisc_main, an_shorts->shrt);
 
     if (an_revers) {
@@ -130,28 +211,16 @@ static void an_remove_short(AN_shorts *an_shorts) {
     free(an_shorts);
 }
 
-
-
-
-static AN_shorts *an_find_shrt(AN_shorts *sin,char *search)
-{
-    while (sin) {
-        if (!an_strnicmp(sin->shrt,search,3)) {
-            return sin;
-        }
-        sin = sin->next;
-    }
-    return 0;
-}
-
-static char *nas_string_2_key(const char *str)  /* converts any string to a valid key */
+static char *nas_string_2_key(const char *str)
+// converts any string to a valid key
 {
 #if defined(DUMP_NAME_CREATION)
-    const char *org_str          = str;
+    const char *org_str = str;
 #endif // DUMP_NAME_CREATION
+    
     char buf[GB_KEY_LEN_MAX+1];
-    int i;
-    int c;
+    int  i;
+    int  c;
     for (i=0;i<GB_KEY_LEN_MAX;) {
         c                        = *(str++);
         if (!c) break;
@@ -248,10 +317,11 @@ static void an_autocaps(char *str) {
     }
 }
 
-static char *an_get_short(AN_shorts *shorts, dll_public *parent, const char *full){
+static char *an_get_short(AN_shorts *IF_DEBUG(shorts), dll_public *parent, const char *full){
     AN_shorts *look;
 
     gb_assert(full);
+    gb_assert(shorts == aisc_main->shorts1); // otherwise prefix_hash does not work!
 
     if (full[0]==0) {
         return strdup("Xxx");
@@ -281,7 +351,7 @@ static char *an_get_short(AN_shorts *shorts, dll_public *parent, const char *ful
     UPPERCASE(shrt[0]);
     shrt[3] = 0;
 
-    look = an_find_shrt(shorts,shrt);
+    look = an_find_shrt_prefix(shrt);
     if (!look) {
         len2   = strlen(full2);
         an_complete_shrt(shrt, len2>=3 ? full2+3 : "");
@@ -297,7 +367,7 @@ static char *an_get_short(AN_shorts *shorts, dll_public *parent, const char *ful
         shrt[1] = full3[p1];
         for (p2=p1+1; p2<len3; p2++) {
             shrt[2] = full3[p2];
-            look = an_find_shrt(shorts, shrt);
+            look = an_find_shrt_prefix(shrt);
             if (!look) {
                 an_complete_shrt(shrt, full3+p2+1);
                 goto found_short;
@@ -312,7 +382,7 @@ static char *an_get_short(AN_shorts *shorts, dll_public *parent, const char *ful
         shrt[1] = full2[p1];
         for (p2=p1+1; p2<len2; p2++) {
             shrt[2] = full2[p2];
-            look = an_find_shrt(shorts, shrt);
+            look = an_find_shrt_prefix(shrt);
             if (!look) {
                 an_complete_shrt(shrt, full2+p2+1);
                 goto found_short;
@@ -326,7 +396,7 @@ static char *an_get_short(AN_shorts *shorts, dll_public *parent, const char *ful
         shrt[1] = full2[p1];
         for (p2=0; p2<=9; p2++) {
             shrt[2] = '0'+p2;
-            look = an_find_shrt(shorts, shrt);
+            look = an_find_shrt_prefix(shrt);
             if (!look) {
                 an_complete_shrt(shrt, full2+p1+1);
                 goto found_short;
@@ -339,7 +409,7 @@ static char *an_get_short(AN_shorts *shorts, dll_public *parent, const char *ful
     for (p1=1; p1<=99; p1++) {
         shrt[1] = '0'+(p1/10);
         shrt[2] = '0'+(p1%10);
-        look = an_find_shrt(shorts, shrt);
+        look = an_find_shrt_prefix(shrt);
         if (!look) {
             an_complete_shrt(shrt, full2+1);
             goto found_short;
@@ -353,7 +423,7 @@ static char *an_get_short(AN_shorts *shorts, dll_public *parent, const char *ful
         for (p2=0; p2<=99; p2++) {
             shrt[1] = '0'+(p2/10);
             shrt[2] = '0'+(p2%10);
-            look = an_find_shrt(shorts, shrt);
+            look = an_find_shrt_prefix(shrt);
             if (!look) {
                 an_complete_shrt(shrt, full2);
                 goto found_short;
@@ -369,7 +439,7 @@ static char *an_get_short(AN_shorts *shorts, dll_public *parent, const char *ful
             shrt[1] = p2;
             for (p3=0; p3<=9; p3++) {
                 shrt[2] = '0'+p3;
-                look = an_find_shrt(shorts, shrt);
+                look = an_find_shrt_prefix(shrt);
                 if (!look) {
                     an_complete_shrt(shrt, full2);
                     goto found_short;
@@ -386,7 +456,7 @@ static char *an_get_short(AN_shorts *shorts, dll_public *parent, const char *ful
             shrt[1] = p2;
             for (p3='a'; p3<='z'; p3++) {
                 shrt[2] = p3;
-                look = an_find_shrt(shorts, shrt);
+                look = an_find_shrt_prefix(shrt);
                 if (!look) {
                     an_complete_shrt(shrt, full2);
                 found_short:
@@ -434,6 +504,7 @@ class NameInformation {
     char *parsed_name;
     char *parsed_sym;
     char *parsed_acc;
+    char *parsed_add_id;
 
     char *first_name;
     char *rest_of_name;
@@ -451,40 +522,94 @@ public:
 
     void add_short(AN_local *locs, const char *shrt) const {
         if (strlen(parsed_name)>3) {
-            an_add_short(locs, id, parsed_name, parsed_sym, shrt, parsed_acc);
+            an_add_short(locs, id, parsed_name, parsed_sym, shrt, parsed_acc, parsed_add_id);
         }
     }
 };
 
-#define ILLEGAL_NAME_CHARS         " \t#;,@_"
-#define REPLACE_ILLEGAL_NAME_CHARS " =:\t=:#=:;=:,=:@=:_="
+static bool stralnum(const char *str) {
+    bool nonalnum = false;
+    for (char c = *str++; c; c = *str++) {
+        if (!isalnum(c)) {
+            nonalnum = true;
+            break;
+        }
+    }
+    return !nonalnum;
+}
+
+static char *make_alnum(const char *str) {
+    // returns a heap-copy containing all alphanumeric characters of 'str'
+
+    char *newStr = (char*)malloc(strlen(str)+1);
+    int   n      = 0;
+
+    for (int p = 0; str[p]; ++p) {
+        if (isalnum(str[p])) newStr[n++] = str[p];
+    }
+    newStr[n] = 0;
+    return newStr;
+}
+static char *make_alpha(const char *str) {
+    // returns a heap-copy containing all alpha characters of 'str'
+
+    char *newStr = (char*)malloc(strlen(str)+1);
+    int   n      = 0;
+
+    for (int p = 0; str[p]; ++p) {
+        if (isalpha(str[p])) newStr[n++] = str[p];
+    }
+    newStr[n] = 0;
+    return newStr;
+}
+
+#if defined(DEBUG)
+#define assert_alnum(s) gb_assert(stralnum(s))
+#else
+#define assert_alnum(s)
+#endif // DEBUG
 
 NameInformation::NameInformation(AN_local *locs) {
     full_name = locs->full_name;
     if (!full_name || !full_name[0]) full_name = default_full_name;
 
-    parsed_name = GBS_string_eval(full_name, "\t= :\"=:'=:* * *=*1 *2:sp.=species:spec.=species:SP.=SPECIES:SPEC.=SPECIES:.=",0);
+    parsed_name = GBS_string_eval(full_name,
+                                  "\t= :\"= :'= :" // replace TABs and quotes by space
+                                  "sp.=species:spec.=species:SP.=SPECIES:SPEC.=SPECIES:" // replace common abbreviations of 'species' 
+                                  ".= :" // replace dots by spaces
+                                  "  = :" // multiple spaces -> 1 space
+                                  "* * *=*1 *2" // skip all beyond 2nd word
+                                  ,0);
     an_autocaps(parsed_name);
 
-    /* delete ' " \t and more than two words */
     parsed_sym = GBS_string_eval(full_name, "\t= :* * *sym*=S",0);
     if (strlen(parsed_sym)>1) {
         free(parsed_sym);
         parsed_sym = strdup("");
     }
 
-    parsed_acc   = GBS_string_eval(locs->acc,REPLACE_ILLEGAL_NAME_CHARS,0);
-    first_name   = GBS_string_eval(parsed_name,"* *=*1",0);
-    rest_of_name = GBS_string_eval(parsed_name+strlen(first_name), REPLACE_ILLEGAL_NAME_CHARS, 0);
+    parsed_acc      = make_alnum(locs->acc);
+    parsed_add_id   = make_alnum(locs->add_id);
+    first_name      = GBS_string_eval(parsed_name,"* *=*1",0);
+    rest_of_name    = make_alnum(parsed_name+strlen(first_name));
+
+    {
+        char *first_name_an = make_alnum(first_name);
+        free(first_name);
+        first_name = first_name_an;
+    }
+
+    assert_alnum(parsed_acc);
+    assert_alnum(first_name);
+    assert_alnum(rest_of_name);
+
     UPPERCASE(rest_of_name[0]);
 
     // build id
 
-    id = strlen(parsed_acc)
-        ? GBS_global_string_copy("*%s", parsed_acc)
+    id = (strlen(parsed_acc)+strlen(parsed_add_id))
+        ? GBS_global_string_copy("*%s*%s", parsed_acc, parsed_add_id)
         : GBS_global_string_copy("%s*%s*%s", first_name, rest_of_name, parsed_sym);
-
-    
 }
 
 NameInformation::~NameInformation() {
@@ -493,12 +618,14 @@ NameInformation::~NameInformation() {
     free(rest_of_name);
     free(first_name);
 
+    free(parsed_add_id);
     free(parsed_acc);
     free(parsed_sym);
     free(parsed_name);
 }
 
 // --------------------------------------------------------------------------------
+// AISC functions
 
 extern "C" int del_short(AN_local *locs)
 /* forget about a short name */
@@ -515,6 +642,8 @@ extern "C" int del_short(AN_local *locs)
     return removed;
 }
 
+static GB_HASH *nameModHash = 0; // key = default name; value = max. counter tested
+
 extern "C" aisc_string get_short(AN_local *locs)
 /* get the short name from the previously set names */
 {
@@ -526,11 +655,11 @@ extern "C" aisc_string get_short(AN_local *locs)
 
     NameInformation  info(locs);
     AN_shorts       *an_shorts = lookup_an_shorts(aisc_main, info.get_id());
-    
+
     if (an_shorts) {            // we already have a short name
         bool recreate = false;
 
-        if (strpbrk(an_shorts->shrt, ILLEGAL_NAME_CHARS) != 0) { // contains illegal characters
+        if (!stralnum(an_shorts->shrt)) { // contains non-alphanumeric characters
             recreate = true;
         }
         else if (strcmp(an_shorts->full_name, default_full_name) == 0 && // fullname in name server is default_full_name
@@ -549,12 +678,12 @@ extern "C" aisc_string get_short(AN_local *locs)
     if (!shrt) { /* now there is no short name (or an illegal one) */
         char *first_advice=0,*second_advice=0;
 
-        if (locs->advice[0] && strpbrk(locs->advice, ILLEGAL_NAME_CHARS)!=0) {
-            locs->advice[0] = 0; // delete advice
+        if (locs->advice[0] && !stralnum(locs->advice)) { // bad advice 
+            locs->advice[0] = 0; // delete it
         }
 
         if (locs->advice[0]) {
-            char *advice = GBS_string_eval(locs->advice, "0=:1=:2=:3=:4=:5=:6=:7=:8=:9=:" REPLACE_ILLEGAL_NAME_CHARS, 0);
+            char *advice = make_alpha(locs->advice);
 
             first_advice = strdup(advice);
             if (strlen(advice) > 3) {
@@ -609,42 +738,100 @@ extern "C" aisc_string get_short(AN_local *locs)
             second_len   = strlen(second_short);
         }
 
-        if (first_len<3) {
-            char *new_first = GBS_global_string_copy("%s___", first_short);
-            free(first_short);
-            first_short     = new_first;
+        if (first_len>3) {
+            first_short[3] = 0;
+            first_len      = 3;;
         }
-        first_short[3] = 0;
-        first_len      = 3;
 
-        if (second_len<5) {
-            char *new_second = GBS_global_string_copy("%s_____", second_short);
+        int both_len = first_len+second_len;
+        if (both_len<8) {
+            char *new_second  = GBS_global_string_copy("%s00000000", second_short);
             free(second_short);
-            second_short     = new_second;
+            second_short      = new_second;
+            second_len       += 8;
+            both_len         += 8;
+        }
+
+        if (both_len>8) {
+            second_len               = 8-first_len;
+            second_short[second_len] = 0;
+            both_len                 = 8;
         }
-        second_short[5] = 0;
-        second_len      = 5;
 
         char test_short[9];
         sprintf(test_short,"%s%s", first_short, second_short);
+        
+        gb_assert(size_t(both_len) == strlen(test_short));
+        gb_assert(second_len>=5 && second_len <= 8);
 
         if (lookup_an_revers(aisc_main, test_short)) {
-            int count;
+            if (!nameModHash) nameModHash = GBS_create_hash(100, 1);
 
-            gb_assert(second_len==5);
-            second_short[second_len-1] = 0;
-            for (count = 2; count <99999; count++) {
-                if      (count==10)    second_short[second_len-2] = 0;
-                else if (count==100)   second_short[second_len-3] = 0;
-                else if (count==1000)  second_short[second_len-4] = 0;
-                else if (count==10000) second_short[second_len-5] = 0;
+            char *test_short_dup = strdup(test_short);
+            long  count          = GBS_read_hash(nameModHash, test_short);
+            if (count<2) count   = 2; // first name modification uses number 2
 
-                int printed = sprintf(test_short,"%s%s%i",first_short, second_short, count);
-                gb_assert(printed <= 8);
-                if (!lookup_an_revers(aisc_main, test_short)) break;
+            int  printOffset = both_len;
+            bool foundUnused = false;
+
+            // first create alternate name with digits only
+            {
+                int digLimit[6] = { 0, 9, 99, 999, 9999, 99999 };
+                for (int digits = 1; !foundUnused && digits <= 5; ++digits) {
+                    int maxOffset = 8-digits;
+                    int limit     = digLimit[digits];
+
+                    if (printOffset>maxOffset) printOffset = maxOffset;
+                
+                    char *printAt = test_short+printOffset;
+
+                    for (; !foundUnused && count <= limit; ++count) {
+                        IF_DEBUG(int printed =) sprintf(printAt, "%li", count);
+                        gb_assert((printed+printOffset) <= 8);
+                        if (!lookup_an_revers(aisc_main, test_short)) foundUnused = true; // name does not exist
+                    }
+                }
             }
+
+            // if no unused name found, create one with alpha-chars
+            if (!foundUnused) {
+                strcpy(test_short, test_short_dup);
+
+                long        count2  = count-100000; // 100000 numbers were used above
+                char       *printAt = test_short+3;
+                const char *base36  = "0123456789abcdefghijklmnopqrstuvwxyz";
+
+                printAt[5] = 0;
+
+                for (; !foundUnused && count2<16796160; ++count2) { // 16796160 = 36^4*10
+                    // now print count2 converted to base 36
+
+                    int c = count2;
+                    for (int pos = 0; pos<5; ++pos) {
+                        int nextc = c/36;
+                        int rest  = c-36*nextc;
+
+                        printAt[4-pos] = base36[rest];
+                        c              = nextc;
+
+                        gb_assert(pos != 4 || c == 0);
+                    }
+
+                    if (!lookup_an_revers(aisc_main, test_short)) foundUnused = true; // name does not exist
+                }
+
+                count = count2+100000;
+            }
+
+            gb_assert(foundUnused);
+            GBS_write_hash(nameModHash, test_short_dup, count);
+            GBS_optimize_hash(nameModHash);
+
+            free(test_short_dup);
         }
-        
+
+        assert_alnum(test_short);
+
         shrt = strdup(test_short);
         info.add_short(locs, shrt);
 
@@ -654,6 +841,7 @@ extern "C" aisc_string get_short(AN_local *locs)
         free(second_advice);
     }
 
+    assert_alnum(shrt);
     return shrt;
 }
 
@@ -699,7 +887,7 @@ extern "C" int server_save(AN_main *main, int dummy)
     return 0;
 }
 
-#if defined(DEBUG)
+#if defined(DEBUG) && 0
 static void check_list(AN_shorts *start) {
     int count = 0;
     while (++count) {
@@ -715,19 +903,18 @@ static void check_list(AN_shorts *start) {
 }
 #endif // DEBUG
 
-static void check_for_broken_short_names(AN_main *main) {
+static void check_for_case_error(AN_main *main) {
     // test for duplicated names or name parts (only differing in case)
     // such names were created by old name server versions
 
     bool case_error_occurred = false;
     int  idents_changed      = 0;
-
     // first check name parts
     for (AN_shorts *shrt = main->shorts1; shrt; ) {
         AN_shorts *next  = shrt->next;
-        AN_shorts *found = an_find_shrt(main->shorts1, shrt->shrt);
+        AN_shorts *found = an_find_shrt_prefix(shrt->shrt);
         if (found != shrt) {
-            fprintf(stderr, "- Correcting case-error in name-database: '%s' equals '%s'\n",
+            fprintf(stderr, "- Correcting error in name-database: '%s' equals '%s'\n",
                     found->shrt, shrt->shrt);
             an_remove_short(shrt);
             case_error_occurred = true;
@@ -741,7 +928,14 @@ static void check_for_broken_short_names(AN_main *main) {
         AN_revers *found = lookup_an_revers(main, shrt->shrt);
 
         if (found && (shrt->acc && found->acc && an_stricmp(shrt->acc, found->acc) != 0)) {
-            fprintf(stderr, "- Correcting case-error in name-database: '%s' equals '%s' (but acc differs)\n",
+            fprintf(stderr, "- Correcting error in name-database: '%s' equals '%s' (but acc differs)\n",
+                    found->mh.ident, shrt->shrt);
+
+            an_remove_short(shrt);
+            case_error_occurred = true;
+        }
+        else if (found && (shrt->add_id && found->add_id && an_stricmp(shrt->add_id, found->add_id) != 0)) {
+            fprintf(stderr, "- Correcting error in name-database: '%s' equals '%s' (but add_id differs)\n",
                     found->mh.ident, shrt->shrt);
 
             an_remove_short(shrt);
@@ -752,14 +946,14 @@ static void check_for_broken_short_names(AN_main *main) {
             if (!self_find) { // stored with wrong key (not lowercase)
                 aisc_unlink((struct_dllheader_ext*)shrt);
                 an_strlwr(shrt->mh.ident);
-                aisc_link((struct_dllpublic_ext*)&(main->pnames), (struct_dllheader_ext*)shrt);
+                aisc_link(&main->pnames, shrt);
                 main->touched = 1;
-                
+
                 case_error_occurred = true;
                 idents_changed++;
             }
             else if (self_find != shrt) {
-                fprintf(stderr, "- Correcting case-error in name-database: '%s' equals '%s' (case-difference in full_name or acc)\n",
+                fprintf(stderr, "- Correcting error in name-database: '%s' equals '%s' (case-difference in full_name or acc)\n",
                         shrt->mh.ident, self_find->mh.ident);
                 an_remove_short(shrt);
                 case_error_occurred = true;
@@ -821,6 +1015,37 @@ static void check_for_broken_short_names(AN_main *main) {
     }
 }
 
+static void check_for_illegal_chars(AN_main *main) {
+    // test for names containing illegal characters
+    int illegal_names = 0;
+
+    // first check name parts
+    for (AN_shorts *shrt = main->shorts1; shrt; ) {
+        AN_shorts *next = shrt->next;
+        if (!stralnum(shrt->shrt)) {
+            fprintf(stderr, "- Fixing illegal chars in '%s'\n", shrt->shrt);
+            an_remove_short(shrt);
+            illegal_names++;
+        }
+        shrt = next;
+    }
+    // then check full short-names
+    for (AN_shorts *shrt = main->names; shrt; ) {
+        AN_shorts *next  = shrt->next;
+        if (!stralnum(shrt->shrt)) {
+            fprintf(stderr, "- Fixing illegal chars in '%s'\n", shrt->shrt);
+            an_remove_short(shrt);
+            illegal_names++;
+        }
+        shrt = next;
+    }
+    
+    if (illegal_names>0) {
+        fprintf(stderr, "* Removed %i names containing illegal characters.\n"
+                "=> This leads to name changes when generating new names (which is recommended now).\n", illegal_names);
+    }
+}
+
 static GB_ERROR server_load(AN_main *main)
 {
     FILE      *file;
@@ -835,6 +1060,7 @@ static GB_ERROR server_load(AN_main *main)
         error = GBS_global_string("No such file '%s'", main->server_file);
     }
     else {
+        fprintf(stderr, "* Loading %s\n", main->server_file);
         int err_code = load_AN_main(main,file);
         if (err_code) {
             error = GBS_global_string("Error #%i while loading '%s'", err_code, main->server_file);
@@ -842,26 +1068,47 @@ static GB_ERROR server_load(AN_main *main)
     }
 
     if (!error) {
+        fprintf(stderr, "* Parsing data\n");
+        long nameCount =  0;
         for (shrt = main->names; shrt; shrt = shrt->next) {
-            revers = create_AN_revers();
-            revers->mh.ident = an_strlwr(strdup(shrt->shrt));
+            revers            = create_AN_revers();
+            revers->mh.ident  = an_strlwr(strdup(shrt->shrt));
             revers->full_name = strdup(shrt->full_name);
-            revers->acc  = strdup(shrt->acc);
-            aisc_link((struct_dllpublic_ext*)&(main->prevers),(struct_dllheader_ext*)revers);
+            revers->acc       = strdup(shrt->acc);
+            revers->add_id    = shrt->add_id ? strdup(shrt->add_id) : 0;
+            aisc_link(&main->prevers, revers);
+            nameCount++;
         }
-        check_for_broken_short_names(main);
+
+        int namesDBversion = main->dbversion; // version used to save names.dat
+        fprintf(stderr, "* Loaded NAMEDB v%i (contains %li names)\n", namesDBversion, nameCount);
+
+        if (namesDBversion < SERVER_VERSION) {
+            if (namesDBversion<4) {
+                fprintf(stderr, "* Checking for case-errors\n");
+                check_for_case_error(main);
+                
+                fprintf(stderr, "* Checking for illegal characters\n");
+                check_for_illegal_chars(main);
+            }
+
+            fprintf(stderr, "* NAMEDB version upgrade %i -> %i\n", namesDBversion, SERVER_VERSION);
+            main->dbversion = SERVER_VERSION;
+            main->touched   = 1; // force save
+        }
+
         fprintf(stderr, "ARB_name_server is up.\n");
         main->server_filedate = GB_time_of_file(main->server_file);
     }
     else {
-        main->server_filedate = -1; 
+        main->server_filedate = -1;
     }
     return error;
 }
 
-int names_server_shutdown(int exitcode) {
+void names_server_shutdown(int exitcode) __attribute__((noreturn));
+void names_server_shutdown(int exitcode) {
     aisc_server_shutdown_and_exit(AN_global.server_communication, exitcode); // never returns
-    return 0;
 }
 
 int names_server_save(void) {
@@ -885,6 +1132,19 @@ extern "C" int server_shutdown(AN_main *pm, aisc_string passwd){
     return 0;
 }
 
+static void usage(const char *exeName, const char *err) __attribute__((noreturn));
+static void usage(const char *exeName, const char *err) {
+    printf("ARB nameserver v%i\n"
+           "Usage: %s command server-parameters\n"
+           "command = -boot\n"
+           "          -kill\n"
+           "          -look\n"
+           , SERVER_VERSION, exeName);
+    arb_print_server_params();
+    if (err) printf("Error: %s\n", err);
+    exit(-1);
+}
+
 int main(int argc,char **argv)
 {
     char              *name;
@@ -892,34 +1152,32 @@ int main(int argc,char **argv)
     struct Hs_struct  *so;
     struct arb_params *params;
 
-    params = arb_trace_argv(&argc,argv);
-    if (!params->default_file) {
-        printf("No file: Syntax: %s -boot/-kill/-look -dfile\n",argv[0]);
-        exit(-1);
-    }
+    params                 = arb_trace_argv(&argc,argv);
+    const char *executable = argv[0];
 
-    if (argc ==1) {
+    if (!params->default_file) usage(executable, "Missing default file");
+
+    if (argc==1) { // default command is '-look'
         char flag[]="-look";
         argv[1] = flag;
         argc = 2;
     }
 
-    if (argc!=2) {
-        printf("Wrong Parameters %i Syntax: %s -boot/-kill/-look -dfile\n",argc,argv[0]);
-        exit(-1);
-    }
+    if (argc!=2) usage(executable, "Too many parameters");
 
     aisc_main = create_AN_main();
     /***** try to open com with any other pb server ******/
     if (params->tcp) {
         name = params->tcp;
-    }else{
-        if( !(name=(char *)GBS_read_arb_tcp("ARB_NAME_SERVER")) ){
+    }
+    else {
+        const char *cname = GBS_read_arb_tcp("ARB_NAME_SERVER");
+
+        if (!cname) {
             GB_print_error();
             exit(-1);
-        }else{
-            name=strdup(name);
         }
+        name = strdup(cname);
     }
 
     AN_global.cl_link = (aisc_com *)aisc_open(name, (long *)&AN_global.cl_main,AISC_MAGIC_NUMBER);
@@ -931,7 +1189,7 @@ int main(int argc,char **argv)
             exit(0);
         }
 
-        printf("There is another activ server. I try to kill him...\n");
+        printf("There is another active nameserver. I try to kill it..\n");
         aisc_nput(AN_global.cl_link, AN_MAIN, AN_global.cl_main,
                   MAIN_SHUTDOWN, "ldfiojkherjkh",
                   NULL);
@@ -956,37 +1214,72 @@ int main(int argc,char **argv)
     aisc_main->server_file     = strdup(params->default_file);
     aisc_main->server_filedate = GB_time_of_file(aisc_main->server_file);
 
-    GB_ERROR error             = server_load(aisc_main);
-    long     accept_calls_init = NAME_SERVER_TIMEOUT/long(NAME_SERVER_SLEEP);
-    long     accept_calls      = accept_calls_init;
+    GB_ERROR error = server_load(aisc_main);
 
-    if (aisc_main->touched) server_save(aisc_main, 0);
+    if (!error) {
+        const char *field     = params->field;
+        if (field == 0) field = ""; // default to no field
+        
+        if (aisc_main->add_field == 0) {
+            printf("* add. field not defined yet -> using '%s'\n", field);
+            aisc_main->add_field = strdup(field);
+            aisc_main->touched   = 1;
+        }
+        else {
+            if (strcmp(aisc_main->add_field, field) != 0) {
+                if (aisc_main->add_field[0]) {
+                    error = GBS_global_string("This nameserver has to be started with -f%s", aisc_main->add_field);
+                }
+                else {
+                    error = "This nameserver has to be started w/o -f option";
+                }
+            }
+        }
+    }
+
+    long accept_calls_init = NAME_SERVER_TIMEOUT/long(NAME_SERVER_SLEEP);
+    long accept_calls      = accept_calls_init;
+    bool isTimeout         = true;
+
+    if (!error && aisc_main->touched) server_save(aisc_main, 0);
 
     while (!error && accept_calls>0) {
         aisc_accept_calls(so);
-        accept_calls--;
 
         if (aisc_main->ploc_st.cnt <=0) { // timeout passed and no clients
+            accept_calls--;
+
             long server_date = GB_time_of_file(aisc_main->server_file);
             if (server_date == 0 && aisc_main->server_filedate != 0) {
                 fprintf(stderr, "ARB_name_server data has been deleted.\n");
                 accept_calls = 0;
+                isTimeout    = false;
             }
             if (server_date>aisc_main->server_filedate) {
                 fprintf(stderr, "ARB_name_server data changed on disc.\n");
                 accept_calls = 0;
+                isTimeout    = false;
             }
             else if (aisc_main->touched) {
                 server_save(aisc_main,0);
                 accept_calls = accept_calls_init;
             }
         }
+        else { // got a client
+            accept_calls = accept_calls_init;
+        }
     }
 
     if (error) {
         fprintf(stderr, "Error in ARB_name_server: %s\n", error);
     }
+    else if (accept_calls == 0) {
+        if (isTimeout) {
+            fprintf(stderr, "Been idle for %i minutes.\n", int(NAME_SERVER_TIMEOUT/60));
+        }
+    }
 
     printf("ARB_name_server terminating...\n");
+    if (nameModHash) GBS_free_hash(nameModHash);
     names_server_shutdown(error ? EXIT_FAILURE : EXIT_SUCCESS); // never returns
 }
