@@ -1,9 +1,13 @@
 
-#define MAX_TRY 20
-#define RETRY_SLEEP 3 // in seconds
+#define MAX_TRY     20
+#define RETRY_SLEEP 3           // in seconds
 
 // Lifetime of nameserver in seconds
+#if defined(DEBUG)
+#define NAME_SERVER_TIMEOUT (1*60) // 1 minute (for debugging)
+#else
 #define NAME_SERVER_TIMEOUT (15*60) // 15 minutes
+#endif // DEBUG
 
 // Sleeptime of nameserver in seconds
 #define NAME_SERVER_SLEEP 5
