@@ -42,7 +42,7 @@
 #include "nt_edconf.hxx"
 #include "ap_pos_var_pars.hxx"
 #include "nt_import.hxx"
-#include "nt_date.h"
+#include <arb_version.h>
 #include "nt_internal.h"
 #include <st_window.hxx>
 #include <probe_design.hxx>
@@ -53,6 +53,7 @@
 
 #include "nt_concatenate.hxx"
 #include "seq_quality.h"
+#include "NT_trackAliChanges.h"
 
 #ifndef ARB_ASSERT_H
 #include <arb_assert.h>
@@ -155,16 +156,9 @@ void create_export_nds_awars(AW_root *awr, AW_default def)
     awr->awar_int( AWAR_EXPORT_NDS"/tabbed", 0, def);
 }
 
-static void AWAR_SEARCH_BUTTON_TEXT_change_cb(AW_root *awr) {
-    char *value  = awr->awar(AWAR_SPECIES_NAME)->read_string();
-
-    if (strcmp(value,"")==0) {
-        awr->awar(AWAR_SEARCH_BUTTON_TEXT)->write_string("Species Information");
-    }
-    else {
-        awr->awar(AWAR_SEARCH_BUTTON_TEXT)->write_string(value);
-    }
-
+static void AWAR_INFO_BUTTON_TEXT_change_cb(AW_root *awr) {
+    char *value = awr->awar(AWAR_SPECIES_NAME)->read_string();
+    awr->awar(AWAR_INFO_BUTTON_TEXT)->write_string(value[0] ? value : "Species Info");
     free(value);
 }
 
@@ -183,8 +177,8 @@ void create_all_awars(AW_root *awr, AW_default def)
     awr->awar_string( AWAR_SAI_NAME, "" ,   gb_main);
     awr->awar_string( AWAR_SAI_GLOBAL, "" ,   gb_main);
     awr->awar_string( AWAR_MARKED_SPECIES_COUNTER, "unknown" ,  gb_main);
-    awr->awar_string( AWAR_SEARCH_BUTTON_TEXT, "Species Information" ,  gb_main);
-    awr->awar(AWAR_SPECIES_NAME)->add_callback(AWAR_SEARCH_BUTTON_TEXT_change_cb);
+    awr->awar_string( AWAR_INFO_BUTTON_TEXT, "Species Info" ,  gb_main);
+    awr->awar(AWAR_SPECIES_NAME)->add_callback(AWAR_INFO_BUTTON_TEXT_change_cb);
     awr->awar_int( AWAR_NTREE_TITLE_MODE, 1);
 
     awr->awar_string(AWAR_SAI_COLOR_STR, "", gb_main); //sai visualization in probe match
@@ -193,6 +187,8 @@ void create_all_awars(AW_root *awr, AW_default def)
     EXP_create_awars(awr, def);
     AWT_create_db_browser_awars(awr, def);
 
+    AW_create_namesadmin_awars(awr, gb_main);
+
     awr->awar_int( AWAR_SECURITY_LEVEL, 0, def);
     awr->awar(AWAR_SECURITY_LEVEL)->add_callback(nt_changesecurity);
 #if defined(DEBUG) && 0
@@ -210,11 +206,12 @@ void create_all_awars(AW_root *awr, AW_default def)
     create_cprofile_var(awr,def);
     NT_create_transpro_variables(awr,def);
     NT_build_resort_awars(awr,def);
+    NT_create_trackAliChanges_Awars(awr, gb_main);
 
     NT_create_alignment_vars(awr,def);
     create_nds_vars(awr,def,gb_main);
     create_export_nds_awars(awr,def);
-    AWTC_create_rename_variables(awr,def);
+    AWTC_create_rename_awars(awr,def);
     create_check_gcg_awars(awr,def);
     awt_create_dtree_awars(awr,gb_main);
 
@@ -232,31 +229,28 @@ void create_all_awars(AW_root *awr, AW_default def)
 }
 
 
-void
-nt_exit(AW_window *aw_window){
-#ifdef DEBUG
-    exit(0);
-#endif
-    AWUSE(aw_window);
+void nt_exit(AW_window * /*aw_window*/) {
     if (gb_main) {
         if (GB_read_clients(gb_main)>=0) {
-#ifdef NDEBUG
             if (GB_read_clock(gb_main) > GB_last_saved_clock(gb_main)){
                 long secs;
                 secs = GB_last_saved_time(gb_main);
+
+#if defined(DEBUG)
+                secs =  GB_time_of_day(); // simulate "just saved"
+#endif // DEBUG
+
                 if (secs) {
                     secs = GB_time_of_day() - secs;
                     if (secs>10){
                         sprintf(AW_ERROR_BUFFER,"You last saved your data %li:%li minutes ago\nSure to quit ?",secs/60,secs%60);
-                        if (aw_message(AW_ERROR_BUFFER,
-                                       "QUIT ARB,DO NOT QUIT")) return;
+                        if (aw_message(AW_ERROR_BUFFER, "QUIT ARB,DO NOT QUIT")) return;
                     }
-                }else{
-                    if (aw_message("You never saved any data\nSure to quit ???",
-                                   "QUIT ARB,DO NOT QUIT")) return;
+                }
+                else {
+                    if (aw_message("You never saved any data\nSure to quit ???", "QUIT ARB,DO NOT QUIT")) return;
                 }
             }
-#endif // NDEBUG
         }
         GBCMS_shutdown(gb_main);
         GB_exit(gb_main);
@@ -264,9 +258,7 @@ nt_exit(AW_window *aw_window){
     exit(0);
 }
 
-void
-NT_save_cb(AW_window *aww)
-{
+void NT_save_cb(AW_window *aww) {
     char *filename = aww->get_root()->awar(AWAR_DB_PATH)->read_string();
     GB_ERROR error = GB_save(gb_main, filename, "b");
     delete filename;
@@ -275,9 +267,7 @@ NT_save_cb(AW_window *aww)
 }
 
 
-void
-NT_save_quick_cb(AW_window *aww)
-{
+void NT_save_quick_cb(AW_window *aww) {
     char *filename = aww->get_root()->awar(AWAR_DB_PATH)->read_string();
     GB_ERROR error = GB_save_quick(gb_main,filename);
     free(filename);
@@ -285,9 +275,7 @@ NT_save_quick_cb(AW_window *aww)
     else awt_refresh_selection_box(aww->get_root(), "tmp/nt/arbdb");
 }
 
-void
-NT_save_quick_as_cb(AW_window *aww)
-{
+void NT_save_quick_as_cb(AW_window *aww) {
     char *filename = aww->get_root()->awar(AWAR_DB_PATH)->read_string();
     GB_ERROR error = GB_save_quick_as(gb_main, filename);
     free(filename);
@@ -575,7 +563,10 @@ void NT_submit_mail(AW_window *aww, AW_CL cl_awar_base) {
     else{
         fprintf(mail,"%s\n",text);
         fprintf(mail,"------------------------------\n");
-        fprintf(mail,"VERSION       :" DATE "\n");
+        fprintf(mail,"VERSION       :" ARB_VERSION "\n");
+#if defined(SHOW_WHERE_BUILD)
+        fprintf(mail,"BUILD_BY      :" BUILD_USER "@" BUILD_HOST "\n");        
+#endif // SHOW_WHERE_BUILD
         fprintf(mail,"SYSTEMINFO    :\n");
         fclose(mail);
 
@@ -699,12 +690,21 @@ void NT_mark_long_branches(AW_window *aww, AW_CL ntwcl){
     char *val = aw_input("Enter relativ diff [0 .. 5.0]",0);
     if (!val) return;   // operation cancelled
     NT_mark_all_cb(aww,(AW_CL)ntw, (AW_CL)0);
-    // NT_unmark_all_cb(aww,ntw);
     AWT_TREE(ntw)->tree_root->mark_long_branches(gb_main,atof(val));
     AWT_TREE(ntw)->tree_root->compute_tree(ntw->gb_main);
     free(val);
     ntw->refresh();
 }
+
+void NT_mark_duplicates(AW_window *aww, AW_CL ntwcl){
+    GB_transaction dummy(gb_main);
+    AWT_canvas *ntw = (AWT_canvas *)ntwcl;
+    NT_mark_all_cb(aww,(AW_CL)ntw, (AW_CL)0);
+    AWT_TREE(ntw)->tree_root->mark_duplicates(gb_main);
+    AWT_TREE(ntw)->tree_root->compute_tree(ntw->gb_main);
+    ntw->refresh();
+}
+
 void NT_justify_branch_lenghs(AW_window *, AW_CL cl_ntw, AW_CL){
     GB_transaction dummy(gb_main);
     AWT_canvas *ntw = (AWT_canvas *)cl_ntw;
@@ -722,10 +722,10 @@ void NT_fix_database(AW_window *) {
     if (err) aw_message(err);
 }
 
-static void relink_pseudo_species_to_organisms(GBDATA *&ref_gb_node, char *&ref_name) {
+static void relink_pseudo_species_to_organisms(GBDATA *&ref_gb_node, char *&ref_name, GB_HASH *organism_hash) {
     if (ref_gb_node) {
         if (GEN_is_pseudo_gene_species(ref_gb_node)) {
-            GBDATA *gb_organism = GEN_find_origin_organism(ref_gb_node);
+            GBDATA *gb_organism = GEN_find_origin_organism(ref_gb_node, organism_hash);
             if (gb_organism) {
                 GBDATA *gb_name = GB_find(gb_organism, "name", 0, down_level);
                 if (gb_name) {
@@ -743,10 +743,12 @@ void NT_pseudo_species_to_organism(AW_window *, AW_CL ntwcl){
     GB_transaction  dummy(gb_main);
     AWT_canvas     *ntw = (AWT_canvas *)ntwcl;
     if (AWT_TREE(ntw)->tree_root){
-        AWT_TREE(ntw)->tree_root->relink_tree(ntw->gb_main, relink_pseudo_species_to_organisms);
+        GB_HASH *organism_hash = GBT_create_organism_hash(gb_main);
+        AWT_TREE(ntw)->tree_root->relink_tree(ntw->gb_main, relink_pseudo_species_to_organisms, organism_hash);
         AWT_TREE(ntw)->tree_root->compute_tree(ntw->gb_main);
         AWT_TREE(ntw)->save(ntw->gb_main,0,0,0);
         ntw->refresh();
+        GBS_free_hash(organism_hash);
     }
 }
 
@@ -861,17 +863,19 @@ void NT_popup_species_window(AW_window *aww, AW_CL, AW_CL) {
 
 //--------------------------------------- to increase the area of display --------------------
 static int windowHeight = 0;
-static void title_mode_changed(AW_root *aw_root, AW_window *aww)
-{
-    int title_mode = aw_root->awar(AWAR_NTREE_TITLE_MODE)->read_int();
 
-    if (title_mode==0) {
-        aww->set_info_area_height(25);
-    }
-    else {
-        aww->set_info_area_height(windowHeight);
-    }
-}
+// static void title_mode_changed(AW_root *aw_root, AW_window *aww)
+// {
+//     int title_mode = aw_root->awar(AWAR_NTREE_TITLE_MODE)->read_int();
+
+//     if (title_mode==0) {
+//         aww->set_info_area_height(25);
+//     }
+//     else {
+//         aww->set_info_area_height(windowHeight);
+//     }
+// }
+
 //--------------------------------------------------------------------------------------------------
 #if defined(DEBUG)
 void NT_rename_test(AW_window *, AW_CL cl_gb_main, AW_CL) {
@@ -1028,6 +1032,156 @@ void NT_dump_gcs(AW_window *aww, AW_CL, AW_CL) {
 
 //--------------------------------------------------------------------------------------------------
 
+static GBT_TREE *fixDeletedSon(GBT_TREE *tree) {
+    // fix tree after one son has been deleted
+    // (Note: this function only works correct for trees with minimum element size!)
+    GBT_TREE *delNode = tree;
+
+    if (delNode->leftson) {
+        nt_assert(!delNode->rightson);
+        tree             = delNode->leftson;
+        delNode->leftson = 0;
+    }
+    else {
+        nt_assert(!delNode->leftson);
+        nt_assert(delNode->rightson);
+        
+        tree              = delNode->rightson;
+        delNode->rightson = 0;
+    }
+
+    // now tree is the new tree
+    tree->father = delNode->father;
+
+    if (delNode->remark_branch && !tree->remark_branch) { // rescue remarks if possible
+        tree->remark_branch    = delNode->remark_branch;
+        delNode->remark_branch = 0;
+    }
+    if (delNode->gb_node && !tree->gb_node) { // rescue group if possible
+        tree->gb_node    = delNode->gb_node;
+        delNode->gb_node = 0;
+    }
+
+    delNode->is_leaf = GB_TRUE; // don't try recursive delete
+
+    if (delNode->father) { // not root
+        GBT_delete_tree(delNode);
+    }
+    else { // root node
+        if (delNode->tree_is_one_piece_of_memory) {
+            // dont change root -> copy instead
+            memcpy(delNode, tree, sizeof(GBT_TREE));
+            tree = delNode;
+        }
+        else {
+            GBT_delete_tree(delNode);
+        }
+    }
+    return tree;
+}
+
+static GBT_TREE *remove_leafs(GBT_TREE *tree, long mode, GB_HASH *species_hash, int& removed, int& groups_removed) {
+    if (tree->is_leaf) {
+        if (tree->name) {
+            GBDATA *gb_node    = (GBDATA*)GBS_read_hash(species_hash, tree->name);
+            bool    deleteSelf = false;
+            if (gb_node) {
+                if (mode & AWT_REMOVE_MARKED) {
+                    if (GB_read_flag(gb_node)) deleteSelf = true;
+                }
+            }
+            else { // zombie
+                if (mode & AWT_REMOVE_DELETED) deleteSelf = true;
+            }
+
+            if (deleteSelf) {
+                GBT_delete_tree(tree);
+                removed++;
+                tree = 0;
+            }
+        }
+    }
+    else {
+        tree->leftson  = remove_leafs(tree->leftson, mode, species_hash, removed, groups_removed);
+        tree->rightson = remove_leafs(tree->rightson, mode, species_hash, removed, groups_removed);
+
+        if (tree->leftson) {
+            if (!tree->rightson) { // right son deleted
+                tree = fixDeletedSon(tree);
+            }
+            // otherwise no son deleted
+        }
+        else if (tree->rightson) { // left son deleted
+            tree = fixDeletedSon(tree);
+        }
+        else {                  // everything deleted -> delete self
+            if (tree->gb_node) groups_removed++;
+            tree->is_leaf = GB_TRUE;
+            GBT_delete_tree(tree);
+            tree          = 0;
+        }
+    }
+
+    return tree;
+}
+
+void NT_alltree_remove_leafs(AW_window *, AW_CL cl_mode, AW_CL cl_gb_main) {
+    GBDATA *gb_main = (GBDATA*)cl_gb_main;
+    long    mode    = (long)cl_mode;
+
+    GB_transaction ta(gb_main);
+
+    int    treeCount;
+    char **tree_names = GBT_get_tree_names_and_count(gb_main, &treeCount);
+
+    if (tree_names) {
+        aw_openstatus("Deleting from trees");
+        GB_HASH *species_hash = GBT_create_species_hash(gb_main);
+
+        for (int t = 0; t<treeCount; t++) {
+            GB_ERROR  error = 0;
+            aw_status(t/double(treeCount));
+            aw_status(tree_names[t]);
+            GBT_TREE *tree  = GBT_read_tree(gb_main, tree_names[t], -sizeof(GBT_TREE));
+            if (!tree) {
+                aw_message(GBS_global_string("Can't load tree '%s' - skipped", tree_names[t]));
+            }
+            else {
+                int removed        = 0;
+                int groups_removed = 0;
+
+                tree = remove_leafs(tree, mode, species_hash, removed, groups_removed);
+
+                gb_assert(removed >= groups_removed);
+
+                if (!tree) {
+                    aw_message(GBS_global_string("'%s' would disappear. Please delete tree manually.", tree_names[t]));
+                }
+                else {
+                    if (removed>0) {
+                        error = GBT_write_tree(gb_main, 0, tree_names[t], tree);
+                        if (groups_removed>0) {
+                            aw_message(GBS_global_string("Removed %i species and %i groups from '%s'", removed, groups_removed, tree_names[t]));
+                        }
+                        else {
+                            aw_message(GBS_global_string("Removed %i species from '%s'", removed, tree_names[t]));
+                        }
+                    }
+                    GBT_delete_tree(tree);
+                }
+            }
+            if (error) aw_message(error);
+        }
+        aw_status(1.0);
+
+        GBS_free_names(tree_names);
+        GBS_free_hash(species_hash);
+        aw_closestatus();
+    }
+}
+
+//--------------------------------------------------------------------------------------------------
+
 // ##########################################
 // ##########################################
 // ###                                    ###
@@ -1067,9 +1221,10 @@ AW_window * create_nt_main_window(AW_root *awr, AW_CL clone){
 
     AWT_canvas *ntw;
     {
-        AP_tree_sort  old_sort_type = nt.tree->tree_sort;
+        AP_tree_sort old_sort_type = nt.tree->tree_sort;
         nt.tree->set_tree_type(AP_LIST_SIMPLE); // avoid NDS warnings during startup
-        ntw = new AWT_canvas(gb_main,(AW_window *)awm,nt.tree, aw_gc_manager,awar_tree) ;
+
+        ntw = new AWT_canvas(gb_main, awm, nt.tree, aw_gc_manager, awar_tree);
         nt.tree->set_tree_type(old_sort_type);
         ntw->set_mode(AWT_MODE_SELECT);
     }
@@ -1138,7 +1293,7 @@ AW_window * create_nt_main_window(AW_root *awr, AW_CL clone){
 
             awm->insert_sub_menu(  0,"Import",      "I" );
             {
-                AWMIMT("import_seq",    "Import sequences and fields (ARB)","I","arb_import.hlp",AWM_ALL, NT_import_sequences,0,0);
+                AWMIMT("import_seq", "Import sequences and fields","I","arb_import.hlp",AWM_ALL, NT_import_sequences,0,0);
                 GDE_load_menu(awm,"import");
 #if defined(DEVEL_LOTHAR) || defined(DEVEL_RALF)
                 AWMIMT("MAUS", "Filter through MAUS..", "M", "MAUS.hlp", AWM_ALL, AW_POPUP, (AW_CL)NT_create_MAUS_window, 0);
@@ -1162,7 +1317,7 @@ AW_window * create_nt_main_window(AW_root *awr, AW_CL clone){
             {
                 AWMIMT( "registration", "Registration",         "R","registration.hlp", AWM_EXP, (AW_CB)AW_POPUP,   (AW_CL)NT_submit_bug, 0 );
                 AWMIMT( "bug_report",   "Bug report",           "B","registration.hlp", AWM_ALL, (AW_CB)AW_POPUP,   (AW_CL)NT_submit_bug, 1 );
-                AWMIMT( "version_info", "Version info (" DATE ")",  "V","version.hlp",  AWM_ALL, (AW_CB)AW_POPUP_HELP,  (AW_CL)"version.hlp", 0 );
+                AWMIMT( "version_info", "Version info (" ARB_VERSION ")",  "V","version.hlp",  AWM_ALL, (AW_CB)AW_POPUP_HELP,  (AW_CL)"version.hlp", 0 );
             }
             awm->close_sub_menu();
 #if 0
@@ -1270,33 +1425,23 @@ AW_window * create_nt_main_window(AW_root *awr, AW_CL clone){
 
             awm->insert_sub_menu(0, "Align Sequences",  "S");
             {
-                AWMIMT("arb_align",   "Align sequence into an existing alignment",         "A", "align.hlp",       AWM_ALL, (AW_CB) AW_POPUP_HELP,(AW_CL)"align.hlp",0   );
-                AWMIMT("realign_dna", "Realign nucleic acid according to aligned protein", "R", "realign_dna.hlp", AWM_ALL, AW_POPUP, (AW_CL)NT_create_realign_dna_window, 0);
+                AWMIMT("arb_align",   "Align sequence into an existing alignment",         "A", "align.hlp",       AWM_ALL, (AW_CB) AW_POPUP_HELP, (AW_CL)"align.hlp",                  0);
+                AWMIMT("realign_dna", "Realign nucleic acid according to aligned protein", "R", "realign_dna.hlp", AWM_ALL, AW_POPUP,              (AW_CL)NT_create_realign_dna_window, 0);
                 GDE_load_menu(awm,"align");
             }
             awm->close_sub_menu();
-            AWMIMT("seq_concat","Concatenate Sequences/Alignments", "C","concatenate_align.hlp", AWM_EXP,    AW_POPUP, (AW_CL)NT_createConcatenationWindow, (AW_CL)ntw );
+            AWMIMT("seq_concat",    "Concatenate Sequences/Alignments", "C", "concatenate_align.hlp", AWM_EXP, AW_POPUP, (AW_CL)NT_createConcatenationWindow,     (AW_CL)ntw );
+            AWMIMT("track_changes", "Track alignment changes",          "k", "track_ali_changes.hlp", AWM_ALL, AW_POPUP, (AW_CL)NT_create_trackAliChanges_window, 0)     ;
             SEP________________________SEP();
 
             AWMIMT("dna_2_pro", "Perform translation",                     "t", "translate_dna_2_pro.hlp", AWM_PRO, AW_POPUP, (AW_CL)NT_create_dna_2_pro_window, 0      );
             AWMIMT("arb_dist",  "Compare sequences using Distance Matrix", "D", "dist.hlp",                AWM_SEQ, (AW_CB)NT_system_cb,    (AW_CL)"arb_dist &",    0);
             SEP________________________SEP();
 
-#if defined(DEBUG)
-#if defined(DEVEL_RALF)
-#warning sequence quality check for big databases needs to be fixed
-#endif // DEVEL_RALF
-            AWMIMT("seq_quality", "Check Sequence Quality",    "",  "seq_quality.hlp",   AWM_EXP,  AW_POPUP, (AW_CL)SQ_create_seq_quality_window, 0);
-#endif // DEBUG
-
-#if defined(DEBUG)
-#if defined(DEVEL_RALF)
-#warning chimere check needs to be fixed
-#endif // DEVEL_RALF
-            AWMIMT("seq_quality", "Chimere Check [broken!]",   "m", "check_quality.hlp", AWM_SEQ2, AW_POPUP, (AW_CL)st_create_quality_check_window, (AW_CL)gb_main);
+            AWMIMT("seq_quality",   "Check Sequence Quality", "Q", "seq_quality.hlp",   AWM_EXP,  AW_POPUP, (AW_CL)SQ_create_seq_quality_window,   0)     ;
+            AWMIMT("chimera_check", "Chimera Check",          "m", "check_quality.hlp", AWM_SEQ2, AW_POPUP, (AW_CL)st_create_quality_check_window, (AW_CL)gb_main);
 
             SEP________________________SEP();
-#endif // DEBUG
 
             GDE_load_menu(awm,"pretty_print");
         }
@@ -1324,10 +1469,6 @@ AW_window * create_nt_main_window(AW_root *awr, AW_CL clone){
                     AWMIMT("pos_var_dist",          "Positional variability (distance method)",     "P", "pos_variability.ps", AWM_EXP, AW_POPUP,      (AW_CL)AP_open_cprofile_window,          0);
                     AWMIMT("count_different_chars", "Count different chars/column",                 "C", "count_chars.hlp",    AWM_EXP, NT_system_cb2, (AW_CL)"arb_count_chars",                0);
                     AWMIMT("export_pos_var",        "Export Column Statistic (GNUPLOT format)",     "E", "csp_2_gnuplot.hlp",  AWM_EXP, AW_POPUP,      (AW_CL)AP_open_csp_2_gnuplot_window,     0);
-#ifdef DEVEL_YADHU
-#warning will deal later &^%*^*!!
-                    // AWMIMT("conservation_profile",  "Display Conservation Profile (Using GNUPLOT)", "D", "conser_profile.hlp", AWM_EXP, AW_POPUP,      (AW_CL)AP_openConservationProfileWindow, 0); // spelling fixed [ralf]
-#endif
                 }
                 awm->close_sub_menu();
             }
@@ -1370,6 +1511,12 @@ AW_window * create_nt_main_window(AW_root *awr, AW_CL clone){
             AWMIMT("tree_remove_deleted", "Remove zombies", "z", "trm_del.hlp",    AWM_TREE, (AW_CB)NT_remove_leafs, (AW_CL)ntw, AWT_REMOVE_DELETED );
             AWMIMT("tree_remove_marked",  "Remove marked",  "m", "trm_mrkd.hlp",   AWM_TREE, (AW_CB)NT_remove_leafs, (AW_CL)ntw, AWT_REMOVE_MARKED );
             AWMIMT("tree_keep_marked",    "Keep marked",    "K", "tkeep_mrkd.hlp", AWM_TREE, (AW_CB)NT_remove_leafs, (AW_CL)ntw, AWT_REMOVE_NOT_MARKED|AWT_REMOVE_DELETED);
+#if defined(DEVEL_RALF)
+#warning add "remove duplicates from tree"
+            SEP________________________SEP();
+            AWMIMT("all_tree_remove_deleted", "Remove zombies from all trees", "a", "trm_del.hlp", AWM_TREE, (AW_CB)NT_alltree_remove_leafs, AWT_REMOVE_DELETED, (AW_CL)gb_main);
+            AWMIMT("all_tree_remove_marked",  "Remove marked from all trees",  "l", "trm_del.hlp", AWM_TREE, (AW_CB)NT_alltree_remove_leafs, AWT_REMOVE_MARKED, (AW_CL)gb_main);
+#endif // DEVEL_RALF
         }
         awm->close_sub_menu();
 
@@ -1437,6 +1584,7 @@ AW_window * create_nt_main_window(AW_root *awr, AW_CL clone){
         }
         awm->close_sub_menu();
         AWMIMT("mark_long_branches", "Mark long branches", "k", "mark_long_branches.hlp", AWM_EXP, (AW_CB)NT_mark_long_branches, (AW_CL)ntw, 0);
+        AWMIMT("mark_duplicates", "Mark duplicates", "u", "mark_duplicates.hlp", AWM_EXP, (AW_CB)NT_mark_duplicates, (AW_CL)ntw, 0);
 
         SEP________________________SEP();
 
@@ -1456,8 +1604,8 @@ AW_window * create_nt_main_window(AW_root *awr, AW_CL clone){
         SEP________________________SEP();
 
         if (!clone){
-            AWMIMT("print_tree",  "Print tree",          "P", "tree2prt.hlp",  AWM_ALL, (AW_CB)AWT_create_print_window, (AW_CL)ntw, 0         );
-            AWMIMT("tree_2_xfig", "Export tree to XFIG", "X", "tree2file.hlp", AWM_ALL, AW_POPUP, (AW_CL)AWT_create_export_window,  (AW_CL)ntw);
+            AWMIMT("print_tree",  "Print tree",          "P", "tree2prt.hlp",  AWM_ALL, AWT_popup_print_window,       (AW_CL)ntw, 0);
+            AWMIMT("tree_2_xfig", "Export tree to XFIG", "X", "tree2file.hlp", AWM_ALL, AWT_popup_tree_export_window, (AW_CL)ntw, 0);
             SEP________________________SEP();
         }
 
@@ -1475,7 +1623,7 @@ AW_window * create_nt_main_window(AW_root *awr, AW_CL clone){
         // --------------------------------------------------------------------------------
         awm->create_menu(0,"Tools","o","nt_etc.hlp", AWM_ALL);
         {
-            AWMIMT("names_admin",      "Name server admin",    "s","namesadmin.hlp",   AWM_EXP, AW_POPUP, (AW_CL)create_awtc_names_admin_window, 0 );
+            AWMIMT("names_admin",      "Name server admin",    "s","namesadmin.hlp",   AWM_EXP, AW_POPUP, (AW_CL)AW_create_namesadmin_window, (AW_CL)gb_main );
             awm->insert_sub_menu( 0,   "Network", "N");
             {
                 AWMIMT("ors",   "ORS (disabled) ...",      "O","ors.hlp",          AWM_ALL, (AW_CB)NT_system_cb, (AW_CL)"netscape http://pop.mikro.biologie.tu-muenchen.de/ORS/ &", 0);
@@ -1561,130 +1709,200 @@ AW_window * create_nt_main_window(AW_root *awr, AW_CL clone){
 
     awm->set_info_area_height( 250 );
     awm->at(5,2);
-    awm->auto_space(0,-5);
+    awm->auto_space(-2,-2);
     awm->shadow_width(1);
 
-    // path, tree, alignment & searchSpecies buttons:
+    // -------------------------
+    //      create top area
+    // -------------------------
 
     int first_liney;
-    int db_pathx;
-    awm->get_at_position( &db_pathx, &first_liney);
-    awm->callback( AW_POPUP, (AW_CL)NT_create_save_quick_as, (AW_CL)"tmp/nt/arbdb");
-    awm->button_length(19);
-    awm->help_text("saveas.hlp");
-    awm->create_button("SAVE_AS",AWAR_DB_NAME);
+    int leftx;
+    awm->get_at_position( &leftx, &first_liney);
 
-    int db_treex;
-    awm->get_at_position( &db_treex, &first_liney);
+    // ---------------------
+    //      quit + help
+    // ---------------------
+
+    awm->button_length(0);
+    
+    awm->callback(nt_exit);
+    awm->help_text("quit.hlp");
+    awm->create_button("QUIT", "#quit.xpm");
+
+    int db_pathx = awm->get_at_xposition();
+
+    // now fetch the yposition for the 2nd button line
+    awm->at_newline();
+    int second_liney = awm->get_at_yposition();
+
+    awm->callback( AW_help_entry_pressed );
+    awm->help_text("arb_ntree.hlp");
+    awm->create_button("?","#help.xpm");
+
+    // -------------------------
+    //      save + undo/redo
+    // -------------------------
+    
+    awm->callback(NT_save_quick_cb);
+    awm->help_text("save.hlp");
+    awm->create_button("SAVE", "#save.xpm");
+
+    awm->callback( AW_POPUP, (AW_CL)NT_create_save_as, (AW_CL)"tmp/nt/arbdb");
+    awm->help_text("saveas.hlp");
+    awm->create_button("SAVE_AS", "#saveAs.xpm");
+
+    // undo/redo:
+    awm->callback(NT_undo_cb,(AW_CL)GB_UNDO_UNDO,(AW_CL)ntw);
+    awm->help_text("undo.hlp");
+    awm->create_button("UNDO", "#undo.bitmap");
+
+    awm->callback(NT_undo_cb,(AW_CL)GB_UNDO_REDO,(AW_CL)ntw);
+    awm->help_text("undo.hlp");
+    awm->create_button("REDO", "#redo.bitmap");
+
+    int db_pathx2 = awm->get_at_xposition();
+
+    // fetch position for mode help-line: 
+    awm->at_newline();
+    int third_liney = awm->get_at_yposition();
+    
+    awm->at(db_pathx, first_liney);
+    // size of DB-name button is determined by buttons below:
+    awm->at_set_to(AW_FALSE, AW_FALSE, db_pathx2-db_pathx-1, second_liney-first_liney+1);
+    awm->callback( AW_POPUP, (AW_CL)NT_create_save_quick_as, (AW_CL)"tmp/nt/arbdb"); // do sth else
+    awm->help_text("saveas.hlp");
+    awm->create_button("QUICK_SAVE_AS",AWAR_DB_NAME);
+
+    // ----------------------------
+    //      tree + tree display
+    // ----------------------------
+
+#define TREE_BUTTON_OVERSIZE 6
+
+    int db_treex      = awm->get_at_xposition();
+    int second_uppery = second_liney-TREE_BUTTON_OVERSIZE;
+
+    awm->at(db_treex, second_uppery);
+    awm->button_length(0);
+
+    awm->callback((AW_CB)NT_set_tree_style,(AW_CL)ntw,(AW_CL)AP_TREE_RADIAL);
+    awm->help_text("tr_type_radial.hlp");
+    awm->create_button("RADIAL_TREE_TYPE", "#radial.bitmap");
+
+    awm->callback((AW_CB)NT_set_tree_style,(AW_CL)ntw,(AW_CL)AP_TREE_NORMAL);
+    awm->help_text("tr_type_list.hlp");
+    awm->create_button("LIST_TREE_TYPE", "#dendro.bitmap");
+
+    awm->callback((AW_CB)NT_set_tree_style,(AW_CL)ntw,(AW_CL)AP_TREE_IRS);
+    awm->help_text("tr_type_irs.hlp");
+    awm->create_button("FOLDED_LIST_TREE_TYPE", "#dendroIrs.bitmap");
+    
+    awm->callback((AW_CB)NT_set_tree_style, (AW_CL)ntw, (AW_CL)AP_LIST_NDS);
+    awm->help_text("tr_type_nds.hlp");
+    awm->create_button("NO_TREE_TYPE", "#listdisp.bitmap");
+
+    int db_treex2 = awm->get_at_xposition();
+    
+    awm->at(db_treex, first_liney);
+    // size of tree-name button is determined by buttons below:
+    awm->at_set_to(AW_FALSE, AW_FALSE, db_treex2-db_treex-1, second_uppery-first_liney+1);
     awm->callback((AW_CB2)AW_POPUP,(AW_CL)NT_open_select_tree_window,(AW_CL)awar_tree);
-    awm->button_length(19);
     awm->help_text("nt_tree_select.hlp");
     awm->create_button("SELECT_A_TREE", awar_tree);
 
-    int db_alignx;
-    awm->get_at_position( &db_alignx, &first_liney);
+    // --------------------------------------------------
+    //      protection, alignment + editor (and maybe Genome Map)
+    // --------------------------------------------------
+
+    int db_alignx = awm->get_at_xposition();
+
+    // editor and genemap buttons have to be 44x24 sized
+#define EDIT_XSIZE 50
+#define EDIT_YSIZE 30 // same as 'save as' buttons
+
+    // first draw protection:
+    int protectx = db_alignx + 2*EDIT_XSIZE;
+
+    awm->at(protectx+2, first_liney+1);
+    awm->button_length(0);
+    awm->create_button("PROTECT","#protect.xpm");
+
+    awm->at(protectx, second_liney+2);
+    awm->create_option_menu(AWAR_SECURITY_LEVEL);
+    awm->insert_option("0",0,0);
+    awm->insert_option("1",0,1);
+    awm->insert_option("2",0,2);
+    awm->insert_option("3",0,3);
+    awm->insert_option("4",0,4);
+    awm->insert_option("5",0,5);
+    awm->insert_default_option("6",0,6);
+    awm->update_option_menu();
+
+    int db_searchx = awm->get_at_xposition() - 7;
+
+    // draw ali/editor buttons AFTER protect menu (to get rid of it's label)
+    awm->at(db_alignx, second_liney);
+    
+    awm->at_set_to(AW_FALSE, AW_FALSE, ((2-is_genome_db)*EDIT_XSIZE), EDIT_YSIZE);
+    awm->callback((AW_CB)NT_start_editor_on_tree, (AW_CL)&(nt.tree->tree_root), 0);
+    awm->help_text("arb_edit4.hlp");
+    awm->create_button("EDIT_SEQUENCES", "#edit.xpm");
+
+    if (is_genome_db) {
+        awm->at_set_to(AW_FALSE, AW_FALSE, EDIT_XSIZE, EDIT_YSIZE);
+        awm->callback((AW_CB)AW_POPUP, (AW_CL)GEN_map_first, 0); // initial gene map
+        awm->help_text("gene_map.hlp");
+        awm->create_button("OPEN_GENE_MAP", "#gen_map.xpm");
+    }
+
+    int db_alignx2 = awm->get_at_xposition();
+    
+    awm->at(db_alignx, first_liney);
+    awm->at_set_to(AW_FALSE, AW_FALSE, db_alignx2-db_alignx-1, second_liney-first_liney+1);
     awm->callback(AW_POPUP,   (AW_CL)NT_open_select_alignment_window, 0 );
     awm->help_text("nt_align_select.hlp");
     awm->create_button("SELECT_AN_ALIGNMENT", AWAR_DEFAULT_ALIGNMENT);
 
-    int db_searchx;
-    awm->get_at_position(&db_searchx, &first_liney);
-    awm->callback(NT_popup_species_window, (AW_CL)awm, 0);
-//     awm->callback(AW_POPUP,   (AW_CL)create_speciesOrganismWindow/*ad_create_query_window*/, 0 );
-    awm->button_length(20);
-    awm->help_text("sp_search.hlp");
-    awm->create_button("SEARCH_SPECIES",  AWAR_SEARCH_BUTTON_TEXT);
-
-    int behind_buttonsx; // after first row of buttons
-    awm->get_at_position( &behind_buttonsx, &first_liney );
-
-    awm->at_newline();
-    int second_linex, second_liney;
-    awm->get_at_position(&second_linex, &second_liney);
-    second_liney += 5;
-    awm->at_y(second_liney);
-
-    // undo/redo:
-
-    awm->auto_space(-2,-2);
-
-    awm->button_length(9);
-    awm->at_x(db_pathx);
-    awm->callback(NT_undo_cb,(AW_CL)GB_UNDO_UNDO,(AW_CL)ntw);
-    awm->help_text("undo.hlp");
-    awm->create_button("UNDO", "#undo.bitmap",0);
-
-    awm->callback(NT_undo_cb,(AW_CL)GB_UNDO_REDO,(AW_CL)ntw);
-    awm->help_text("undo.hlp");
-    awm->create_button("REDO", "#redo.bitmap",0);
-
-    awm->at_newline();
-    int third_linex, third_liney;
-    awm->get_at_position(&third_linex, &third_liney);
-
-    // tree buttons:
-
-    awm->button_length(9);
-    awm->at(db_treex, second_liney);
-    awm->callback((AW_CB)NT_set_tree_style,(AW_CL)ntw,(AW_CL)AP_TREE_RADIAL);
-    awm->help_text("tr_type_radial.hlp");
-    awm->create_button("RADIAL_TREE_TYPE", "#radial.bitmap",0);
-
-    awm->callback((AW_CB)NT_set_tree_style,(AW_CL)ntw,(AW_CL)AP_TREE_NORMAL);
-    awm->help_text("tr_type_list.hlp");
-    awm->create_button("LIST_TREE_TYPE", "#list.bitmap",0);
-
-    awm->at(db_treex, third_liney);
-    awm->callback((AW_CB)NT_set_tree_style,(AW_CL)ntw,(AW_CL)AP_TREE_IRS);
-    awm->help_text("tr_type_irs.hlp");
-    awm->create_button("FOLDED_LIST_TREE_TYPE", "#list.bitmap",0);
-
-    awm->callback((AW_CB)NT_set_tree_style,(AW_CL)ntw,(AW_CL)AP_LIST_NDS);
-    awm->help_text("tr_type_nds.hlp");
-    awm->create_button("NO_TREE_TYPE", "#ndstree.bitmap",0);
-
-    awm->at_newline();
+    // ------------------------------
+    //      create mode-help line
+    // ------------------------------
+    
+    awm->at(leftx, third_liney);
     awm->button_length(AWAR_FOOTER_MAX_LEN);
     awm->create_button(0,AWAR_FOOTER);
+
     awm->at_newline();
-    int last_linex, last_liney;
-    awm->get_at_position( &last_linex,&last_liney );
+    int bottomy = awm->get_at_yposition();
 
-    // edit, gene-map, jump & web buttons (+help buttons):
+    // --------------------------------------
+    //      Info / marked / Search / Jump
+    // --------------------------------------
 
-    awm->button_length(9);
-    awm->at(db_alignx, second_liney);
-    awm->callback((AW_CB)NT_start_editor_on_tree, (AW_CL)&(nt.tree->tree_root), 0);
-    awm->help_text("arb_edit4.hlp");
-    awm->create_button("EDIT_SEQUENCES", "#edit.bitmap",0);
-
-    if (is_genome_db) {
-        awm->button_length(4);
-        awm->callback((AW_CB)AW_POPUP, (AW_CL)GEN_map_first, 0/*, (AW_CL)ntw*/); // initial gene map
-        awm->help_text("gene_map.hlp");
-        awm->create_button("OPEN_GENE_MAP", "#gen_map.bitmap",0);
-    }
-
-    awm->button_length(0);
-    awm->at(db_searchx, second_liney);
-
-    awm->callback(AW_POPUP,   (AW_CL)ad_create_query_window, 0 );
-    awm->create_button("SEARCH", "#search.bitmap",0);
+    awm->button_length(7);
+    
+    awm->at(db_searchx, first_liney);
+    awm->callback(AW_POPUP, (AW_CL)ad_create_query_window, 0 );
+    awm->help_text("sp_search.hlp");
+    awm->create_button("SEARCH",  "Search");
 
+    awm->at(db_searchx, second_uppery);
     awm->callback((AW_CB)NT_jump_cb,(AW_CL)ntw,1);
     awm->help_text("tr_jump.hlp");
-    awm->create_button("JUMP", "#pjump.bitmap",0);
+    awm->create_button("JUMP", "Jump");
+    
+    int db_infox = awm->get_at_xposition();
+    
+    awm->at(db_infox, first_liney);
+    awm->button_length(13);
+    awm->callback(NT_popup_species_window, (AW_CL)awm, 0);
+    awm->help_text("sp_search.hlp");
+    awm->create_button("INFO",  AWAR_INFO_BUTTON_TEXT);
 
-    awm->callback((AW_CB1)awt_openDefaultURL_on_species,(AW_CL)gb_main);
-    awm->help_text("www.hlp");
-    awm->create_button("WWW", "#www.bitmap",0);
-
-    // count marked species
-
-    awm->button_length(20);
-    awm->at(db_searchx, third_liney);
-    awm->callback(nt_count_marked);
+    awm->at(db_infox, second_uppery);
+    awm->button_length(13);
     awm->help_text("marked_species.hlp");
+    awm->callback(NT_configuration_admin, (AW_CL)&(nt.tree->tree_root), 0);
     awm->create_button(0, AWAR_MARKED_SPECIES_COUNTER);
     {
         GBDATA *gb_species_data = GB_search(gb_main,"species_data",GB_CREATE_CONTAINER);
@@ -1692,41 +1910,9 @@ AW_window * create_nt_main_window(AW_root *awr, AW_CL clone){
         nt_count_marked(awm);
     }
 
-    // help:
-
-    awm->at(behind_buttonsx, first_liney);
-    awm->callback(AW_POPUP_HELP,(AW_CL)"arb_ntree.hlp");
-    awm->button_length(0);
-    awm->help_text("help.hlp");
-    awm->create_button("HELP","HELP","H");
-
-    awm->callback( AW_help_entry_pressed );
-    awm->create_button("?","?");
-
-    // expand and collapse display icons
-
-    awr->awar_int(AWAR_NTREE_TITLE_MODE)->add_callback((AW_RCB1)title_mode_changed, (AW_CL)awm);
-    awm->create_toggle(AWAR_NTREE_TITLE_MODE, "#more.bitmap", "#less.bitmap");
-
-    // protect:
-
-    awm->at(behind_buttonsx, second_liney+3);
-    if (!GB_NOVICE){
-        awm->label("Protect");
-        awm->create_option_menu(AWAR_SECURITY_LEVEL);
-        awm->insert_option("0",0,0);
-        awm->insert_option("1",0,1);
-        awm->insert_option("2",0,2);
-        awm->insert_option("3",0,3);
-        awm->insert_option("4",0,4);
-        awm->insert_option("5",0,5);
-        awm->insert_default_option("6",0,6);
-        awm->update_option_menu();
-    }else{
-        awr->awar(AWAR_SECURITY_LEVEL)->write_int(6);
-    }
-
-    awm->set_info_area_height( last_liney+4 ); windowHeight = last_liney+4;
+    
+    // set height of top area:
+    awm->set_info_area_height(bottomy+2); windowHeight = bottomy+2;
     awm->set_bottom_area_height( 0 );
     awr->set_focus_callback((AW_RCB)NT_focus_cb,(AW_CL)gb_main,0);
 
