@@ -26,7 +26,7 @@
 #include "nt_concatenate.hxx"
 #include "ntree.hxx"
 #include "nt_cb.hxx"
-#include "nt_date.h"
+#include <arb_version.h>
 
 #include <set>
 #include <string>
@@ -76,22 +76,25 @@ GB_ERROR NT_format_all_alignments(GBDATA *gb_main) {
 
             GBDATA     *gb_ali_name  = GB_find(gb_ali,"alignment_name",0,down_level);
             const char *ali_name     = GB_read_char_pntr(gb_ali_name);
-            bool        is_ali_genom = strcmp(ali_name, GENOM_ALIGNMENT) == 0;
 
-            if (GBDATA *gb_auto_format = GB_find(gb_ali, "auto_format", 0, down_level)) {
-                format_action = GB_read_int(gb_auto_format);
+            {
+                bool    is_ali_genom   = strcmp(ali_name, GENOM_ALIGNMENT) == 0;
+                GBDATA *gb_auto_format = GB_find(gb_ali, "auto_format", 0, down_level);
+                if (gb_auto_format) {
+                    format_action = GB_read_int(gb_auto_format);
 
-                if (is_ali_genom) {
-                    if (format_action != 2) {
-                        format_action = 2; // always skip ali_genom
-                        GB_write_int(gb_auto_format, 2);
+                    if (is_ali_genom) {
+                        if (format_action != 2) {
+                            format_action = 2; // always skip ali_genom
+                            GB_write_int(gb_auto_format, 2);
+                        }
                     }
                 }
-            }
-            else if (is_ali_genom) {
-                GBDATA *gb_auto_format = GB_search(gb_ali, "auto_format", GB_INT);
-                GB_write_int(gb_auto_format, 2); // always skip
-                format_action          = 2;
+                else if (is_ali_genom) {
+                    gb_auto_format = GB_search(gb_ali, "auto_format", GB_INT);
+                    GB_write_int(gb_auto_format, 2); // always skip
+                    format_action          = 2;
+                }
             }
 
             bool perform_format = false;
@@ -141,17 +144,18 @@ GB_ERROR NT_format_all_alignments(GBDATA *gb_main) {
     return err;
 }
 
-static GB_ERROR NT_fix_gene_data(GBDATA *gb_main) {
+static GB_ERROR NT_fix_gene_data(GBDATA *gb_main, size_t species_count, size_t /*sai_count*/) {
     GB_transaction ta(gb_main);
-    int            deleted_gene_datas   = 0;
-    int            generated_gene_datas = 0;
+    size_t         deleted_gene_datas   = 0;
+    size_t         generated_gene_datas = 0;
     GB_ERROR       error                = 0;
+    size_t         count                = 0;
 
     for (GBDATA *gb_species = GBT_first_species(gb_main);
          gb_species && !error;
          gb_species = GBT_next_species(gb_species))
     {
-        bool    is_organism  = GEN_is_organism(gb_species);
+        bool    is_organism  = (GB_find(gb_species, GENOM_ALIGNMENT, 0, down_level) != 0); // same test as GEN_is_organism, but w/o genome-db-assertion
         GBDATA *gb_gene_data = GEN_find_gene_data(gb_species);
 
         if (is_organism && !gb_gene_data) {
@@ -175,28 +179,116 @@ static GB_ERROR NT_fix_gene_data(GBDATA *gb_main) {
                                           "This causes problems - please fix!", name);
             }
         }
+        
+        aw_status(double(++count)/species_count);
     }
 
     if (error) {
         ta.abort();
     }
     else {
-        if (deleted_gene_datas>0) {
-            aw_message(GBS_global_string("Deleted %i useless empty 'gene_data' entries.", deleted_gene_datas));
+        if (deleted_gene_datas) {
+            aw_message(GBS_global_string("Deleted %u useless empty 'gene_data' entries.", deleted_gene_datas));
         }
-        if (generated_gene_datas>0) {
-            aw_message(GBS_global_string("Re-created %i missing 'gene_data' entries.\nThese organisms have no genes!", generated_gene_datas));
+        if (generated_gene_datas) {
+            aw_message(GBS_global_string("Re-created %u missing 'gene_data' entries.\nThese organisms have no genes yet!", generated_gene_datas));
         }
     }
     return error;
 }
 
+static GB_ERROR NT_del_mark_move_REF(GBDATA *gb_main, size_t species_count, size_t sai_count) {
+    GB_transaction ta(gb_main);
+    GB_ERROR       error   = 0;
+    size_t         count   = 0;
+    size_t         all     = species_count+sai_count;
+    size_t         removed = 0;
+
+    // delete 'mark' entries from all alignments of species/SAIs
+
+    char **ali_names = GBT_get_alignment_names(gb_main);
+
+    for (int pass = 0; pass < 2 && !error; ++pass) {
+        for (GBDATA *gb_item = (pass == 0) ? GBT_first_species(gb_main) : GBT_first_SAI(gb_main);
+             gb_item && !error;
+             gb_item = (pass == 0) ? GBT_next_species(gb_item) : GBT_next_SAI(gb_item))
+        {
+            for (int ali = 0; ali_names[ali] && !error; ++ali) {
+                GBDATA *gb_ali = GB_find(gb_item, ali_names[ali], 0, down_level);
+                if (gb_ali) {
+                    GBDATA *gb_mark = GB_find(gb_ali, "mark", 0, down_level);
+                    if (gb_mark) {
+                        error = GB_delete(gb_mark);
+                        removed++;
+                    }
+                }
+            }
+
+            aw_status(double(++count)/all);
+        }
+    }
+
+    {
+        char   *helix_name = GBT_get_default_helix(gb_main);
+        GBDATA *gb_helix   = GBT_find_SAI(gb_main, helix_name);
+
+        if (gb_helix) {
+            for (int ali = 0; ali_names[ali] && !error; ++ali) {
+                GBDATA *gb_ali     = GB_find(gb_helix, ali_names[ali], 0, down_level);
+                GBDATA *gb_old_ref = GB_find(gb_ali, "REF", 0, down_level);
+                GBDATA *gb_new_ref = GB_find(gb_ali, "_REF", 0, down_level);
+
+                if (gb_old_ref) {
+                    if (gb_new_ref) {
+                        error = GBS_global_string("SAI:%s has 'REF' and '_REF' in '%s' (data corrupt?!)",
+                                                  helix_name, ali_names[ali]);
+                    }
+                    else { // move info from REF -> _REF
+                        char *content = GB_read_string(gb_old_ref);
+                        if (content) {
+                            gb_new_ref = GB_create(gb_ali, "_REF", GB_STRING);
+                            if (!gb_new_ref) {
+                                error = GB_get_error();
+                            }
+                            else {
+                                error = GB_write_string(gb_new_ref, content);
+                                if (!error) error = GB_delete(gb_old_ref);
+                            }
+                            free(content);
+                        }
+                        else {
+                            error = GB_get_error();
+                        }
+                    }
+                }
+            }
+        }
+
+        free(helix_name);
+    }
+
+    GBT_free_names(ali_names);
+
+    if (error) {
+        ta.abort();
+    }
+    else {
+        if (removed) {
+            aw_message(GBS_global_string("Deleted %u useless 'mark' entries.", removed));
+        }
+    }
+
+    return error;
+}
+
 // CheckedConsistencies provides an easy way to automatically correct flues in the database
 // by calling a check routine exactly one.
 // For an example see nt_check_database_consistency()
 
 class CheckedConsistencies {
     GBDATA      *gb_main;
+    size_t       species_count;
+    size_t       sai_count;
     set<string>  consistencies;
 
 public:
@@ -210,6 +302,9 @@ public:
         {
             consistencies.insert(GB_read_char_pntr(gb_check));
         }
+
+        species_count = GBT_get_species_count(gb_main);
+        sai_count = GBT_get_SAI_count(gb_main);
     }
 
     bool was_performed(const string& check_name) const {
@@ -237,6 +332,19 @@ public:
         }
         return error;
     }
+
+    void perform_check(const string& check_name,
+                       GB_ERROR (*do_check)(GBDATA *gb_main, size_t species, size_t sais),
+                       GB_ERROR& error)
+    {
+        if (!error && !was_performed(check_name)) {
+            aw_status(check_name.c_str());
+            aw_status(0.0);
+            error = do_check(gb_main, species_count, sai_count);
+            aw_status(1.0);
+            if (!error) register_as_performed(check_name);
+        }
+    }
 };
 
 
@@ -248,10 +356,8 @@ static GB_ERROR nt_check_database_consistency() {
     CheckedConsistencies check(gb_main);
     GB_ERROR             err = NT_format_all_alignments(gb_main);
 
-    if (!err && !check.was_performed("fix gene_data")) {
-        err = NT_fix_gene_data(gb_main);
-        if (!err) check.register_as_performed("fix gene_data");
-    }
+    check.perform_check("fix gene_data",     NT_fix_gene_data,     err);
+    check.perform_check("del_mark_move_REF", NT_del_mark_move_REF, err);
 
     aw_closestatus();
     return err;
@@ -388,7 +494,7 @@ void nt_intro_start_import(AW_window *aws)
     aws->hide();
     aws->get_root()->awar_string( AWAR_DB_PATH )->write_string( "noname.arb");
     aws->get_root()->awar_int(AWAR_READ_GENOM_DB, IMP_PLAIN_SEQUENCE); // Default toggle  in window  "Create&import" is Non-Genom
-    gb_main = open_AWTC_import_window(aws->get_root(),"",1,(AW_RCB)main3,0,0);
+    gb_main = open_AWTC_import_window(aws->get_root(), "", true, 0, (AW_RCB)main3, 0, 0);
 }
 
 AW_window *nt_create_intro_window(AW_root *awr)
@@ -413,7 +519,7 @@ AW_window *nt_create_intro_window(AW_root *awr)
     aws->create_button(0,"#logo.bitmap");
 
     aws->at("version");
-    aws->create_button(0, GBS_global_string("Version " DATE), 0); // version
+    aws->create_button(0, GBS_global_string("Version " ARB_VERSION), 0); // version
 
     aws->at("copyright");
     aws->create_button(0, GBS_global_string("(C) 1993-" DATE_YEAR), 0);
@@ -487,7 +593,7 @@ int main(int argc, char **argv)
     unsigned long mtime = GB_time_of_file("$(ARBHOME)/lib/message");
     unsigned long rtime = GB_time_of_file("$(HOME)/.arb_prop/msgtime");
     if (mtime > rtime){
-        GB_edit("${ARBHOME}/lib/message");
+        AWT_edit("${ARBHOME}/lib/message");
         system("touch ${HOME}/.arb_prop/msgtime");
     }
     aw_initstatus();
@@ -511,10 +617,12 @@ int main(int argc, char **argv)
 
     NT_createConcatenationAwars(aw_root, aw_default); // creating AWARS for concatenation and merge simlar species function
 
-    if (argc==3) {      // looks like merge
-        MG_create_all_awars(aw_root,aw_default,argv[1],argv[2]);
-        nt_intro_start_merge(0,aw_root);
-        aw_root->main_loop();
+    if (argc==3) {              // looks like merge
+        if (argv[1][0] != '-') { // not if first argument is a switch 
+            MG_create_all_awars(aw_root,aw_default,argv[1],argv[2]);
+            nt_intro_start_merge(0,aw_root);
+            aw_root->main_loop();
+        }
     }
 
     bool  abort            = false;
@@ -530,9 +638,12 @@ int main(int argc, char **argv)
         {
             fprintf(stderr,
                     "\n"
-                    "arb_ntree version " DATE "\n"
+                    "arb_ntree version " ARB_VERSION "\n"
                     "(C) 1993-" DATE_YEAR " Lehrstuhl fuer Mikrobiologie - TU Muenchen\n"
                     "http://www.arb-home.de/\n"
+#if defined(SHOW_WHERE_BUILD)
+                    "(version build by: " BUILD_USER "@" BUILD_HOST ")\n"
+#endif // SHOW_WHERE_BUILD
                     "\n"
                     "Known command line arguments:\n"
                     "\n"
@@ -540,6 +651,7 @@ int main(int argc, char **argv)
                     ":                  => start ARB_NTREE and connect to existing db_server\n"
                     "db1.arb db2.arb    => merge databases db1.arb and db2.arb\n"
                     "-export            => connect to existing ARB server and export database to noname.arb\n"
+                    "-import file       => import 'file' into new database\n"
                     "w/o arguments      => start database browser\n"
                     "\n"
                     );
@@ -563,8 +675,17 @@ int main(int argc, char **argv)
             aw_root->main_loop();
         }
 
-        db_server = argv[1];
-        if (GB_ERROR load_file_err = GBT_check_arb_file(db_server)) {
+        bool run_importer = false;
+        if (strcmp(argv[1], "-import") == 0) {
+            argv++;
+            run_importer = true;
+        }
+
+        db_server              = argv[1];
+        GB_ERROR load_file_err = 0;
+        if (!run_importer) load_file_err = GBT_check_arb_file(db_server);
+
+        if (load_file_err) {
             int   answer    = -1;
             char *full_path = AWT_unfold_path(db_server);
 
@@ -595,9 +716,7 @@ int main(int argc, char **argv)
                     break;
                 }
                 case 1: {        // Start converter
-                    aw_root->awar_int(AWAR_READ_GENOM_DB, IMP_PLAIN_SEQUENCE);
-                    gb_main = open_AWTC_import_window(aw_root,db_server, 1,(AW_RCB)main3,0,0);
-                    aw_root->main_loop();
+                    run_importer =  true;
                     break;
                 }
                 case 2: {        // Browse
@@ -629,6 +748,12 @@ int main(int argc, char **argv)
             }
             free(full_path);
         }
+
+        if (run_importer) {
+            aw_root->awar_int(AWAR_READ_GENOM_DB, IMP_PLAIN_SEQUENCE);
+            gb_main = open_AWTC_import_window(aw_root, db_server, true, 0, (AW_RCB)main3, 0, 0);
+            aw_root->main_loop();
+        }
     }
 
 
