@@ -139,7 +139,8 @@ static void ad_species_copy_cb(AW_window *aww, AW_CL, AW_CL) {
     if (gb_species) {
         GB_transaction  ta(gb_main);
         GBDATA         *gb_species_data = GB_get_father(gb_species);
-        char           *copy_name       = AWTC_makeUniqueShortName(GBS_global_string("c_%s", name), gb_species_data);
+        UniqueNameDetector und(gb_species_data);
+        char           *copy_name       = AWTC_makeUniqueShortName(GBS_global_string("c%s", name), und);
         GBDATA         *gb_new_species  = GB_create_container(gb_species_data, "species");
         GB_ERROR        error           = 0;
 
@@ -797,7 +798,7 @@ static AW_window *ad_spec_next_neighbours_listed_create(AW_root *aw_root,AW_CL c
     aws->create_button("HELP","HELP","H");
 
     aws->at("pt_server");
-    awt_create_selection_list_on_pt_servers(aws, AWAR_PROBE_ADMIN_PT_SERVER, true);
+    awt_create_selection_list_on_pt_servers(aws, AWAR_PROBE_ADMIN_PT_SERVER, AW_TRUE);
 
     aws->at("field");
     awt_create_selection_list_on_scandb(gb_main,aws,"next_neighbours/dest_field",
