@@ -121,8 +121,8 @@ void AP_display::initialize (display_type dpyt)
 void AP_display::resized(void)
 {
     AW_rectangle squ;
-    AW_rectangle rect;
-    long horiz_paint_size,vert_paint_size;
+    AW_rectangle rect =  { 0, 0, 0, 0 };
+    long         horiz_paint_size,vert_paint_size;
 
     PH_used_windows::windowList->phylo_main_window->get_device(AW_MIDDLE_AREA)-> get_area_size(&squ);
     screen_width  = squ.r-squ.l;
@@ -209,8 +209,7 @@ void AP_display::display(void)   // draw area
     {
         case NONE: return;
         case species_dpy:
-            device->shift_dy(off_dy);
-            device->shift_dx(off_dx);
+            device->shift(AW::Vector(off_dx, off_dy));
             ypos=0;
             for(y=vert_page_start;y<(vert_page_start+vert_page_size) &&
                     (y<total_cells_vert);y++)
@@ -229,12 +228,11 @@ void AP_display::display(void)   // draw area
                              0.0,-1,0,0);                          // alignment
                 ypos++;
             }
-            device->shift_dy(-off_dy);
-            device->shift_dx(-off_dx);
+            device->shift(-AW::Vector(off_dx, off_dy));
             break;
 
-        case matrix_dpy: device->shift_dx(off_dx);
-            device->shift_dy(off_dy);
+        case matrix_dpy:
+            device->shift(AW::Vector(off_dx, off_dy));
             xpos=0;
             for(x=horiz_page_start;x<(horiz_page_start+horiz_page_size) &&
                     (x<total_cells_horiz);x++)
@@ -253,7 +251,7 @@ void AP_display::display(void)   // draw area
                              xpos*cell_width,cell_height-off_dy-cell_offset,0.0,-1,0,0);
                 xpos++;
             }
-            device->shift_dx(-off_dx);
+            device->shift(AW::Vector(-off_dx, 0));
             // display vertical speciesnames
             ypos=0;
             for(y=vert_page_start;y<vert_page_start+vert_page_size;y++)
@@ -262,15 +260,14 @@ void AP_display::display(void)   // draw area
                              0,ypos*cell_height-cell_offset,0.0,-1,0,0);
                 ypos++;
             }
-            device->shift_dy(-off_dy);
+            device->shift(AW::Vector(0, -off_dy));
             break;
 
 
 
 
         case filter_dpy: {
-            device->shift_dy(off_dy);
-            device->shift_dx(off_dx);
+            device->shift(AW::Vector(off_dx, off_dy));
             ypos=0;
             for(y=vert_page_start;y<(vert_page_start+vert_page_size) &&
                     (y<total_cells_vert);y++)
@@ -323,15 +320,13 @@ void AP_display::display(void)   // draw area
                 }
                 xpos++;
             }
-            device->shift_dy(-off_dy);
-            device->shift_dx(-off_dx);
+            device->shift(-AW::Vector(off_dx, off_dy));
             break;
         }
 
         default:
             printf("\ndisplay: unknown display type (maybe not implemented yet)\n");
-    }  // switch
-    //  display_status(0,(AW_CL) main_win->get_root(),0);
+    }
 }
 
 
@@ -481,6 +476,12 @@ void AP_display_status::write(const char *text)
     x_pos+=strlen(text);
 }
 
+void AP_display_status::writePadded(const char *text, size_t len)
+{
+    device->text(0,text,x_pos*font_width,y_pos*font_height,0.0,-1,0,0);
+    x_pos += len;
+}
+
 void AP_display_status::write(long numl)
 {
     char buf[20];
@@ -512,6 +513,9 @@ void display_status(AW_window *dummy,AW_CL cl_awroot,AW_CL cd2)    // bottom are
     {
         static AP_display_status apds(PH_used_windows::windowList->phylo_main_window->get_device (AW_BOTTOM_AREA));
         apds.clear();
+        
+        const int LABEL_LEN = 21;
+
         switch(AP_display::apdisplay->displayed())
         {
             case NONE: return;
@@ -520,36 +524,44 @@ void display_status(AW_window *dummy,AW_CL cl_awroot,AW_CL cd2)    // bottom are
                 apds.set_cursor((apds.get_size('x')/2)-10,0);
                 apds.write("STATUS REPORT FILTER");
                 apds.newline();
-                apds.newline();
-                apds.write("start at column:     ");
+                
+                apds.writePadded("Start at column:", LABEL_LEN);
                 apds.write((long)aw_root->awar("phyl/filter/startcol")->read_int());
-                apds.move_x(40);
+                apds.move_x(15);
                 apds.set_tab();
-                apds.write("stop at column:      ");
+                apds.writePadded("Stop at column:", LABEL_LEN);
                 apds.write((long)aw_root->awar("phyl/filter/stopcol")->read_int());
                 apds.newline();
-                apds.write("minimal similarity:  ");
+
+                apds.writePadded("Minimal similarity:", LABEL_LEN);
                 apds.write((long)aw_root->awar("phyl/filter/minhom")->read_int());
                 apds.set_cursor_x(apds.get_tab());
-                apds.write("maximal similarity:  ");
+                apds.writePadded("Maximal similarity:", LABEL_LEN);
                 apds.write((long)aw_root->awar("phyl/filter/maxhom")->read_int());
                 apds.newline();
-                apds.write("'.' in column:     ");
+                apds.newline();
+
+                apds.writePadded("'.':", LABEL_LEN);
                 apds.write(filter_text[aw_root->awar("phyl/filter/point")->read_int()]);
                 apds.newline();
-                apds.write("'-' in column:     ");
+
+                apds.writePadded("'-':", LABEL_LEN);
                 apds.write(filter_text[aw_root->awar("phyl/filter/minus")->read_int()]);
                 apds.newline();
-                apds.write("'rest' in column:  ");
+
+                apds.writePadded("ambiguity codes:", LABEL_LEN);
                 apds.write(filter_text[aw_root->awar("phyl/filter/rest")->read_int()]);
                 apds.newline();
-                apds.write("'acgtu' in column: ");
+
+                apds.writePadded("lowercase chars:", LABEL_LEN);
                 apds.write(filter_text[aw_root->awar("phyl/filter/lower")->read_int()]);
                 break;
+
             case matrix_dpy: apds.set_origin();
                 apds.set_cursor((apds.get_size('x')/2)-10,0);
                 apds.write("STATUS REPORT MATRIX");
                 break;
+                
             default: printf("\nstatus: unknown display type (maybe not implemented yet)\n");
         }
     }
