@@ -216,7 +216,7 @@ int init_local_com_struct()
     return 0;
 }
 
-char *probe_pt_look_for_server(AW_root *root)
+static const char *PD_probe_pt_look_for_server(AW_root *root)
 {
     char choice[256];
     sprintf(choice, "ARB_PT_SERVER%li", root->awar(AWAR_PT_SERVER)->read_int());
@@ -319,7 +319,7 @@ GB_ERROR pd_get_the_gene_names(bytestring &bs, bytestring &checksum){
                 gene_name = GB_read_char_pntr(gb_gene_name);
             }
 
-            long checksum;
+            long CheckSum;
             {
                 GBDATA *gb_gene_pos_begin = GB_find(gb_gene, "pos_begin", 0, down_level);
                 if (!gb_gene_pos_begin) { error = gene_requires(gb_gene, "pos_begin"); break; }
@@ -335,14 +335,14 @@ GB_ERROR pd_get_the_gene_names(bytestring &bs, bytestring &checksum){
                 strncpy(gene_seq, sequence+pos_begin, len);
                 gene_seq[len]  = 0;
 
-                checksum = GBS_checksum(gene_seq, 1, ".-");
+                CheckSum = GBS_checksum(gene_seq, 1, ".-");
 
                 // @@@ FIXME: what to do with splitted genes ?
             }
 
             const char *id = GBS_global_string("%s/%s", species_name, gene_name);
 
-            GBS_intcat(checksums, checksum);
+            GBS_intcat(checksums, CheckSum);
             GBS_strcat(names, id);
             GBS_chrcat(checksums, '#');
             GBS_chrcat(names, '#');
@@ -408,7 +408,6 @@ int probe_design_send_data(AW_root *root, T_PT_PDC  pdc)
 void probe_design_event(AW_window *aww)
 {
     AW_root     *root  = aww->get_root();
-    char        *servername;
     T_PT_PDC     pdc;
     T_PT_TPROBE  tprobe;
     bytestring   bs;
@@ -419,13 +418,16 @@ void probe_design_event(AW_window *aww)
     aw_openstatus("Probe Design");
     aw_status("Search a free running server");
 
-    if( !(servername=(char *)probe_pt_look_for_server(root)) ){
-        aw_closestatus();
-        return;
-    }
+    {
+        const char *servername = PD_probe_pt_look_for_server(root);
+        if (!servername) {
+            aw_closestatus();
+            return;
+        }
 
-    pd_gl.link = (aisc_com *)aisc_open(servername, &pd_gl.com,AISC_MAGIC_NUMBER);
-    free (servername); servername = 0;
+        pd_gl.link = (aisc_com *)aisc_open(servername, &pd_gl.com,AISC_MAGIC_NUMBER);
+        servername = 0;
+    }
 
     if (!pd_gl.link) {
         aw_message ("Cannot contact Probe bank server ");
@@ -504,7 +506,7 @@ void probe_design_event(AW_window *aww)
         if (design_gene_probes) { // updating sequences of missing genes is not possible with gene PT server
             aw_message(GBS_global_string("Your PT server is not up to date or wrongly chosen.\n"
                                          "The following genes are new to it: %s\n"
-                                         "You'll have to update the PT server.", unames));
+                                         "You'll have to re-build the PT server.", unames));
             abort = true;
         }
         else if (aw_message(GBS_global_string(
@@ -689,7 +691,7 @@ void probe_match_event(AW_window *aww, AW_CL cl_selection_id, AW_CL cl_count_ptr
         }
 
         if (!error) {
-            char *servername = probe_pt_look_for_server(root);
+            const char *servername = PD_probe_pt_look_for_server(root);
             if (!servername) error = GB_get_error();
 
             if (!error) {
@@ -708,7 +710,6 @@ void probe_match_event(AW_window *aww, AW_CL cl_selection_id, AW_CL cl_count_ptr
                 }
 
                 pd_gl.link = (aisc_com *)aisc_open(servername, &pd_gl.com,AISC_MAGIC_NUMBER);
-                free(servername);
             }
         }
 
@@ -774,9 +775,15 @@ void probe_match_event(AW_window *aww, AW_CL cl_selection_id, AW_CL cl_count_ptr
             {
                 error = "Connection to PT_SERVER lost (3)";
             }
-
-            if (*locs_error) error = GBS_global_string("%s", locs_error);
-            free(locs_error);
+            else {
+                if (locs_error) {
+                    if (*locs_error) error = GBS_global_string("%s", locs_error);
+                    free(locs_error);
+                }
+                else {
+                    error = "Missing status from server (connection aborted?)";
+                }
+            }
 
             root->awar(AWAR_PD_MATCH_NHITS)->write_string(GBS_global_string(matches_truncated ? "> %li" : "%li", match_list_cnt));
             if (matches_truncated) {
@@ -840,11 +847,14 @@ void probe_match_event(AW_window *aww, AW_CL cl_selection_id, AW_CL cl_count_ptr
                          gb_species;
                          gb_species = GBT_next_species(gb_species))
                     {
-                        for (GBDATA *gb_gene = GEN_first_marked_gene(gb_species);
-                             gb_gene;
-                             gb_gene = GEN_next_marked_gene(gb_gene))
-                        {
-                            GB_write_flag(gb_gene,0);
+                        GBDATA *genData = GEN_find_gene_data(gb_species);
+                        if (genData) {
+                            for (GBDATA *gb_gene = GEN_first_marked_gene(gb_species);
+                                 gb_gene;
+                                 gb_gene = GEN_next_marked_gene(gb_gene))
+                            {
+                                GB_write_flag(gb_gene,0);
+                            }
                         }
                     }
                 }
@@ -946,7 +956,7 @@ void probe_match_event(AW_window *aww, AW_CL cl_selection_id, AW_CL cl_count_ptr
 
                         if (write_2_tmp) {
                             GBDATA   *gb_tmp = 0;
-                            GB_ERROR  error  = 0;
+                            GB_ERROR  error2  = 0;
                             {
                                 GBDATA *gb_parent = gene_flag ? gb_gene : gb_species;
                                 gb_tmp            = GB_search(gb_parent, "tmp", GB_FIND);
@@ -954,7 +964,7 @@ void probe_match_event(AW_window *aww, AW_CL cl_selection_id, AW_CL cl_count_ptr
                                     const char *name    = "<unknown>";
                                     GBDATA     *gb_name = GB_search(gb_parent, "name", GB_FIND);
                                     if (gb_name) name   = GB_read_char_pntr(gb_name);
-                                    error               = GBS_global_string("field 'tmp' already exists for %s '%s'", gene_flag ? "gene" : "species", name);
+                                    error2               = GBS_global_string("field 'tmp' already exists for %s '%s'", gene_flag ? "gene" : "species", name);
                                     gb_tmp              = 0; // don't overwrite
                                 }
                                 else {
@@ -962,12 +972,12 @@ void probe_match_event(AW_window *aww, AW_CL cl_selection_id, AW_CL cl_count_ptr
                                 }
                             }
 
-                            if (!error) {
+                            if (!error2) {
                                 pd_assert(gb_tmp);
-                                error = GB_write_string(gb_tmp, match_info);
+                                error2 = GB_write_string(gb_tmp, match_info);
                             }
 
-                            if (error) aw_message(error);
+                            if (error2) aw_message(error2);
                         }
                     }
                     else {
@@ -1737,29 +1747,43 @@ void pd_start_pt_server(AW_window *aww)
 
 void pd_kill_pt_server(AW_window *aww, AW_CL kill_all)
 {
-    AW_root *awr = aww->get_root();
-    char pt_server[256];
-    long min = 0;
-    long max = 1000;
-    long i;
-    if (!kill_all) min = max = awr->awar(AWAR_PROBE_ADMIN_PT_SERVER)->read_int();
-    if (!aw_message("Are you sure to kill a server","YES,CANCEL")){
-        aw_openstatus("Kill a server");
-        GB_ERROR error;
-        for (i= min ; i <=max ; i++) {
-            char *choice = GBS_ptserver_id_to_choice((int)i);
-            if (!choice) break;
-            sprintf(AW_ERROR_BUFFER,"Try to call '%s'",choice);
-            aw_status(AW_ERROR_BUFFER);
-            sprintf(pt_server,"ARB_PT_SERVER%li",i);
-            error = arb_look_and_kill_server(AISC_MAGIC_NUMBER,pt_server);
-            if (error) {
-                sprintf(AW_ERROR_BUFFER,"NOP    '%s' -- %s",choice,error);
-                aw_message();
-            }else{
-                sprintf(AW_ERROR_BUFFER,"Killed '%s' killed",choice);
-                aw_message();
+    if (!aw_message(GBS_global_string("Are you sure to stop %s",
+                                      kill_all ? "all servers" : "that server"),
+                    "YES,CANCEL"))
+    {
+        long min = 0;
+        long max = 0;
+
+        if (kill_all) {
+            const char * const *pt_servers = GBS_get_arb_tcp_entries("ARB_PT_SERVER*");
+            while (pt_servers[max]) max++;
+        }
+        else {
+            min = max = aww->get_root()->awar(AWAR_PROBE_ADMIN_PT_SERVER)->read_int(); // selected server
+        }
+
+        aw_openstatus("Stopping PT-servers..");
+
+        for (int i= min ; i <=max ; i++) {
+            char *choice = GBS_ptserver_id_to_choice(i, 0);
+            if (!choice) {
+                GB_ERROR error = GB_get_error();
+                if (!error) {
+                    error = GBS_global_string("Failed to get pt-server id #%i (please report)", i);
+                    gb_assert(0); // should not occur
+                }
+                aw_message(error);
+                break;
             }
+
+            aw_status(GBS_global_string("Trying to stop '%s'",choice));
+
+            const char *pt_server = GBS_global_string("ARB_PT_SERVER%i",i);
+            GB_ERROR    error     = arb_look_and_kill_server(AISC_MAGIC_NUMBER, pt_server);
+
+            if (error) aw_message(GBS_global_string("Could not stop '%s' (Reason: %s)", choice, error));
+            else aw_message(GBS_global_string("Stopped '%s'", choice));
+            
             free(choice);
         }
         aw_closestatus();
@@ -1769,39 +1793,33 @@ void pd_kill_pt_server(AW_window *aww, AW_CL kill_all)
 void pd_query_pt_server(AW_window *aww)
 {
     AW_root *awr = aww->get_root();
-    char *server;
-    char pt_server[256];
+    char     pt_server[256];
     sprintf(pt_server,"ARB_PT_SERVER%li",awr->awar(AWAR_PROBE_ADMIN_PT_SERVER)->read_int());
-    server = GBS_read_arb_tcp(pt_server);
-    char *ssh = 0;
-    if ( server[0] && strchr(server+1,':') ) {
-        strchr(server+1,':') [0] = 0;
-        ssh = (char *)calloc(1, 4+10+strlen(server) );
-        sprintf(ssh,"ssh %s ",server);
-    }else{
-        ssh = strdup("");
-    }
-    void *strstruct = GBS_stropen(1024);
+
+    void       *strstruct = GBS_stropen(1024);
     GBS_strcat(strstruct,   "echo Contents of directory ARBHOME/lib/pts:;echo;"
                "(cd $ARBHOME/lib/pts; ls -l);"
                "echo; echo Disk Space for PT_server files:; echo;"
                "df $ARBHOME/lib/pts;");
     GBS_strcat(strstruct,"echo;echo Running ARB Programms:;");
-    GBS_strcat(strstruct,ssh);
-    GBS_strcat(strstruct,"$ARBHOME/bin/arb_who");
+
+    {
+        const char *server  = GBS_read_arb_tcp(pt_server);
+        char       *arb_who = prefixSSH(server, "\\$ARBHOME/bin/arb_who", 0);
+        GBS_strcat(strstruct, arb_who);
+        free(arb_who);
+    }
+
     char *sys = GBS_strclose(strstruct);
     GB_xcmd(sys,GB_TRUE, GB_FALSE);
     free(sys);
-    free(ssh);
 }
 
-void pd_export_pt_server(AW_window *aww)
+static void pd_export_pt_server(AW_window *aww)
 {
-    AW_root  *awr                = aww->get_root();
+    AW_root  *awr   = aww->get_root();
+    GB_ERROR  error = 0;
     char      pt_server[256];
-    char     *server;
-    char     *file;
-    GB_ERROR  error              = 0;
 
     bool create_gene_server = awr->awar(AWAR_PROBE_CREATE_GENE_SERVER)->read_int();
     {
@@ -1824,8 +1842,9 @@ void pd_export_pt_server(AW_window *aww)
         }
     }
 
-    sprintf(pt_server,"ARB_PT_SERVER%li",awr->awar(AWAR_PROBE_ADMIN_PT_SERVER)->read_int());
-    
+    long serverid = awr->awar(AWAR_PROBE_ADMIN_PT_SERVER)->read_int();
+    sprintf(pt_server,"ARB_PT_SERVER%li", serverid);
+
     if (!error &&
         aw_message("This function will send your currently loaded data as the new data to the pt_server !!!\n"
                    "The server will need a long time (up to several hours) to analyse the data.\n"
@@ -1834,14 +1853,19 @@ void pd_export_pt_server(AW_window *aww)
                    "Note 2: The server will do the job in background,\n"
                    "        quitting this program won't affect the server","Cancel,Do it"))
     {
-        aw_openstatus("Export db to server");
-        aw_status("Search server to kill");
+        aw_openstatus("Updating PT-server");
+        aw_status("Stopping PT-server");
         arb_look_and_kill_server(AISC_MAGIC_NUMBER,pt_server);
-        server = GBS_read_arb_tcp(pt_server);
-        file = server;              /* i got the machine name of the server */
-        if (*file) file += strlen(file)+1;  /* now i got the command string */
-        if (*file) file += strlen(file)+1;  /* now i got the file */
-        if (*file == '-') file += 2;
+
+        const char *ipPort = GBS_read_arb_tcp(pt_server);
+        const char *file   = GBS_scan_arb_tcp_param(ipPort, "-d");
+
+        GBS_add_ptserver_logentry(GBS_global_string("Started build of '%s'", file));
+        {
+            char *db_name = awr->awar(AWAR_DB_PATH)->read_string();
+            GBS_add_ptserver_logentry(GBS_global_string("Exporting DB '%s'", db_name));
+            free(db_name);
+        }
 
         aw_status("Exporting the database");
         {
@@ -1853,7 +1877,8 @@ void pd_export_pt_server(AW_window *aww)
 
                 if (!error) {
                     // convert database (genes -> species)
-                    aw_status("Preparing database for gene PT server");
+                    aw_status("Preparing DB for gene PT server");
+                    GBS_add_ptserver_logentry("Preparing DB for gene PT server");
                     char *command = GBS_global_string_copy("$ARBHOME/bin/arb_gene_probe %s %s", temp_server_name, file);
                     printf("Executing '%s'\n", command);
                     int result = system(command);
@@ -1881,17 +1906,29 @@ void pd_export_pt_server(AW_window *aww)
         }
 
         if (!error ) {
-            aw_status("Start new server");
+            aw_status("Start PT-server (builds in background)");
             error = arb_start_server(pt_server,gb_main, 1);
         }
         aw_closestatus();
+
     }
     if (error) aw_message(error);
 }
 
-void pd_edit_arb_tcp(AW_window *aww){
-    awt_edit(aww->get_root(),"$(ARBHOME)/lib/arb_tcp.dat",900,400);
+static void arb_tcp_dat_changed_cb(const char */*path*/, bool fileChanged, bool /*editorTerminated*/) {
+#if defined(DEBUG) && 0
+    printf("File '%s': changed=%i editorTerminated=%i\n", path, int(fileChanged), int(editorTerminated));
+#endif // DEBUG
+    if (fileChanged) {
+        awt_refresh_all_pt_server_selection_lists();
+    }
 }
+
+static void pd_edit_arb_tcp(AW_window *aww, AW_CL cl_gb_main) {
+    GBDATA *gb_main = (GBDATA*)cl_gb_main;
+    AWT_edit("$(ARBHOME)/lib/arb_tcp.dat", arb_tcp_dat_changed_cb, aww, gb_main);
+}
+
 AW_window *create_probe_admin_window( AW_root *root, AW_CL cl_genome_db)  {
     AW_window_simple *aws         = new AW_window_simple;
     bool              is_genom_db = (bool)cl_genome_db;
@@ -1920,7 +1957,7 @@ AW_window *create_probe_admin_window( AW_root *root, AW_CL cl_genome_db)  {
 
     aws->at( "kill" );
     aws->callback(pd_kill_pt_server,0);
-    aws->create_button("KILL_SERVER","Kill server");
+    aws->create_button("KILL_SERVER","Stop server");
 
     aws->at( "query" );
     aws->callback(pd_query_pt_server);
@@ -1928,15 +1965,15 @@ AW_window *create_probe_admin_window( AW_root *root, AW_CL cl_genome_db)  {
 
     aws->at( "kill_all" );
     aws->callback(pd_kill_pt_server,1);
-    aws->create_button("KILL_ALL_SERVERS","Kill all servers");
+    aws->create_button("KILL_ALL_SERVERS","Stop all servers");
 
     aws->at( "edit" );
-    aws->callback(pd_edit_arb_tcp);
+    aws->callback(pd_edit_arb_tcp, (AW_CL)gb_main);
     aws->create_button("CREATE_TEMPLATE","Configure");
 
     aws->at( "export" );
     aws->callback(pd_export_pt_server);
-    aws->create_button("UPDATE_SERVER","Update server");
+    aws->create_button("UPDATE_SERVER","Build server");
 
     if (is_genom_db) {
         aws->at( "gene_server" );
