@@ -1,6 +1,8 @@
 #include "RNA3D_GlobalHeader.hxx"
 
 // The following includes are needed to use AW_window_Motif 
+#include <aw_root.hxx>
+#include <aw_window.hxx>
 #include "../WINDOW/aw_awar.hxx"
 #include "../WINDOW/aw_Xm.hxx"
 #include "../WINDOW/aw_click.hxx"
@@ -13,6 +15,16 @@
 #include "RNA3D_OpenGLEngine.hxx"
 #include "RNA3D_Interface.hxx"
 #include "RNA3D_OpenGLGraphics.hxx"
+#include "RNA3D_StructureData.hxx"
+
+#include <string>
+#include <iostream>
+
+#include <X11/keysym.h>
+
+#include <aw_preset.hxx>
+#include <aw_awars.hxx>
+
 
 using namespace std;
 
@@ -29,8 +41,7 @@ static GLboolean                    Spinning = GL_FALSE;
 //    the animation will continue.
 //===============================================================================//
 
-Boolean SpinMolecule(XtPointer clientData) {
-    AWUSE(clientData);
+Boolean SpinMolecule(XtPointer /*clientData*/) {
     RNA3D->ROTATION_SPEED = 0.05;
     RefreshOpenGLDisplay();
     return false; /* leave work proc active */
@@ -69,7 +80,7 @@ static void RotateMoleculeStateChanged_cb(AW_root *awr) {
     RefreshOpenGLDisplay();
 }
 
-void ResizeOpenGLWindow( Widget w, XtPointer client_data, XEvent *event, char* x ) {
+void ResizeOpenGLWindow( Widget /* w*/, XtPointer /*client_data*/, XEvent *event, char* /*x*/ ) {
 	XConfigureEvent *evt;
 	evt = (XConfigureEvent*) event;
 	
@@ -82,47 +93,18 @@ void ResizeOpenGLWindow( Widget w, XtPointer client_data, XEvent *event, char* x
     RefreshOpenGLDisplay();
 }
 
-void KeyReleaseEventHandler( Widget w, XtPointer client_data, XEvent *event, char* x ) {
-	XKeyEvent *evt;
-    evt = (XKeyEvent*) event;
-
-    char   buffer[1];
-    KeySym keysym;
-    int    count;
-
-    // Converting keycode to keysym
-    count = XLookupString((XKeyEvent *) event, buffer, 1, &keysym, NULL);
-
-    switch(keysym) {
-    case XK_Up:
-        //        RNA3D->bRotateMolecule = false;   
-        break;
-    case XK_Down:
-        //      RNA3D->bRotateMolecule = false;   
-        break;
-    case XK_Left:
-        //        RNA3D->bRotateMolecule = false;   
-        break;
-    case XK_Right:
-        //        RNA3D->bRotateMolecule = false;   
-        break;
-    }
-
+void KeyReleaseEventHandler(Widget /* w*/, XtPointer /*client_data*/, XEvent *event, char* /*x*/ )  {
     RefreshOpenGLDisplay();
 }
 
-void KeyPressEventHandler( Widget w, XtPointer client_data, XEvent *event, char* x ) {
-	XKeyEvent *evt;
+void KeyPressEventHandler(Widget /* w*/, XtPointer /*client_data*/, XEvent *event, char* /*x*/ )  {
+    XKeyEvent *evt;
     evt = (XKeyEvent*) event;
 
     char   buffer[1];
     KeySym keysym;
     int    count;
-    float fRotationFactor = 2.0f;
-
-//     RNA3D->saved_x = evt->x;
-//     RNA3D->saved_y = evt->y;
-
+    
     // Converting keycode to keysym
     count = XLookupString((XKeyEvent *) event, buffer, 1, &keysym, NULL);
 
@@ -132,38 +114,29 @@ void KeyPressEventHandler( Widget w, XtPointer client_data, XEvent *event, char*
         RNA3D->root->awar(AWAR_3D_MOL_ROTATE)->write_int(RNA3D->iRotateMolecule);
         break;
     case XK_Escape:
-        exit(0); 
+        RNA3D->bDisplayComments = !RNA3D->bDisplayComments;
+        break;
+    case XK_Tab:
+        RNA3D->bDisplayMask = !RNA3D->bDisplayMask;
         break;
     case XK_Up:
         RNA3D->Center.y -= 0.1;
-//         RNA3D->bRotateMolecule = true;   
-//         RNA3D->saved_y += fRotationFactor;
-//         ComputeRotationXY(evt->x, evt->y);
         break;
     case XK_Down:
         RNA3D->Center.y += 0.1;
-//         RNA3D->bRotateMolecule = true;   
-//         RNA3D->saved_y -= fRotationFactor;
-//         ComputeRotationXY(evt->x, evt->y);
         break;
     case XK_Left:
         RNA3D->Center.x -= 0.1;
-//         RNA3D->bRotateMolecule = true;   
-//         RNA3D->saved_x += fRotationFactor;
-//         ComputeRotationXY(evt->x, evt->y);
         break;
     case XK_Right:
         RNA3D->Center.x += 0.1;
-//         RNA3D->bRotateMolecule = true;   
-//         RNA3D->saved_x -= fRotationFactor;
-//         ComputeRotationXY(evt->x, evt->y);
         break;
     }
 
     RefreshOpenGLDisplay();
 }
 
-void ButtonReleaseEventHandler( Widget w, XtPointer client_data, XEvent *event, char* x ) {
+void ButtonReleaseEventHandler(Widget /* w*/, XtPointer /*client_data*/, XEvent *event, char* /*x*/ ) {
 	XButtonEvent *xr;
 	xr = (XButtonEvent*) event;
 
@@ -183,7 +156,7 @@ void ButtonReleaseEventHandler( Widget w, XtPointer client_data, XEvent *event,
     RefreshOpenGLDisplay();
 }
 
-void ButtonPressEventHandler( Widget w, XtPointer client_data, XEvent *event, char* x ) {
+void ButtonPressEventHandler( Widget /* w*/, XtPointer /*client_data*/, XEvent *event, char* /*x*/ ){
 
 	XButtonEvent *xp;
 	xp = (XButtonEvent*) event;
@@ -215,7 +188,7 @@ void ButtonPressEventHandler( Widget w, XtPointer client_data, XEvent *event, ch
     RefreshOpenGLDisplay();
 }
 
-void MouseMoveEventHandler( Widget w, XtPointer client_data, XEvent *event, char* x ) {
+void MouseMoveEventHandler(Widget /* w*/, XtPointer /*client_data*/, XEvent *event, char* /*x*/ ) {
 	
 	XMotionEvent *xp;
 	xp = (XMotionEvent*) event;
@@ -232,7 +205,7 @@ void MouseMoveEventHandler( Widget w, XtPointer client_data, XEvent *event, char
     RefreshOpenGLDisplay();
 }
 
-void ExposeOpenGLWindow( Widget w, XtPointer client_data, XEvent *event, char* x ) {
+void ExposeOpenGLWindow( Widget  w, XtPointer /*client_data*/, XEvent *event, char* /*x*/ ) {
     static bool ok = false;
 
 	if ( RNA3D->OpenGLEngineState == NOT_CREATED ) {
@@ -271,9 +244,55 @@ static void RefreshCanvas(AW_root *awr) {
     RefreshOpenGLDisplay();
 }
 
+static void SyncronizeColorsWithEditor(AW_window *aww) {
+    // overwrites color settings with those from EDIT4
+
+    AW_copy_GCs(aww->get_root(), "ARB_EDIT4", "RNA3D", false,
+                "User1",   "User2",   "Probe",
+                "Primerl", "Primerr", "Primerg",
+                "Sigl",    "Sigr",    "Sigg",
+                "RANGE_0", "RANGE_1", "RANGE_2",
+                "RANGE_3", "RANGE_4", "RANGE_5",
+                "RANGE_6", "RANGE_7", "RANGE_8",
+                "RANGE_9",
+                0);
+}
+
+static void Change3DMolecule_CB(AW_root *awr) {
+    cout<<"Rendering new 3D molecule.... please wait..."<<endl;
+
+    RNA3D->cStructure->LSU_molID = awr->awar(AWAR_3D_23S_RRNA_MOL)->read_int();
+
+    RNA3D->cStructure->DeleteOldMoleculeData();        // Deleting the old molecule data
+    RNA3D->cStructure->ReadCoOrdinateFile();           // Reading Structure information
+    RNA3D->cStructure->GetSecondaryStructureInfo();    // Getting Secondary Structure Information
+    RNA3D->cStructure->Combine2Dand3DstructureInfo();  // Combining Secondary Structure data with 3D Coordinates
+    RNA3D->cStructure->GenerateDisplayLists();         // Generating Display Lists for Rendering
+
+    // recaluculate the mapping information
+    awr->awar(AWAR_SPECIES_NAME)->touch();
+
+    // recaluculate helix numbering  
+    awr->awar(AWAR_3D_HELIX_FROM)->touch();
+
+    // render new structure in OpenGL window
+    RefreshCanvas(awr);
+}
+
+static void Change3DMolecule(AW_window *aww, long int molID) {
+    // changes the displayed 3D structure in the case of 23S rRNA
+    aww->get_root()->awar(AWAR_3D_23S_RRNA_MOL)->write_int(molID);
+}
+
+static void DisplayMoleculeMask(AW_root */*awr */){
+   RNA3D->bDisplayMask = !RNA3D->bDisplayMask;
+    RefreshOpenGLDisplay();
+}
+
+
 /*---------------------------- Creating WINDOWS ------------------------------ */
 static void AddCallBacks(AW_root *awr) {
-    // adding callbacks to the awars to refresh the display if recieved any changes
+    // adding callbacks to the awars to refresh the display if received any changes
 
     // General Molecule Display  Section
     awr->awar(AWAR_3D_MOL_BACKBONE)->add_callback(RefreshCanvas);
@@ -321,13 +340,15 @@ static void AddCallBacks(AW_root *awr) {
     awr->awar(AWAR_3D_MAP_SPECIES_DISP_MISSING)->add_callback(RefreshCanvas);
     awr->awar(AWAR_3D_MAP_SPECIES_DISP_INSERTIONS)->add_callback(RefreshCanvas);
     awr->awar(AWAR_3D_MAP_SPECIES_DISP_INSERTIONS_INFO)->add_callback(RefreshCanvas);
+
+    awr->awar(AWAR_3D_DISPLAY_MASK)->add_callback(DisplayMoleculeMask);
+    awr->awar(AWAR_3D_23S_RRNA_MOL)->add_callback(Change3DMolecule_CB);
 }
 
-static void RefreshMappingDisplay(AW_window *aw) {
+static void RefreshMappingDisplay(AW_window */*aw */) {
     // Refreshes the SAI Display if and when ...
     // 1.Changes made to SAI related settings in EDIT4 and not updated automatically 
     // 2.Colors related to SAI Display changed in RNA3D Application
-
     MapSaiToEcoliTemplateChanged_CB(RNA3D->root);
 
     // Refreshes the Search Strings Display if 
@@ -353,7 +374,7 @@ static AW_window *CreateDisplayBases_window(AW_root *aw_root) {
     aws->init( aw_root, "DISPLAY_BASES", "RNA3D : Display BASES");
     aws->load_xfig("RNA3D_DisplayBases.fig");
 
-    aws->callback( AW_POPUP_HELP,(AW_CL)"rna3d_displayOptions.hlp");
+    aws->callback( AW_POPUP_HELP,(AW_CL)"rna3d_dispBases.hlp");
     aws->at("help");
     aws->button_length(0);
     aws->create_button("HELP","#help.xpm");
@@ -425,7 +446,7 @@ static AW_window *CreateDisplayHelices_window(AW_root *aw_root) {
     aws->init( aw_root, "DISPLAY_HELICES", "RNA3D : Display HELICES");
     aws->load_xfig("RNA3D_DisplayHelices.fig");
 
-    aws->callback( AW_POPUP_HELP,(AW_CL)"rna3d_displayOptions.hlp");
+    aws->callback( AW_POPUP_HELP,(AW_CL)"rna3d_dispHelices.hlp");
     aws->at("help");
     aws->button_length(0);
     aws->create_button("HELP","#help.xpm");
@@ -451,6 +472,22 @@ static AW_window *CreateDisplayHelices_window(AW_root *aw_root) {
         aws->create_input_field(AWAR_3D_HELIX_TO, 5);
         aws->at("helixSize");
         aws->create_input_field(AWAR_3D_HELIX_SIZE, 5);
+        {
+            const char *helixRange;
+            Structure3D *s;
+#if defined(DEVEL_RALF)
+#warning s is not initialized here
+#endif // DEVEL_RALF
+            int rnaType = s->FindTypeOfRNA();
+            
+            switch (rnaType) {
+            case LSU_23S: helixRange = "[1-101]"; break;
+            case SSU_16S: helixRange = "[1-50]";  break;
+            case LSU_5S:  helixRange = "[1-5]";   break;
+            }
+            aws->at("rangeLabel");
+            aws->create_autosize_button(0,helixRange);
+        }
 
         aws->at("dispTI");
         aws->create_toggle(AWAR_3D_DISPLAY_TERTIARY_INTRACTIONS);
@@ -468,7 +505,7 @@ static AW_window *CreateDisplayOptions_window(AW_root *aw_root) {
     aws->init( aw_root, "GENERAL_DISPLAY", "RNA3D : General Display ");
     aws->load_xfig("RNA3D_DisplayOptions.fig");
 
-    aws->callback( AW_POPUP_HELP,(AW_CL)"rna3d_displayOptions.hlp");
+    aws->callback( AW_POPUP_HELP,(AW_CL)"rna3d_dispMolecule.hlp");
     aws->at("help");
     aws->button_length(0);
     aws->create_button("HELP","#help.xpm");
@@ -508,7 +545,7 @@ static AW_window *CreateMapSequenceData_window(AW_root *aw_root) {
     aws->init( aw_root, "MAP_SPECIES", "RNA3D : Map Sequence Data ");
     aws->load_xfig("RNA3D_SeqMapping.fig");
 
-    aws->callback( AW_POPUP_HELP,(AW_CL)"rna3d_displayOptions.hlp");
+    aws->callback( AW_POPUP_HELP,(AW_CL)"rna3d_mapSeqData.hlp");
     aws->at("help");
     aws->button_length(0);
     aws->create_button("HELP","#help.xpm");
@@ -517,6 +554,11 @@ static AW_window *CreateMapSequenceData_window(AW_root *aw_root) {
     aws->callback((AW_CB0)AW_POPDOWN);
     aws->button_length(0);
     aws->create_button("CLOSE","#closeText.xpm");
+    
+    aws->callback(SyncronizeColorsWithEditor);
+    aws->at("sync");
+    aws->button_length(35);
+    aws->create_button("SYNC","SYNCHRONIZE COLORS WITH EDITOR");
 
     {  // Display Map Current Species Section
         aws->at("en");
@@ -552,8 +594,93 @@ static AW_window *CreateMapSequenceData_window(AW_root *aw_root) {
     return (AW_window *)aws;
 }
 
+static AW_window *CreateChangeMolecule_window(AW_root *aw_root) {
+    static AW_window_simple *aws = 0;
+    if (aws) return (AW_window *)aws;
+
+    aws = new AW_window_simple;
+
+    aws->init( aw_root, "CHANGE_MOLECULE", "RNA3D : Change 3D Molecule");
+    aws->load_xfig("RNA3D_ChangeMolecule.fig");
+
+    aws->callback( AW_POPUP_HELP,(AW_CL)"rna3d_changeMolecule.hlp");
+    aws->at("help");
+    aws->button_length(0);
+    aws->create_button("HELP","#help.xpm");
+
+    aws->at("close");
+    aws->callback((AW_CB0)AW_POPDOWN);
+    aws->button_length(0);
+    aws->create_button("CLOSE","#closeText.xpm");
+
+    aws->callback(Change3DMolecule,1);
+    aws->at("1pnu");
+    aws->button_length(73);
+    aws->create_button(0,"1PNU: 8.7 A^ Vila-Sanjurjo et al. Proc.Nat.Acad.Sci.(2003) 100, 8682.");
+
+    aws->callback(Change3DMolecule,2);
+    aws->at("1vor");
+    aws->button_length(73);
+    aws->create_button(0,"1VOR: 11.5 A^ Vila-Sanjurjo et al. Nat.Struct.Mol.Biol.(2004) 11, 1054.");
+
+    aws->callback(Change3DMolecule,3);
+    aws->at("1c2w");
+    aws->button_length(73);
+    aws->create_button(0,"1C2W: 7.5 A^ Mueller et al. J.Mol.Biol.(2000) 298, 35-59.", 0, "white");
+
+    aws->show();
+    return (AW_window *)aws;
+}
+
+static AW_window *CreateHelp_window(AW_root *aw_root) {
+    static AW_window_simple *aws = 0;
+    if (aws) return (AW_window *)aws;
+
+    aws = new AW_window_simple;
+
+    aws->init( aw_root, "HELP", "RNA3D : Display Options & Shortcuts");
+    aws->load_xfig("RNA3D_Help.fig");
+
+    aws->button_length(0);
+
+    aws->callback( AW_POPUP_HELP,(AW_CL)"rna3d_general.hlp");
+    aws->at("help");
+    aws->create_button("HELP","#help.xpm");
+
+    aws->at("close");
+    aws->callback((AW_CB0)AW_POPDOWN);
+    aws->create_button("CLOSE","#closeText.xpm");
+
+    aws->at("reload");
+    aws->create_button("reload","#refresh.xpm");
+    aws->at("color");
+    aws->create_button("colors", "#colors.xpm");
+    aws->at("base");
+    aws->create_button("displayBases", "#bases.xpm");
+    aws->at("helix");
+    aws->create_button("displayHelix", "#helix.xpm");
+    aws->at("mol");
+    aws->create_button("displayMolecule", "#molText.xpm");
+    aws->at("map");
+    aws->create_button("mapSpecies", "#mapping.xpm");
+    aws->at("check");
+    aws->create_button("check", "#check.xpm");
+    aws->at("uncheck");
+    aws->create_button("uncheck", "#uncheck.xpm");
+    aws->at("mask");
+    aws->create_button("mask", "#mask.xpm");
+    aws->at("unmask");
+    aws->create_button("unmask", "#unmask.xpm");
+    aws->at("exit");
+    aws->create_button("exit", "#quit.xpm");
+
+    aws->show();
+    return (AW_window *)aws;
+}
+
 AW_window *CreateRNA3DMainWindow(AW_root *awr){
     // Main Window - Canvas on which the actual painting is done
+    extern GBDATA *gb_main;
     GB_transaction dummy(gb_main);
 
     awr->awar_int(AWAR_3D_SAI_SELECTED, 0, AW_ROOT_DEFAULT); 
@@ -563,9 +690,11 @@ AW_window *CreateRNA3DMainWindow(AW_root *awr){
     awm = new AW_window_menu_modes_opengl();
     awm->init(awr,"RNA3D", "RNA3D: 3D Structure of Ribosomal RNA", WINDOW_WIDTH, WINDOW_HEIGHT);
 
-    AW_gc_manager aw_gc_manager;
+    AW_gc_manager   aw_gc_manager;
     RNA3D_Graphics *rna3DGraphics = new RNA3D_Graphics(awr,gb_main);
 
+    Structure3D::gb_main = gb_main;
+
     RNA3D->gl_Canvas = new AWT_canvas(gb_main,awm, rna3DGraphics, aw_gc_manager,AWAR_SPECIES_NAME);
 
     RNA3D->gl_Canvas->recalc_size();
@@ -573,19 +702,34 @@ AW_window *CreateRNA3DMainWindow(AW_root *awr){
     RNA3D->gl_Canvas->set_mode(AWT_MODE_NONE); 
 
     awm->create_menu( 0, "File", "F", 0,  AWM_ALL );
+    {    
+        Structure3D *s;
+        int rnaType = s->FindTypeOfRNA();
+        if (rnaType == LSU_23S) 
+            awm->insert_menu_topic( "changeMolecule", "Change Molecule", "M","rna3d_changeMolecule.hlp", AWM_ALL, AW_POPUP, (AW_CL)CreateChangeMolecule_window, 0);
+    }
     awm->insert_menu_topic( "close", "Close", "C","quit.hlp", AWM_ALL, (AW_CB)AW_POPDOWN, 1,0);
 
     {
-        awm->at(11,2);
+        awm->at(1,2);
         awm->auto_space(2,0);
         awm->shadow_width(1);
 
-        int cur_x, cur_y, start_x, first_line_y;
+        int cur_x, cur_y, start_x, first_line_y, second_line_y;
         awm->get_at_position( &start_x,&first_line_y);
         awm->button_length(0);
         awm->callback( (AW_CB0)AW_POPDOWN );
-        awm->create_button("Quit", "#quitText.xpm");
-    
+        awm->create_button("Quit", "#quit.xpm");
+
+        awm->get_at_position( &cur_x,&cur_y );
+        awm->callback(RefreshMappingDisplay);
+        awm->button_length(0);
+        awm->create_button("REFRESH","#refresh.xpm");
+
+        awm->get_at_position( &cur_x,&cur_y );
+        awm->button_length(0);
+        awm->create_toggle(AWAR_3D_DISPLAY_MASK, "#unmask.xpm", "#mask.xpm");
+
         awm->get_at_position( &cur_x,&cur_y );
         awm->callback(AW_POPUP,(AW_CL)AW_create_gc_window,(AW_CL)aw_gc_manager);
         awm->button_length(0);
@@ -616,7 +760,7 @@ AW_window *CreateRNA3DMainWindow(AW_root *awr){
 
         awm->get_at_position( &cur_x,&cur_y );
         awm->callback(AW_POPUP,(AW_CL)CreateMapSequenceData_window,(AW_CL)0);
-        awm->button_length(12);
+        awm->button_length(0);
         awm->create_button("mapSpecies", "#mapping.xpm");
 
         awm->get_at_position( &cur_x,&cur_y );
@@ -624,14 +768,13 @@ AW_window *CreateRNA3DMainWindow(AW_root *awr){
         awm->create_toggle(AWAR_3D_MAP_ENABLE, "#uncheck.xpm", "#check.xpm");
 
         awm->get_at_position( &cur_x,&cur_y );
-        awm->callback(RefreshMappingDisplay);
-        awm->button_length(0);
-        awm->create_button("REFRESH","#refresh.xpm");
-
-        awm->get_at_position( &cur_x,&cur_y );
-        awm->callback( AW_POPUP_HELP,(AW_CL)"rna3d_displayOptions.hlp");
+        awm->callback(AW_POPUP,(AW_CL)CreateHelp_window,(AW_CL)0);
         awm->button_length(0);
         awm->create_button("help", "#helpText.xpm");
+
+        awm->at_newline();
+        awm->get_at_position( &cur_x,&second_line_y);
+        awm->create_autosize_button(0," Spacebar = auto rotate mode on/off | Mouse Left Button + Move = Rotates Molecule | Mouse Wheel = Zoom in/out");
     }
 
     AddCallBacks(awr);
