@@ -8,10 +8,17 @@
 #include "RNA3D_Graphics.hxx"
 #include "RNA3D_Interface.hxx"
 
+#include <iostream>
+
+#include <aw_awars.hxx>
+#include <BI_helix.hxx>
+
+
 // --------------------------------------------------------------------------------
 // global data
 
 RNA3D_Global *RNA3D = 0; 
+char globalComment[1000];
 
 void RNA3D_init_global_data() {
     if (!RNA3D) {
@@ -27,6 +34,8 @@ RNA3D_Global::RNA3D_Global(){
     bMapSaiDispListCreated           = false;
     bAutoRotate                      = false;
     bRotateMolecule                  = false;
+    bDisplayMask                     = false;
+    bDisplayComments                 = false;
     bMapSearchStringsDispListCreated = false;
 
     ROTATION_SPEED = 0.5;
@@ -53,9 +62,9 @@ RNA3D_Global::~RNA3D_Global() {
 // --------------------------------------------------------------------------------
 
 static float fAspectRatio;
-const float fViewAngle = 90.0;
-const float fClipNear  = 0.01;
-const float fClipFar   = 100000;
+const float fViewAngle = 40.0;
+const float fClipNear  = 0.5f;
+const float fClipFar   = 100;
 
 // GBDATA *OpenGL_gb_main;
 
@@ -159,13 +168,17 @@ void InitializeOpenGLEngine(GLint width, GLint height ) {
     ShowVendorInformation();
 
     GLenum err = glewInit();
-    if (GLEW_OK != err) {
+    if (GLEW_OK != err){
         /* problem: glewInit failed, something is seriously wrong */
         fprintf(stderr, "Error: %s\n", glewGetErrorString(err));
     }
     fprintf(stdout, "Status: Using GLEW %s\n", glewGetString(GLEW_VERSION));
 
     initExtensions();
+    
+    { // Preparing secondary structure masks for RNA3D program
+        //        RNA3D->cStructure->PrepareSecondaryStructureData();
+    }
 
     // Prepare the structure Data  and Generate Display Lists  
 
@@ -173,21 +186,21 @@ void InitializeOpenGLEngine(GLint width, GLint height ) {
     RNA3D->cStructure->GetSecondaryStructureInfo();  // Getting Secondary Structure Information
     RNA3D->cStructure->Combine2Dand3DstructureInfo(); // Combining Secondary Structure data with 3D Coordinates
 
-    //    RNA3D->cStructure->PrepareStructureSkeleton();    // Preparing structure skeleton with just coordinates  
     RNA3D->cStructure->GenerateDisplayLists(); // Generating Display Lists for Rendering
 
     // Generate Textures
     RNA3D->cTexture->LoadGLTextures();  // Load The Texture(s) 
-    glEnable(GL_TEXTURE_2D);    // Enable Texture Mapping
 
-    glDepthFunc(GL_LEQUAL);          // The Type Of Depth Test To Do
+    glShadeModel(GL_SMOOTH);
+    glClearColor(0,0,0,1);
+    glClearDepth(1.0f); 
+
     glEnable(GL_DEPTH_TEST);         // Enables Depth Testing
-    glClearDepth(1.0);
-    glShadeModel(GL_FLAT);           
-
-    glEnable(GL_POINT_SMOOTH);
+    glDepthFunc(GL_LEQUAL);          // The Type Of Depth Test To Do
 
+    //    glHint(GL_CLIP_VOLUME_CLIPPING_HINT_EXT,GL_DONT_CARE);
     glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
+    glEnable(GL_TEXTURE_2D);    // Enable Texture Mapping
 
     ReshapeOpenGLWindow(width,height);
 
@@ -276,13 +289,21 @@ void MapDisplayParameters(AW_root *root){
     cRenderer->iMapSpeciesInsInfo = root->awar(AWAR_3D_MAP_SPECIES_DISP_INSERTIONS_INFO)->read_int();
 
     { // Validation of Helix Numbers entered by the User
-        if (cRenderer->iStartHelix < 1 ||  cRenderer->iStartHelix > 50 ) {
+        int NoOfHelices = 0;
+        int rnaType = cStructure->FindTypeOfRNA();
+        switch (rnaType){
+        case LSU_23S: NoOfHelices = 101; break;
+        case SSU_16S: NoOfHelices = 50;  break;
+        case LSU_5S:  NoOfHelices = 5;   break;
+        }
+
+        if (cRenderer->iStartHelix < 1 ||  cRenderer->iStartHelix > NoOfHelices ) {
             cout<<"Invalid Helix NUMBER !!"<<endl;
             root->awar(AWAR_3D_HELIX_FROM)->write_int(1);
         }
-        if (cRenderer->iEndHelix < 1 ||  cRenderer->iEndHelix > 50 ) {
+        if (cRenderer->iEndHelix < 1 ||  cRenderer->iEndHelix > NoOfHelices ) {
             cout<<"Invalid Helix NUMBER !!"<<endl;
-            root->awar(AWAR_3D_HELIX_TO)->write_int(50);
+            root->awar(AWAR_3D_HELIX_TO)->write_int(NoOfHelices);
         }
 
         if(cRenderer->iStartHelix > cRenderer->iEndHelix) {
@@ -390,8 +411,7 @@ void CursorPositionChanged_CB(AW_root *awr) {
 
     if(RNA3D->bEColiRefInitialised) {
         long iCursorPos = awr->awar(AWAR_CURSOR_POSITION)->read_int();
-        long EColiPos, dummy;
-        RNA3D->cStructure->EColiRef->abs_2_rel(iCursorPos, EColiPos, dummy);
+        long EColiPos   = RNA3D->cStructure->EColiRef->abs_2_rel(iCursorPos);
 
         if (!bCursorPosDispListCreated) {
             RNA3D->cStructure->GenerateCursorPositionDispList(EColiPos);
@@ -437,7 +457,7 @@ void DrawStructure(){
 }
 
 void RenderOpenGLScene(Widget w){
-    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); 
+    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);// | GL_STENCIL_BUFFER_BIT); 
 
     // setting the BackGround Color of the OpenGL Scene
     RNA3D->cGraphics->SetOpenGLBackGroundColor();  
@@ -449,18 +469,20 @@ void RenderOpenGLScene(Widget w){
               RNA3D->Up.x,     RNA3D->Up.y,     RNA3D->Up.z);
 
     {// Displaying Molecule Name
-        RNA3D->cRenderer->DisplayMoleculeName(iScreenWidth, iScreenHeight);
+        RNA3D->cRenderer->DisplayMoleculeName(iScreenWidth, iScreenHeight, RNA3D->cStructure);
     }
-
+    
     glScalef(RNA3D->scale, RNA3D->scale, RNA3D->scale);
 
+    RNA3D->cRenderer->DisplayMoleculeMask(iScreenWidth, iScreenHeight);
+
     if(RNA3D->bRotateMolecule || RNA3D->bAutoRotate) {
         CalculateRotationMatrix();
     }
 
     glMultMatrixf(rotation_matrix);
 
-    Vector3& strCen = *RNA3D->cStructure->strCen;
+    Vector3& strCen = *RNA3D->cStructure->strCen; 
     glTranslatef(-strCen.x, -strCen.y, -strCen.z);
 
     DrawStructure();
@@ -473,7 +495,7 @@ void RenderOpenGLScene(Widget w){
 void InitializeOpenGLWindow( Widget w ) {
 
     if (RNA3D->OpenGLEngineState == CREATED) return;
-    
+
     Arg args[1];
     XVisualInfo *vi;
     
@@ -512,6 +534,10 @@ void InitializeOpenGLWindow( Widget w ) {
             RNA3D->glw = w;
 
             RNA3D->OpenGLEngineState = CREATED;
+
+            // Initializing fonts
+	char fontName[] = "fixed";
+            RNA3D->cGraphics->InitMainFont(fontName);
         }
     }
 }
