@@ -4,11 +4,16 @@
 #include "RNA3D_Graphics.hxx"
 #include "RNA3D_StructureData.hxx"
 
-#include "../EDIT4/ed4_class.hxx"
-#include "../EDIT4/ed4_defs.hxx"  //for background colors
-#include "../EDIT4/ed4_visualizeSAI.hxx"
-
 #include <cerrno>
+#include <string>
+#include <iostream>
+#include <fstream>
+
+#include <arbdbt.h>
+#include <aw_awars.hxx>
+#include <ed4_extern.hxx>
+#include <BI_helix.hxx>
+
 
 #define COLORLINK (ED4_G_SBACK_0 - RNA3D_GC_SBACK_0)  // to link to the colors defined in primary editor ed4_defs.hxx
 #define SAICOLORS (ED4_G_CBACK_0 - RNA3D_GC_CBACK_0)
@@ -32,11 +37,14 @@ static char *find_data_file(const char *name) {
     return fname;
 }
 
+static void throw_IO_error(const char *filename) __attribute__((noreturn));
 static void throw_IO_error(const char *filename) {
     string error = string("IO-Error: ")+strerror(errno)+" ('"+filename+"')";
     throw error;
 }
 
+GBDATA *Structure3D::gb_main = 0;
+
 Structure3D::Structure3D(void) {
     strCen          = new Vector3(0.0, 0.0, 0.0);
     GRAPHICS        = new OpenGLGraphics();
@@ -49,6 +57,11 @@ Structure3D::Structure3D(void) {
     iEColiEndPos    = 0;
     iStartPos       = 0;
     iEndPos         = 0;
+    iTotalSubs      = 0;
+    iTotalDels      = 0;
+    iTotalIns       = 0;
+    LSU_molID       = 3;  // default molecule to load in case of 23S rRNA : 1C2W
+    HelixBase       = 500;
 }
 
 Structure3D::~Structure3D(void) {
@@ -79,16 +92,136 @@ void Structure3D::StoreCoordinates(float x, float y, float z, char base, unsigne
     }
 }
 
+//------------------Selecting rRNA type--------------------------//
+
+int Structure3D::FindTypeOfRNA(){
+    int rnaType = 0;       
+    GB_push_transaction(gb_main);  //opening a transaction 
+
+    GBDATA *gbTemplate = GBT_find_SAI(gb_main,"ECOLI");
+    if (!gbTemplate) {
+        aw_message("SAI:ECOLI not found");
+    }
+    else {
+        char   *ali_name          = GBT_get_default_alignment(gb_main);
+        GBDATA *gbAlignment       = GB_find(gbTemplate, ali_name, 0, down_level);
+        GBDATA *gbTemplateSeqData = gbAlignment ? GB_find(gbAlignment, "data", 0, down_level) : 0;
+                 
+        if (!gbTemplateSeqData) {
+          aw_message("Could not find species in the database!");
+        }
+        else {
+            const char *pTemplateSeqData  = GB_read_char_pntr(gbTemplateSeqData);
+            int iSeqLen = strlen(pTemplateSeqData); 
+            int iBaseCount = 0;  
+            for(int i = 0; i<iSeqLen; i++) {
+                if((pTemplateSeqData[i] != '.') && (pTemplateSeqData[i] != '-')){
+                    iBaseCount++;
+                }
+            }
+
+            if (iBaseCount > 2000)      rnaType = LSU_23S;
+            else if (iBaseCount > 300)  rnaType = SSU_16S;
+            else                        rnaType = LSU_5S;
+        }
+    }
+    GB_pop_transaction(gb_main);
+    
+    return rnaType;
+}
+
+//----------Delete old molecule data-------------------------------// 
+// Struct2Dinfo, Struct2Dplus3D, Struct3Dinfo, HelixNrInfo, Insertions
+
+void Structure3D::DeleteOldMoleculeData(){
+    // Struct2Dinfo -> start2D
+    {   
+        Struct2Dinfo *tmp, *data;
+        for(data = start2D; data != NULL; data = tmp) {
+            tmp = data->next;
+            delete data;
+        }
+        start2D = NULL; 
+    }
+
+    // Struct2Dplus3D ->start2D3D
+    {
+        Struct2Dplus3D *tmp, *data;
+        for(data = start2D3D; data != NULL; data = tmp) {
+            tmp = data->next;
+            delete data;
+        }
+        start2D3D = NULL; 
+    }
+
+    // Struct3Dinfo ->start3D
+    {
+        Struct3Dinfo *tmp, *data;
+        for(data = start3D; data != NULL; data = tmp) {
+            tmp = data->next;
+            delete data;
+        }
+        start3D = NULL; 
+    }
+
+    // HelixNrInfo -> start
+    {
+        HelixNrInfo *tmp, *data;
+        for(data = start; data != NULL; data = tmp) {
+            tmp = data->next;
+            delete data;
+        }
+        start = NULL; 
+    }
+
+    // Delete insertions data
+    DeleteOldInsertionData();
+
+    //Delelte mapped species data
+    DeleteOldSpeciesData ();
+}
+
 //=========== Reading 3D Coordinates from PDB file ====================//
 
 void Structure3D::ReadCoOrdinateFile() {
-    char *DataFile           = find_data_file("Ecoli_1M5G_16SrRNA.pdb");
+    static char *DataFile = 0;  
+    int rnaType    = FindTypeOfRNA();
+    
+    RNA3D->bDisplayComments = true; // displaying comments in main window
+
+    switch (rnaType) {
+    case LSU_23S:
+        switch(LSU_molID){
+        case _1PNU: 
+            DataFile = find_data_file("Ecoli_1PNU_23S_rRNA.pdb");
+            sprintf(globalComment, "The 3D molecule rendered from PDB entry : 1PNU at 8.7 Angstrom."); 
+            break;
+        case _1VOR: 
+            DataFile = find_data_file("Ecoli_1VOR_23S_rRNA.pdb");
+            sprintf(globalComment,"The 3D molecule is rendered from PDB entry [1VOR] with 11.5 Angstrom resolution."); 
+            break;
+        case _1C2W: 
+            DataFile = find_data_file("Ecoli_1C2W_23S_rRNA.pdb");
+            sprintf(globalComment,"The 3D molecule is rendered from PDB entry [1C2W] with 7.5 Angstrom resolution."); 
+            break;
+        }
+        break;
+    case LSU_5S: 
+        DataFile = find_data_file("Ecoli_1C2X_5S_rRNA.pdb");
+        sprintf(globalComment,"The 3D molecule is rendered from PDB entry [1C2X] with 7.5 Angstrom resolution."); 
+        break;
+    case SSU_16S:
+        DataFile = find_data_file("Ecoli_1M5G_16S_rRNA.pdb");
+        sprintf(globalComment, "The 3D molecule is rendered from PDB entry [1M5G] with 11.5 Angstrom resolution."); 
+        break;
+    }
+    
     char      buf[256];
 
     float X, Y, Z;
     unsigned int pos;
     char Base;
-
+ 
     ifstream readData;
     readData.open(DataFile, ios::in);
     if (!readData.is_open()) {
@@ -96,6 +229,7 @@ void Structure3D::ReadCoOrdinateFile() {
     }
 
     int cntr = 0;
+    unsigned int last3Dpos = 0; 
 
     static bool bEColiStartPosStored = false;
 
@@ -103,7 +237,7 @@ void Structure3D::ReadCoOrdinateFile() {
         readData.getline(buf,100);  
         string tmp, atom, line = string(buf);
 
-        if (line.find("ATOM") != string::npos ) {
+        if ((line.find("ATOM") != string::npos ) || (line.find("HETATM") != string::npos )) {
             atom = (line.substr(77,2)).c_str();
             if (atom.find("P") != string::npos) {
                 tmp = (line.substr(18,3)).c_str();
@@ -112,11 +246,16 @@ void Structure3D::ReadCoOrdinateFile() {
                 X    = atof((line.substr(31,8)).c_str());
                 Y    = atof((line.substr(39,8)).c_str());
                 Z    = atof((line.substr(47,8)).c_str());
-                StoreCoordinates(X,Y,Z,Base,pos);
-                
-                strCen->x += X; strCen->y += Y; strCen->z += Z;
-                cntr++;
-
+       
+                // special filter for 23S rRNA structure (IVOR/IPNU)
+                // IVOR/IPNU contains artifacts and are not mentioned in any of the 
+                // remarks of PDB file 
+                if (last3Dpos != pos && !(pos >= 3093)) {       
+                    StoreCoordinates(X,Y,Z,Base,pos);
+                    last3Dpos = pos;
+                    strCen->x += X; strCen->y += Y; strCen->z += Z;
+                    cntr++;
+                }
                 if(!bEColiStartPosStored) { 
                     iEColiStartPos = pos;
                     bEColiStartPosStored = true;
@@ -124,7 +263,13 @@ void Structure3D::ReadCoOrdinateFile() {
             }
         }
     }
-    iEColiStartPos = pos; 
+    iEColiEndPos = pos; 
+
+    cout<<"--------------------------------------------------"<<endl
+        <<globalComment<<endl
+        <<"Structure contains "<<iEColiEndPos-iEColiStartPos<<"( "<<iEColiStartPos<<" - "
+        <<iEColiEndPos<<" ) positions."<<endl
+        <<"---------------------------------------------------"<<endl;
 
     strCen->x = strCen->x/cntr; 
     strCen->y = strCen->y/cntr;
@@ -161,7 +306,21 @@ void Structure3D::Store2Dinfo(char *info, int pos, int helixNr){
 //=========== Reading Secondary Structure Data from Ecoli Secondary Structure Mask file ====================//
 
 void Structure3D::GetSecondaryStructureInfo(void) {
-    char *DataFile        = find_data_file("ECOLI_SECONDARY_STRUCTURE_INFO");
+    static char *DataFile = 0;  
+    int rnaType    = FindTypeOfRNA();
+
+     switch (rnaType) {
+     case LSU_23S: 
+         DataFile = find_data_file("SecondaryStructureModel_23SrRNA.data");
+         break;
+     case LSU_5S: 
+         DataFile = find_data_file("SecondaryStructureModel_5SrRNA.data");
+         break;
+     case SSU_16S:
+         DataFile = find_data_file("SecondaryStructureModel_16SrRNA.data");
+         break;
+     }
+
     char  buf[256];
 
     int pos, helixNr, lastHelixNr; lastHelixNr = 0;
@@ -194,7 +353,8 @@ void Structure3D::GetSecondaryStructureInfo(void) {
         if ((info[2] == 'S') || (info[2] == 'E')) {
             if (!insideHelix) insideHelix = true;
             else {
-                Store2Dinfo(info, pos, lastHelixNr); skip = true;
+                Store2Dinfo(info, pos, lastHelixNr); 
+                skip        = true;
                 insideHelix = false;
             }
         }
@@ -237,6 +397,9 @@ void Structure3D::Store2D3Dinfo(Struct2Dinfo *s2D, Struct3Dinfo *s3D) {
 void Structure3D::Combine2Dand3DstructureInfo(void) {
     Struct3Dinfo *temp3D;
     Struct2Dinfo *temp2D;
+    int cntr = 0;
+
+    cout<<"Missing Base Positions : "<<endl;
 
     temp3D = start3D;    
     temp2D = start2D;
@@ -244,9 +407,27 @@ void Structure3D::Combine2Dand3DstructureInfo(void) {
         if (temp2D->pos == temp3D->pos) {
             Store2D3Dinfo(temp2D, temp3D);
         }
+        else {
+            while (temp2D->pos != temp3D->pos) {
+                cout<<temp2D->pos<<", "; // missing base positions
+                cntr++;
+                temp2D = temp2D->next;
+            }
+            Store2D3Dinfo(temp2D, temp3D);
+        }
+        
         temp3D = temp3D->next;
         temp2D = temp2D->next;
     }
+    cout<<endl<<"Total No. of bases missing = "<<cntr<<endl;
+    
+    // printing comments in the main window
+    {
+        RNA3D->bDisplayComments = true;
+        char buf[256];
+        sprintf(buf, "Total No. of bases missing = %d. See the console messages for the actual missing base positions.",cntr);
+        strcat(globalComment,buf);
+    }
 }
 
 void Structure3D::PointsToQuads(float x, float y, float z) {
@@ -425,8 +606,25 @@ void Structure3D::GenerateSecStructureUnpairedHelixRegions(void) {
 
 void Structure3D::GenerateTertiaryInteractionsDispLists(){
     Struct2Dplus3D *t;
-    char           *DataFile = find_data_file("ECOLI_Tertiary_Interaction.data");
-    char            buf[256];
+    static char *DataFile = 0;  
+    int rnaType    = FindTypeOfRNA();
+
+    switch (rnaType) {
+    case LSU_23S: 
+        DataFile = find_data_file("TertiaryInteractions_23SrRNA.data");
+        break;
+    case LSU_5S: 
+        cout<<"There are no tertiary interactions observed in 5S rRNA model!"<<endl;
+        return;
+    case SSU_16S:
+        DataFile = find_data_file("TertiaryInteractions_16SrRNA.data");
+        break;
+    }
+    
+    cout<<"Tertiary Interactions are fetched from comparative RNA website [http://www.rna.icmb.utexas.edu]."<<endl
+        <<"The same are located in the file \""<<DataFile<<"\"."<<endl;
+
+    char  buf[256];
 
     ifstream readData;
     readData.open(DataFile, ios::in);
@@ -434,8 +632,8 @@ void Structure3D::GenerateTertiaryInteractionsDispLists(){
         throw_IO_error(DataFile);
     }
 
-    int K[50];
-    int R[50];
+    int K[100];
+    int R[100];
     int k, r; k = r = 0;
 
     while (!readData.eof()) {
@@ -444,14 +642,14 @@ void Structure3D::GenerateTertiaryInteractionsDispLists(){
         tmp = strtok(buf, " ");
         if (tmp != NULL) 
             {
-                if (tmp[0] == 'K') {
+                if (strcmp(tmp, "KNOT") == 0) {
                     tmp = strtok(NULL, ":" );
                     while (tmp != NULL) {
                         K[k++] = atoi(tmp); 
                         tmp = strtok(NULL, ":" );
                     }
                 }
-                else if (tmp[0] == 'R') {
+                else if (strcmp(tmp,"TRIPLE") == 0) {
                     tmp = strtok(NULL, ":" );
                     while (tmp != NULL) {
                         R[r++] = atoi(tmp);
@@ -529,25 +727,22 @@ void Structure3D::StoreHelixNrInfo(float x, float y, float z, int helixNr) {
 }
 
 void Structure3D::GenerateHelixDispLists(int HELIX_NR_ID, int HELIX_NR) {
-    Struct3Dinfo *temp3D;
     Struct2Dinfo *temp2D;
     Struct2Dplus3D *temp2D3D;
 
-    const int MAX = 200;
+    const int MAX = 500;
 
     int thisStrandPos[MAX], otherStrandPos[MAX];
     int i, j; i = j = 0;
     {
-        temp3D = start3D;    
         temp2D = start2D;
-        while ((temp3D != NULL) &&  (temp2D != NULL)) { 
+        while (temp2D != NULL) { 
             if (temp2D->helixNr == HELIX_NR) {
                 if ((temp2D->mask == '[') || (temp2D->mask == '<'))
                     thisStrandPos[i++]  = temp2D->pos;
                 if ((temp2D->mask == ']') || (temp2D->mask == '>'))
                     otherStrandPos[j++] = temp2D->pos;
             }
-            temp3D = temp3D->next;
             temp2D = temp2D->next;
         }
     }
@@ -555,6 +750,11 @@ void Structure3D::GenerateHelixDispLists(int HELIX_NR_ID, int HELIX_NR) {
     int tempPos = 0;
     float x1,x2,y1,y2,z1,z2; x1=x2=y1=y2=z1=z2=0.0;
 
+    bool bThisStrand, bOtherStrand;
+    bThisStrand = bOtherStrand = false;
+
+    bool bMissingBasePair = false;
+
     glNewList(HELIX_NR_ID, GL_COMPILE);
     {
         for (int k = 0, l = j-1; k < i && l >= 0; k++, l--)
@@ -564,7 +764,7 @@ void Structure3D::GenerateHelixDispLists(int HELIX_NR_ID, int HELIX_NR) {
                     temp2D3D = start2D3D;
                     while (temp2D3D != NULL) { 
                         if (temp2D3D->pos == tempPos) { 
-                            glVertex3f(temp2D3D->x, temp2D3D->y, temp2D3D->z);
+                            bThisStrand = true;
                             x1=temp2D3D->x; y1=temp2D3D->y; z1=temp2D3D->z;
                             break;
                         }
@@ -576,16 +776,30 @@ void Structure3D::GenerateHelixDispLists(int HELIX_NR_ID, int HELIX_NR) {
                     temp2D3D = start2D3D;
                     while (temp2D3D != NULL) { 
                         if (temp2D3D->pos == tempPos) { 
-                            glVertex3f(temp2D3D->x, temp2D3D->y, temp2D3D->z);
+                            bOtherStrand = true;
                             x2=temp2D3D->x; y2=temp2D3D->y; z2=temp2D3D->z;
                             break;
                         }
                         temp2D3D = temp2D3D->next;
                     }
                 }
-                x1 = (x1+x2)/2; y1 = (y1+y2)/2; z1 = (z1+z2)/2;
-                StoreHelixNrInfo(x1,y1,z1,HELIX_NR);
+                //if bases present in both the strands then draw a bond
+                //and store the helix number information
+                if (bThisStrand && bOtherStrand){
+                    glVertex3f(x1, y1, z1);
+                    glVertex3f(x2, y2, z2);
+                    
+                    x1 = (x1+x2)/2; y1 = (y1+y2)/2; z1 = (z1+z2)/2;
+                    StoreHelixNrInfo(x1,y1,z1,HELIX_NR);
+
+                    bThisStrand = bOtherStrand = false;
+                }
+                else {
+                    bMissingBasePair = true;
+                    cout<<thisStrandPos[k]<<"-"<<tempPos<<"("<<HELIX_NR_ID<<"), ";
+                }
             }
+        if (bMissingBasePair) cout<<endl;
     }
     glEndList();
 }
@@ -634,11 +848,37 @@ void Structure3D::GenerateDisplayLists(void){
 
     GenerateMoleculeSkeleton();
     ComputeBasePositions();
-
-    for (int i = 1; i <= 50;i++) {
-        GenerateHelixDispLists(i, i);
+    
+    int rnaType = FindTypeOfRNA();
+    switch (rnaType) {
+    case SSU_16S:
+        if (glIsList(HelixBase) != GL_TRUE) 
+            {
+                for (int i = 1; i <= 50;i++) {
+                    GenerateHelixDispLists(HelixBase+i, i);
+                }
+            }
+        break;
+    case LSU_23S:
+        cout<<"=========================================================="<<endl
+            <<"Missing base pairs (bracket number indicates helix number) "<<endl;
+        if (glIsList(HelixBase) != GL_TRUE) 
+            {
+                for (int i = 1; i <= 101;i++) {
+                    GenerateHelixDispLists(HelixBase+i, i);
+                }
+            }
+        cout<<"=========================================================="<<endl;
+        break;
+    case LSU_5S:
+        if (glIsList(HelixBase) != GL_TRUE) 
+            {
+                for (int i = 1; i <= 5;i++) {
+                    GenerateHelixDispLists(HelixBase+i, i);
+                }
+            }
+        break;
     }
-    //    GenerateHelixNrDispList(1,50);
 
     GenerateSecStructureHelixRegions();
     GenerateSecStructureNonHelixRegions();
@@ -719,6 +959,7 @@ void Structure3D::ComputeBasePositions(){
     char POS[50];
     float spacer = 1.5;
     int posSkip = iInterval;
+    if (posSkip <= 0) posSkip = 25; // default interval value
 
     glNewList(STRUCTURE_POS, GL_COMPILE);
     {
@@ -751,13 +992,27 @@ void Structure3D::ComputeBasePositions(){
 }
 
 void Structure3D::PrepareSecondaryStructureData(void) {
+//     const char 
+//         outFile[]      = "/nfshome/yadhu/DataBase/Struct3D/model16SrRNA/test.data",
+//         EcoliFile[]    = "/nfshome/yadhu/DataBase/Struct3D/model16SrRNA/ECOLI_GAPS",
+//         HelixNrFile[]  = "/nfshome/yadhu/DataBase/Struct3D/model16SrRNA/HELIX_NR",
+//         HelixGapFile[] = "/nfshome/yadhu/DataBase/Struct3D/model16SrRNA/HELIX_GAP",
+//         ErrorMsg[] = "\n *** Error Opening File : ";
+
     const char 
-        outFile[]      = "data/test.data",
-        EcoliFile[]    = "data/ECOLI_GAPS",
-        HelixNrFile[]  = "data/HELIX_NR",
-        HelixGapFile[] = "data/HELIX_GAP",
+        outFile[]      = "/home/Yadhu/DataBase/Struct3D/model23SrRNA/test.data",
+        EcoliFile[]    = "/home/Yadhu/DataBase/Struct3D/model23SrRNA/Ecoli_23SrRNA_with_gaps.seq",
+        HelixNrFile[]  = "/home/Yadhu/DataBase/Struct3D/model23SrRNA/HelixNumbers.msk",
+        HelixGapFile[] = "/home/Yadhu/DataBase/Struct3D/model23SrRNA/SecStructureMask_with_gaps.msk",
         ErrorMsg[] = "\n *** Error Opening File : ";
 
+//     const char 
+//         outFile[]      = "/home/Yadhu/DataBase/Struct3D/model5SrRNA/test.data",
+//         EcoliFile[]    = "/home/Yadhu/DataBase/Struct3D/model5SrRNA/Ecoli_5SrRNA_with_gaps.seq",
+//         HelixNrFile[]  = "/home/Yadhu/DataBase/Struct3D/model5SrRNA/HelixNumbers.msk",
+//         HelixGapFile[] = "/home/Yadhu/DataBase/Struct3D/model5SrRNA/SecStructureMask_with_gaps.msk",
+//         ErrorMsg[] = "\n *** Error Opening File : ";
+
     int fileLen = 0;
     char *helixNrBuf, *helixGapBuf, *ecoliBuf;
 
@@ -1008,6 +1263,13 @@ void Structure3D::GenerateBaseDifferenceDisplayList(){
         BuildDisplayList(MAP_SPECIES_DELETION, deletion, d);
         BuildDisplayList(MAP_SPECIES_MISSING, miss, m);
         GenerateBaseDifferencePositionDisplayList();
+
+        iTotalSubs = a+g+c+u;  // Summing up the substitutions/mutations
+        iTotalDels = d;        // Storing total number of deletions
+#ifdef DEBUG
+        cout<<"Substitutions = "<<iTotalSubs<<endl;
+        cout<<"Deletions     = "<<iTotalDels<<endl;
+#endif
     }
 }
 
@@ -1049,6 +1311,7 @@ void Structure3D::GenerateInsertionDisplayList(){
     char inserts[500];
     int i, cntr;
     float spacer = 2.0;
+    iTotalIns = 0;
 
     glNewList(MAP_SPECIES_INSERTION_BASES, GL_COMPILE);
     {   
@@ -1068,11 +1331,17 @@ void Structure3D::GenerateInsertionDisplayList(){
                 sprintf(buffer, "%d:%s", cntr, inserts);
                 GRAPHICS->PrintString(str->x, str->y+spacer, str->z, 
                                       buffer, GLUT_BITMAP_8_BY_13);
+
+                iTotalIns += cntr; // Summing up the insertions
             }
         }
     }
     glEndList();
 
+#ifdef DEBUG
+    cout<<"Insertions = "<<iTotalIns<<endl;
+#endif
+
     glNewList(MAP_SPECIES_INSERTION_BASES_ANCHOR, GL_COMPILE);
     {   
         glLineWidth(1.0);
@@ -1152,7 +1421,6 @@ void Structure3D::MapCurrentSpeciesToEcoliTemplate(AW_root *awr){
 
                     GBDATA *gbSeqData    = GBT_read_sequence(gbSpecies, ali_name);
                     const char *pSeqData = GB_read_char_pntr(gbSeqData); 
-                    int iSeqCount = 0;
 
                     if (pSeqData && pTemplateSeqData) {
                         int iSeqLen = strlen(pTemplateSeqData); 
@@ -1161,7 +1429,7 @@ void Structure3D::MapCurrentSpeciesToEcoliTemplate(AW_root *awr){
                             DeleteOldSpeciesData();
                         }
 
-                        for(int i = 0; i<iSeqLen; i++) {
+                        for(int i = 0, iSeqCount = 0; i<iSeqLen; i++) {
                             if((pTemplateSeqData[i] != '.') && (pTemplateSeqData[i] != '-'))
                             { 
                                 if (!bStartPosStored) {
@@ -1216,34 +1484,27 @@ void Structure3D::MapCurrentSpeciesToEcoliTemplate(AW_root *awr){
     GB_pop_transaction(gb_main);
 }
 
-int Structure3D::ValidateSearchColor(int iColor, int mode){
+static bool ValidSearchColor(int iColor, int mode) {
+    bool isValid = false;
 
-    switch (mode) 
-        {
-        case SAI:
-            if ((iColor >= RNA3D_GC_CBACK_0) && (iColor < RNA3D_GC_MAX))  return  1;
-            else                                                          return  0;
-            break;
-        case SEARCH:
-            if ((iColor >= RNA3D_GC_SBACK_0) && (iColor < RNA3D_GC_MAX))  return  1;
-            else                                                          return  0;
-            break;
-        }
+    switch (mode) {
+        case SAI:    isValid = (iColor >= RNA3D_GC_CBACK_0) && (iColor < RNA3D_GC_MAX); break;
+        case SEARCH: isValid = (iColor >= RNA3D_GC_SBACK_0) && (iColor < RNA3D_GC_MAX); break;
+    }
+    return isValid;
 }
 
-void Structure3D::MapSearchStringsToEcoliTemplate(AW_root *awr){
-    AWUSE(awr);
+void Structure3D::MapSearchStringsToEcoliTemplate(AW_root * /*awr*/){
     if (ED4_SeqTerminal) { 
         const char *pSearchColResults = 0;
 
         if (iMapSearch) {
             // buildColorString builds the background color of each base
-            pSearchColResults = ED4_SeqTerminal->results().buildColorString(ED4_SeqTerminal, iStartPos, iEndPos); 
+            pSearchColResults = ED4_buildColorString(ED4_SeqTerminal, iStartPos, iEndPos); 
         }
         
         if(pSearchColResults) {
             int iColor = 0;
-            // extern OpenGLGraphics *cGraphics;
 
             glNewList(MAP_SEARCH_STRINGS_TO_STRUCTURE, GL_COMPILE);
             {
@@ -1252,17 +1513,16 @@ void Structure3D::MapSearchStringsToEcoliTemplate(AW_root *awr){
                 }
                 for (int i = iStartPos; i < iEndPos; i++) {
                     if(RNA3D->bEColiRefInitialised) {
-                        long absPos = (long) i;
-                        long EColiPos, dummy;
-                        EColiRef->abs_2_rel(absPos, EColiPos, dummy);
+                        long absPos   = (long) i;
+                        long EColiPos = EColiRef->abs_2_rel(absPos);
 
-                        for(Struct3Dinfo *t = start3D; t != NULL; t = t->next) 
-                            {
-                                if ((t->pos == EColiPos) && (pSearchColResults[i] >= 0)) 
+                        for(Struct3Dinfo *t = start3D; t != NULL; t = t->next)
+                        {
+                            if ((t->pos == EColiPos) && (pSearchColResults[i] >= 0))
                                     {
                                         iColor = pSearchColResults[i] - COLORLINK;
 
-                                        if(ValidateSearchColor(iColor, SEARCH)) {
+                                        if(ValidSearchColor(iColor, SEARCH)) {
                                             RNA3D->cGraphics->SetColor(iColor);
                                             PointsToQuads(t->x, t->y, t->z);
                                         }
@@ -1283,17 +1543,16 @@ void Structure3D::MapSearchStringsToEcoliTemplate(AW_root *awr){
                 glBegin(GL_LINES);
                 for (int i = iStartPos; i < iEndPos; i++) {
                     if(RNA3D->bEColiRefInitialised) {
-                        long absPos = (long) i;
-                        long EColiPos, dummy;
-                        EColiRef->abs_2_rel(absPos, EColiPos, dummy);
+                        long absPos   = (long) i;
+                        long EColiPos = EColiRef->abs_2_rel(absPos);
 
-                        for(Struct3Dinfo *t = start3D; t != NULL; t = t->next) 
-                            {
-                                if ((t->pos == EColiPos) && (pSearchColResults[i] >= 0)) 
+                        for(Struct3Dinfo *t = start3D; t != NULL; t = t->next)
+                        {
+                            if ((t->pos == EColiPos) && (pSearchColResults[i] >= 0))
                                     {
                                         iColor = pSearchColResults[i] - COLORLINK;
 
-                                        if(ValidateSearchColor(iColor, SEARCH)) {
+                                        if(ValidSearchColor(iColor, SEARCH)) {
                                             if ((iLastClr == iColor) && (iLastPos == EColiPos-1)) {
                                                 RNA3D->cGraphics->SetColor(iColor);
                                                 glVertex3f(vLastPt.x, vLastPt.y, vLastPt.z);
@@ -1323,14 +1582,12 @@ void Structure3D::MapSaiToEcoliTemplate(AW_root *awr){
     if (ED4_SeqTerminal) { 
         const char *pSearchColResults = 0;
 
-        if (iMapSAI && ED4_ROOT->visualizeSAI) {
-            // returns 0 if sth went wrong
-            pSearchColResults = getSaiColorString(awr, iStartPos, iEndPos);
+        if (iMapSAI && ED4_SAIs_visualized()) {
+            pSearchColResults = ED4_getSaiColorString(awr, iStartPos, iEndPos); // returns 0 if sth went wrong
         }
 
         if(pSearchColResults) {
             int iColor = 0;
-            // extern OpenGLGraphics *cGraphics;
 
             glNewList(MAP_SAI_TO_STRUCTURE, GL_COMPILE);
             {
@@ -1339,17 +1596,16 @@ void Structure3D::MapSaiToEcoliTemplate(AW_root *awr){
                 }
                 for (int i = iStartPos; i < iEndPos; i++) {
                     if(RNA3D->bEColiRefInitialised) {
-                        long absPos = (long) i;
-                        long EColiPos, dummy;
-                        EColiRef->abs_2_rel(absPos, EColiPos, dummy);
+                        long absPos   = (long) i;
+                        long EColiPos = EColiRef->abs_2_rel(absPos);
 
-                        for(Struct3Dinfo *t = start3D; t != NULL; t = t->next) 
-                            {
-                                if ((t->pos == EColiPos) && (pSearchColResults[i] >= 0)) 
+                        for(Struct3Dinfo *t = start3D; t != NULL; t = t->next)
+                        {
+                            if ((t->pos == EColiPos) && (pSearchColResults[i] >= 0))
                                     {
-                                        iColor = pSearchColResults[i] - SAICOLORS;
+                                        iColor = pSearchColResults[i-1] - SAICOLORS;
 
-                                        if(ValidateSearchColor(iColor, SAI)) {
+                                        if(ValidSearchColor(iColor, SAI)) {
                                             RNA3D->cGraphics->SetColor(iColor);
                                             PointsToQuads(t->x, t->y, t->z);
                                         }
@@ -1366,7 +1622,7 @@ void Structure3D::MapSaiToEcoliTemplate(AW_root *awr){
 
             RNA3D->bMapSaiDispListCreated = true;
         }
-        else cout<<"getSaiColorString did not get the colors : SAI cannot be Visualized!"<<endl;                                                     
+        else cout<<"ED4_getSaiColorString did not get the colors : SAI cannot be Visualized!"<<endl;                                                     
     }
     else cout<<"Problem with initialization : SAI cannot be Visualized!"<<endl;  
 }
