@@ -83,7 +83,7 @@ static void awt_edit_changed_cb(GBDATA *dummy, struct adawcbstruct *cbs, GB_CB_T
         cbs->gb_edit = 0;
     }
     if (cbs->gb_edit) {
-        if (awt_check_scanner_key_data(cbs,cbs->gb_edit)) {		// dont exist
+        if (awt_check_scanner_key_data(cbs,cbs->gb_edit)) {		// doesnt exist
             aws->get_root()->awar(cbs->def_dest)->write_string("");
         }else{
             char *data;
@@ -100,9 +100,8 @@ static void awt_edit_changed_cb(GBDATA *dummy, struct adawcbstruct *cbs, GB_CB_T
 }
 
 
-static void awt_arbdb_scanner_value_change(void *dummy, struct adawcbstruct *cbs)
+static void awt_arbdb_scanner_value_change(void *, struct adawcbstruct *cbs)
 {
-    AWUSE(dummy);
     char *value = cbs->aws->get_root()->awar(cbs->def_dest)->read_string();
     int   vlen  = strlen(value);
     
@@ -154,7 +153,7 @@ static void awt_arbdb_scanner_value_change(void *dummy, struct adawcbstruct *cbs
                         if (strlen(value)) {
                             GBT_begin_rename_session(cbs->gb_main,0);
                             
-                            error = GBT_rename_species(name, value);
+                            error = GBT_rename_species(name, value, GB_FALSE);
 
                             if (error) GBT_abort_rename_session();
                             else GBT_commit_rename_session(aw_status, aw_status);
@@ -172,8 +171,6 @@ static void awt_arbdb_scanner_value_change(void *dummy, struct adawcbstruct *cbs
                 }
                 else { // non-species (gene, experiment, etc.)
                     if (strlen(value)) {
-                        
-                        
                         GBDATA *gb_exists    = 0;
                         GBDATA *gb_item_data = GB_get_father(cbs->gb_user);
 
@@ -229,7 +226,7 @@ static void awt_arbdb_scanner_value_change(void *dummy, struct adawcbstruct *cbs
     }
 
     if (update_self) { // if the name changed -> rewrite awars AFTER transaction was closed
-        GB_transaction dummy(cbs->gb_main);
+        GB_transaction ta(cbs->gb_main);
         
         char *my_id = cbs->selector->generate_item_id(cbs->gb_main, cbs->gb_user);
         cbs->selector->update_item_awars(cbs->gb_main, cbs->awr, my_id); // update awars (e.g. AWAR_SPECIES_NAME)
