@@ -5,9 +5,10 @@
 #include <arbdbt.h>
 #include <cat_tree.hxx>
 
-void error_msg(char **argv){
+static void error_msg(char **argv) __attribute__((noreturn));
+static void error_msg(char **argv){
     printf("syntax: %s treefile otb-file\n"
-	   "	syntax of tree tips:	name#full_name\n"
+           "	syntax of tree tips:	name#full_name\n"
 	   ,argv[0]);
     exit(-1);
 }
@@ -22,7 +23,7 @@ int main(int argc,char **argv)
 	char *otb_name = argv[2];
 
 
-	GBT_TREE *tree = GBT_load_tree(nname,sizeof(GBT_TREE), 0, 1);
+	GBT_TREE *tree = GBT_load_tree(nname,sizeof(GBT_TREE), 0, 1, 0);
 	if (!tree) {
 	    GB_print_error();
 	    return -1;
