@@ -549,7 +549,11 @@ static int get_available_fontsizes(Display *tool_d, int f, int *available_sizes)
     for (int size = MAX_FONTSIZE; size >= MIN_FONTSIZE; --size) {
         int      found_size;
         PIX_FONT fontst;
-        bool     was_found = lookfont(tool_d, f, size, found_size, false, true, &fontst);
+        
+#if defined(DEBUG)
+        bool     was_found = 
+#endif // DEBUG
+            lookfont(tool_d, f, size, found_size, false, true, &fontst);
 
         aw_assert(was_found); // because lookfont does fallback
 
@@ -727,7 +731,10 @@ void AW_GC_Xm::set_font(AW_font font_nr, int size, int *found_size)
 
     {
         int  found_font_size;
-        bool was_found = lookfont(common->display, font_nr, size, found_font_size, true, false, &xfs);
+#if defined(DEBUG)
+        bool was_found =
+#endif // DEBUG
+            lookfont(common->display, font_nr, size, found_font_size, true, false, &xfs);
         aw_assert(was_found); // because lookfont does fallback
 
         if (found_size) *found_size = found_font_size;
@@ -750,8 +757,9 @@ void AW_GC_Xm::set_font(AW_font font_nr, int size, int *found_size)
     fontinfo.max_all_letter.reset();
 
     aw_assert(AW_FONTINFO_CHAR_MIN < AW_FONTINFO_CHAR_MAX);
+    aw_assert(AW_FONTINFO_CHAR_ASCII_MIN < AW_FONTINFO_CHAR_ASCII_MAX);
+    aw_assert(AW_FONTINFO_CHAR_MIN <= AW_FONTINFO_CHAR_ASCII_MIN);
     aw_assert(AW_FONTINFO_CHAR_ASCII_MAX <= AW_FONTINFO_CHAR_MAX);
-    aw_assert(AW_FONTINFO_CHAR_MIN < AW_FONTINFO_CHAR_ASCII_MAX);
 
     unsigned int i;
     for (i = AW_FONTINFO_CHAR_MIN; i <= AW_FONTINFO_CHAR_MAX; i++) {
@@ -766,7 +774,7 @@ void AW_GC_Xm::set_font(AW_font font_nr, int size, int *found_size)
             descent_of_chars[i] = cs->descent;
             width_of_chars[i]   = cs->width;
 
-            if (i <= AW_FONTINFO_CHAR_ASCII_MAX) {
+            if (i >= AW_FONTINFO_CHAR_ASCII_MIN && i <= AW_FONTINFO_CHAR_ASCII_MAX) {
                 fontinfo.max_letter.notify_all(cs->ascent, cs->descent, cs->width);
             }
             fontinfo.max_all_letter.notify_all(cs->ascent, cs->descent, cs->width);
